/*****************************************************************************
 *  Rohde&Schwarz RF Signal Generator SMx instrument driver
 *
 *  VXIpnp Instrument Driver
 *****************************************************************************/

#ifndef __RSSMX_HEADER
#define __RSSMX_HEADER

#include "rscore.h"
#include "rssmx_attributes.h"
#include "rssmx_utility.h"
#include "rssmx_internal.h"

#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#if defined(__cplusplus) || defined(__cplusplus__)
extern "C" {
#endif

/*****************************************************************************
 *- Useful Macros & Constants -----------------------------------------------*
 *****************************************************************************/

#define RSSMX_SIMULATION_ID_QUERY      "Rohde&Schwarz,SMA100B,123456/789,1.23"  /* Default definition of ID Query Response for simulation */
#define RSSMX_VALID_ID_RESPONSE_STRING "Rohde&Schwarz,SM"      /* Valid response for identification query - all R&S SMxx generators */
#define RSSMX_SIMULATION_OPT_QUERY     ""      /* Simulated response for *OPT? command */

#define RSSMX_OPC_TIMEOUT              5000L   /* Maximum time to wait for OPC in milliseconds */
#define MAX_OPC_TIME                   60000L

/****************************************************************************
 *----------------- Instrument Driver Revision Information -----------------*
 ****************************************************************************/

/* Class Driver Identification */

#define RSSMX_CLASS_DRIVER_DESCRIPTION           ""
#define RSSMX_CLASS_DRIVER_PREFIX                ""
#define RSSMX_CLASS_DRIVER_VENDOR                ""
#define RSSMX_CLASS_DRIVER_REVISION              ""
#define RSSMX_CLASS_SPEC_MAJOR_VERSION           1L    /* Class specification major version */
#define RSSMX_CLASS_SPEC_MINOR_VERSION           6L    /* Class specification minor version */

/* Driver Identification */

#define RSSMX_SPECIFIC_DRIVER_DESCRIPTION        ""
#define RSSMX_SPECIFIC_DRIVER_PREFIX             "RSSMX"
#define RSSMX_SPECIFIC_DRIVER_LOCATOR            ""
#define RSSMX_SPECIFIC_DRIVER_VENDOR             ""
#define RSSMX_MAJOR_VERSION                      5L   /* Instrument driver major version          */
#define RSSMX_MINOR_VERSION                      0L  /* Instrument driver minor version          */
#define RSSMX_MINOR_MINOR_VERSION                0L   /* Instrument driver minor minor version    */

/* Driver Capabilities */

#define RSSMX_SUPPORTED_INSTRUMENT_MODELS        "SMA,SMB,SMC,SMF"    /* Instrument driver supported model(s)     */
#define RSSMX_GROUP_CAPABILITIES                 ""
#define RSSMX_FUNCTION_CAPABILITIES              ""

/****************************************************************************
 *---------------------------- Attribute Defines ---------------------------*
 ****************************************************************************/

    /*- Inherent Instrument Specific Attributes ----------------------------*/

        /* User Options */

#define RSCMW_ATTR_RANGE_CHECK                                           RS_ATTR_RANGE_CHECK                                     /* ViBoolean */
#define RSCMW_ATTR_QUERY_INSTRUMENT_STATUS                               RS_ATTR_QUERY_INSTRUMENT_STATUS                         /* ViBoolean */
#define RSCMW_ATTR_SIMULATE                                              RS_ATTR_SIMULATE                                        /* ViBoolean */

        /* Driver Identification */

#define RSCMW_ATTR_SPECIFIC_DRIVER_DESCRIPTION                           RS_ATTR_SPECIFIC_DRIVER_DESCRIPTION                     /* ViString, read-only */
#define RSCMW_ATTR_SPECIFIC_DRIVER_PREFIX                                RS_ATTR_SPECIFIC_DRIVER_PREFIX                          /* ViString, read-only */
#define RSCMW_ATTR_SPECIFIC_DRIVER_LOCATOR                               RS_ATTR_SPECIFIC_DRIVER_LOCATOR                         /* ViString, read-only */
#define RSCMW_ATTR_SPECIFIC_DRIVER_VENDOR                                RS_ATTR_SPECIFIC_DRIVER_VENDOR                          /* ViString, read-only */
#define RSCMW_ATTR_SPECIFIC_DRIVER_REVISION                              RS_ATTR_SPECIFIC_DRIVER_REVISION                        /* ViString, read-only */
#define RSCMW_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MAJOR_VERSION              RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MAJOR_VERSION        /* ViInt32,  read-only */
#define RSCMW_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MINOR_VERSION              RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MINOR_VERSION        /* ViInt32,  read-only */

        /* Driver Capabilities */

#define RSCMW_ATTR_SUPPORTED_INSTRUMENT_MODELS                           RS_ATTR_SUPPORTED_INSTRUMENT_MODELS                     /* ViString, read-only */
#define RSCMW_ATTR_GROUP_CAPABILITIES                                    RS_ATTR_GROUP_CAPABILITIES                              /* ViString, read-only */
#define RSCMW_ATTR_FUNCTION_CAPABILITIES                                 RS_ATTR_FUNCTION_CAPABILITIES                           /* ViString, read-only */
#define RSCMW_ATTR_CHANNEL_COUNT                                         RS_ATTR_CHANNEL_COUNT                                   /* ViInt32,  read-only */

        /* Driver Setup */

#define RSCMW_ATTR_DRIVER_SETUP                                          RS_ATTR_DRIVER_SETUP                                    /* ViString            */

        /* Instrument Identification */

#define RSCMW_ATTR_INSTRUMENT_MANUFACTURER                               RS_ATTR_INSTRUMENT_MANUFACTURER                         /* ViString, read-only */
#define RSCMW_ATTR_INSTRUMENT_MODEL                                      RS_ATTR_INSTRUMENT_MODEL                                /* ViString, read-only */
#define RSCMW_ATTR_INSTRUMENT_FIRMWARE_REVISION                          RS_ATTR_INSTRUMENT_FIRMWARE_REVISION                    /* ViString, read-only */

        /* Advanced Session Information */

#define RSCMW_ATTR_IO_RESOURCE_DESCRIPTOR                                RS_ATTR_IO_RESOURCE_DESCRIPTOR                          /* ViString, read-only */
#define RSCMW_ATTR_LOGICAL_NAME                                          RS_ATTR_LOGICAL_NAME                                    /* ViString, read-only */

    /*- Instrument-Specific Attributes -------------------------------------*/

/* BEGIN GENERATE */
#define RSSMX_ATTR_ALL_RF_OUTPUTS_ENABLED                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 355974
#define RSSMX_ATTR_OUTPUT_ENABLED                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 355975
#define RSSMX_ATTR_OUTPUT_IMPEDANCE                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 355976
#define RSSMX_ATTR_POWER_ON_ENABLED                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 355977
#define RSSMX_ATTR_RF_LEVEL_ON_RECALL                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 355978
#define RSSMX_ATTR_RF_FREQUENCY                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 355979
#define RSSMX_ATTR_RF_FREQUENCY_MODE                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 356095
#define RSSMX_ATTR_RF_FREQUENCY_ON_RECALL                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 355980
#define RSSMX_ATTR_RF_FREQUENCY_MAIN_PLL_BANDWIDTH                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 355981
#define RSSMX_ATTR_RF_FREQUENCY_OFFSET                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 355982
#define RSSMX_ATTR_RF_FREQUENCY_MULTIPLIER                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 355983
#define RSSMX_ATTR_RF_FREQUENCY_VARIATION_ACTIVE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 355984
#define RSSMX_ATTR_RF_FREQUENCY_VARIATION_STEP                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 355985
#define RSSMX_ATTR_RF_FREQUENCY_PHASE_CONTINUITY_ENABLED                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 356430
#define RSSMX_ATTR_RF_FREQUENCY_PHASE_CONTINUITY_MODE                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 356431
#define RSSMX_ATTR_RF_FREQUENCY_PHASE_CONTINUITY_MAX                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 356432
#define RSSMX_ATTR_RF_FREQUENCY_PHASE_CONTINUITY_MIN                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 356433
#define RSSMX_ATTR_RF_AMPLITUDE                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 355986
#define RSSMX_ATTR_RF_POWER_MODE                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 355987
#define RSSMX_ATTR_RF_LEVEL_LIMIT                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 355988
#define RSSMX_ATTR_RF_LEVEL_OFFSET                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 355989
#define RSSMX_ATTR_RF_LEVEL_SETTING_CHARACTERISTICS                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 355990
#define RSSMX_ATTR_RF_LEVEL_POWER                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 355991
#define RSSMX_ATTR_RF_LEVEL_MODE                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 355992
#define RSSMX_ATTR_RF_LEVEL_HARMONIC_FILTER                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 355993
#define RSSMX_ATTR_RF_LEVEL_LOWER_RANGE                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 355994
#define RSSMX_ATTR_RF_LEVEL_UPPER_RANGE                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 355995
#define RSSMX_ATTR_RF_LEVEL_RANGE_MIN                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 356756
#define RSSMX_ATTR_RF_LEVEL_RANGE_MAX                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 356757
#define RSSMX_ATTR_RF_LEVEL_IGNORE_WARNINGS                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 355996
#define RSSMX_ATTR_RF_LEVEL_VARIATION_ACTIVE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 355997
#define RSSMX_ATTR_RF_LEVEL_VARIATION_STEP                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 355998
#define RSSMX_ATTR_RF_LEVEL_DISPLAY_AS_VOLTAGE_OF_EMF                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 355999
#define RSSMX_ATTR_RF_ATTENUATOR_MODE                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 356000
#define RSSMX_ATTR_RF_ATTENUATOR_LOWER_RANGE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 356001
#define RSSMX_ATTR_RF_ATTENUATOR_UPPER_RANGE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 356002
#define RSSMX_ATTR_RF_ATTENUATOR_RF_OFF_MODE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 356003
#define RSSMX_ATTR_OVERVOLTAGE_PROTECTION_CLEAR                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 356004
#define RSSMX_ATTR_OVERVOLTAGE_PROTECTION_TRIPPED                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 356005
#define RSSMX_ATTR_ALC_ENABLED                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 356006
#define RSSMX_ATTR_ALC_MODE                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 356758
#define RSSMX_ATTR_ALC_READJUST                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 356007
#define RSSMX_ATTR_ALC_DETECTOR_SENSITIVITY                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 356008
#define RSSMX_ATTR_ALC_RECALCULATE                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 356009
#define RSSMX_ATTR_ALC_LEVEL_CONTROL_MODE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 356010
#define RSSMX_ATTR_ALC_EXTERNAL_DETECTOR_COUPLING_FACTOR                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 356791
#define RSSMX_ATTR_ALC_EXTERNAL_DETECTOR_REQUIRED_GENERATOR_LEVEL                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 356792
#define RSSMX_ATTR_RF_DELTA_PHASE                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 356011
#define RSSMX_ATTR_RESET_RF_DELTA_PHASE_DISPLAY                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 356012
#define RSSMX_ATTR_ALL_MODULATIONS_ENABLED                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 356013
#define RSSMX_ATTR_AM_MODE                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 356434
#define RSSMX_ATTR_AM_TYPE                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 356435
#define RSSMX_ATTR_AM_EXTERNAL_COUPLING                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 356014
#define RSSMX_ATTR_AM_SENSITIVITY                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 356015
#define RSSMX_ATTR_AM_DEVIATION_RATIO                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 356016
#define RSSMX_ATTR_AM_DEVIATION_MODE                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 356436
#define RSSMX_ATTR_AM_DEPTH_TOTAL                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 356437
#define RSSMX_ATTR_AM_ENABLED                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356017
#define RSSMX_ATTR_AM_SOURCE                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 356018
#define RSSMX_ATTR_AM_DEPTH                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 356019
#define RSSMX_ATTR_AM_DEPTH_LINEAR                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 356438
#define RSSMX_ATTR_AM_DEPTH_EXPONENTIAL                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 356439
#define RSSMX_ATTR_AM_SENSITIVITY_EXPONENTIAL                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356440
#define RSSMX_ATTR_AM_SENSITIVITY_LINEAR                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 356441
#define RSSMX_ATTR_FM_MODE                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 356020
#define RSSMX_ATTR_FM_DEVIATION_MODE                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 356442
#define RSSMX_ATTR_FM_DEVIATION_TOTAL                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 356443
#define RSSMX_ATTR_FM_SENSITIVITY                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 356021
#define RSSMX_ATTR_FM_RATIO                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 356022
#define RSSMX_ATTR_FM_ENABLED                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356023
#define RSSMX_ATTR_FM_SOURCE                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 356024
#define RSSMX_ATTR_FM_DEVIATION                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 356025
#define RSSMX_ATTR_PM_MODE                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 356026
#define RSSMX_ATTR_PM_SENSITIVITY                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 356027
#define RSSMX_ATTR_PM_RATIO                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 356028
#define RSSMX_ATTR_PM_DEVIATION_MODE                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 356444
#define RSSMX_ATTR_PM_DEVIATION_TOTAL                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 356445
#define RSSMX_ATTR_PM_ENABLED                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356029
#define RSSMX_ATTR_PM_SOURCE                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 356030
#define RSSMX_ATTR_PM_DEVIATION                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 356031
#define RSSMX_ATTR_PULM_ENABLED                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 356032
#define RSSMX_ATTR_PULM_SOURCE                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 356033
#define RSSMX_ATTR_PULM_TRANSITION_TYPE                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 356035
#define RSSMX_ATTR_PULM_VIDEO_POLARITY                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 356036
#define RSSMX_ATTR_PULM_POLARITY                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 356042
#define RSSMX_ATTR_PULM_IMPEDANCE                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 356043
#define RSSMX_ATTR_PULM_THRESHOLD                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 356046
#define RSSMX_ATTR_PGEN_ENABLED                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 356047
#define RSSMX_ATTR_PGEN_MODE                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 356034
#define RSSMX_ATTR_PGEN_MODE_DOUBLE                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 356429
#define RSSMX_ATTR_PGEN_TRIGGER_MODE                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 356044
#define RSSMX_ATTR_TRIGGER_PULSE_GENERATOR                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 356446
#define RSSMX_ATTR_PGEN_PERIOD                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 356037
#define RSSMX_ATTR_PGEN_WIDTH                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356040
#define RSSMX_ATTR_PGEN_DOUBLE_DELAY                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 356039
#define RSSMX_ATTR_PGEN_DOUBLE_WIDTH                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 356041
#define RSSMX_ATTR_PGEN_DELAY                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356038
#define RSSMX_ATTR_PGEN_OUTPUT                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 356048
#define RSSMX_ATTR_PGEN_OUTPUT_POLARITY                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 356049
#define RSSMX_ATTR_PULM_TRAIN_DELETE                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 356051
#define RSSMX_ATTR_PULM_TRAIN_SELECT                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 356053
#define RSSMX_ATTR_PULM_TRAIN_OFF_TIME_POINTS                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356055
#define RSSMX_ATTR_PULM_TRAIN_ON_TIME_POINTS                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 356057
#define RSSMX_ATTR_PULM_TRAIN_REPETITION_POINTS                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 356059
#define RSSMX_ATTR_PULSE_TRAIN_DATA_EXCHANGE_IMPORT_EXPORT                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 356061
#define RSSMX_ATTR_PULSE_TRAIN_DATA_EXCHANGE_FILE                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 356062
#define RSSMX_ATTR_PULSE_TRAIN_DATA_EXCHANGE_ASCII_FILE                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 356063
#define RSSMX_ATTR_PULSE_TRAIN_DATA_EXCHANGE_ASCII_EXTENSION                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 356064
#define RSSMX_ATTR_PULSE_TRAIN_DATA_EXCHANGE_ASCII_COLUMN_SEPARATOR               RS_SPECIFIC_PUBLIC_ATTR_BASE + 356065
#define RSSMX_ATTR_PULSE_TRAIN_DATA_EXCHANGE_ASCII_DECIMAL_SEPARATOR              RS_SPECIFIC_PUBLIC_ATTR_BASE + 356066
#define RSSMX_ATTR_PULSE_TRAIN_DATA_EXCHANGE_EXECUTE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 356067
#define RSSMX_ATTR_LF_OUTPUT_ENABLED                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 356068
#define RSSMX_ATTR_LF_OUTPUT_SOURCE                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 356069
#define RSSMX_ATTR_LF_OUTPUT_SOURCE_PATH                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 356070
#define RSSMX_ATTR_LF_OUTPUT_VOLTAGE                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 356071
#define RSSMX_ATTR_LF_OUTPUT_DC_OFFSET                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 356074
#define RSSMX_ATTR_LF_OUTPUT_EXTERNAL_LF_BANDWIDTH                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 356075
#define RSSMX_ATTR_LF_GENERATOR_INTERNAL_VOLTAGE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 356072
#define RSSMX_ATTR_LF_GENERATOR_FREQUENCY                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 356073
#define RSSMX_ATTR_LF_GENERATOR_REPETITION_FREQUENCY                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 356076
#define RSSMX_ATTR_LF_GENERATOR_SHAPE                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 356077
#define RSSMX_ATTR_LF_GENERATOR_PULSE_PERIOD                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 356078
#define RSSMX_ATTR_LF_GENERATOR_PULSE_WIDTH                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 356079
#define RSSMX_ATTR_LF_GENERATOR_PULSE_DUTY_CYCLE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 356080
#define RSSMX_ATTR_LF_GENERATOR_TRAPEZOID_PERIOD                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 356081
#define RSSMX_ATTR_LF_GENERATOR_TRAPEZOID_RISE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 356082
#define RSSMX_ATTR_LF_GENERATOR_TRAPEZOID_FALL                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 356083
#define RSSMX_ATTR_LF_GENERATOR_TRAPEZOID_HIGH                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 356084
#define RSSMX_ATTR_LF_GENERATOR_TRIANGLE_PERIOD                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 356085
#define RSSMX_ATTR_LF_GENERATOR_TRIANGLE_RISE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356086
#define RSSMX_ATTR_EXTERNAL_INPUT_COUPLING_MODE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 356092
#define RSSMX_ATTR_EXTERNAL_INPUT_IMPEDANCE                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 356093
#define RSSMX_ATTR_NOISE_DISTRIBUTION                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 356087
#define RSSMX_ATTR_NOISE_BANDWIDTH                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 356088
#define RSSMX_ATTR_NOISE_BANDWIDTH_STATE                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 356089
#define RSSMX_ATTR_RF_NOISE_DENSITY                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 356090
#define RSSMX_ATTR_NOISE_LEVEL                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 356091
#define RSSMX_ATTR_CHIRP_MODULATION_STATE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 356759
#define RSSMX_ATTR_CHIRP_MODULATION_BANDWIDTH                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356760
#define RSSMX_ATTR_CHIRP_MODULATION_COMPRESSION_RATIO                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 356761
#define RSSMX_ATTR_CHIRP_MODULATION_DIRECTION                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356762
#define RSSMX_ATTR_CHIRP_MODULATION_PULSE_PERIOD                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 356763
#define RSSMX_ATTR_CHIRP_MODULATION_PULSE_WIDTH                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 356764
#define RSSMX_ATTR_CHIRP_MODULATION_TRIGGER_MODE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 356765
#define RSSMX_ATTR_CHIRP_MODULATION_EXECUTE_SINGLE_TRIGGER                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 356766
#define RSSMX_ATTR_SWEEP_TYPE                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356447
#define RSSMX_ATTR_LF_MODE                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 356096
#define RSSMX_ATTR_TRIGGER_SOURCE_FOR_ALL_SWEEPS                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 356098
#define RSSMX_ATTR_TRIGGER_INPUT_SLOPE                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 356099
#define RSSMX_ATTR_RESET_ALL_SWEEPS                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 356100
#define RSSMX_ATTR_TRIGGER_ALL_SWEEPS                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 356101
#define RSSMX_ATTR_RF_FREQUENCY_SWEEP_STATUS                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 356103
#define RSSMX_ATTR_RF_FREQUENCY_SWEEP_MODE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 356104
#define RSSMX_ATTR_FREQUENCY_SWEEP_TRIGGER_SOURCE                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 356105
#define RSSMX_ATTR_RF_FREQUENCY_SWEEP_RETRACE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356106
#define RSSMX_ATTR_RF_FREQUENCY_SWEEP_MANUAL_STEP                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 356107
#define RSSMX_ATTR_RF_FREQUENCY_SWEEP_SHAPE                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 356108
#define RSSMX_ATTR_RF_FREQUENCY_SWEEP_TIME                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 356448
#define RSSMX_ATTR_RF_FREQUENCY_SWEEP_DWELL_TIME                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 356109
#define RSSMX_ATTR_RF_FREQUENCY_SWEEP_EXECUTE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356110
#define RSSMX_ATTR_TRIGGER_RF_FREQUENCY_SWEEP                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356111
#define RSSMX_ATTR_RF_FREQUENCY_SWEEP_START_FREQUENCY                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 356112
#define RSSMX_ATTR_RF_FREQUENCY_SWEEP_STOP_FREQUENCY                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 356113
#define RSSMX_ATTR_RF_FREQUENCY_SWEEP_CENTER_FREQUENCY                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 356114
#define RSSMX_ATTR_RF_FREQUENCY_SWEEP_SPAN                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 356115
#define RSSMX_ATTR_RF_FREQUENCY_SWEEP_POINTS                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 356116
#define RSSMX_ATTR_RF_FREQUENCY_SWEEP_SPACING                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356117
#define RSSMX_ATTR_RF_FREQUENCY_SWEEP_STEP_LIN                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 356118
#define RSSMX_ATTR_RF_FREQUENCY_SWEEP_STEP_LOG                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 356119
#define RSSMX_ATTR_LF_FREQUENCY_SWEEP_STATUS                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 356121
#define RSSMX_ATTR_LF_FREQUENCY_SWEEP_MODE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 356122
#define RSSMX_ATTR_LF_FREQUENCY_SWEEP_TRIGGER_SOURCE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 356123
#define RSSMX_ATTR_LF_FREQUENCY_SWEEP_RETRACE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356124
#define RSSMX_ATTR_LF_FREQUENCY_SWEEP_MANUAL_STEP                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 356125
#define RSSMX_ATTR_LF_FREQUENCY_SWEEP_SHAPE                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 356126
#define RSSMX_ATTR_LF_FREQUENCY_SWEEP_DWELL_TIME                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 356127
#define RSSMX_ATTR_LF_FREQUENCY_SWEEP_EXECUTE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356128
#define RSSMX_ATTR_TRIGGER_LF_FREQUENCY_SWEEP                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356129
#define RSSMX_ATTR_TRIGGER_LF_FREQUENCY_SWEEP_AUTO                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 356449
#define RSSMX_ATTR_LF_FREQUENCY_SWEEP_START_FREQUENCY                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 356130
#define RSSMX_ATTR_LF_FREQUENCY_SWEEP_STOP_FREQUENCY                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 356131
#define RSSMX_ATTR_LF_FREQUENCY_SWEEP_POINTS                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 356132
#define RSSMX_ATTR_LF_FREQUENCY_SWEEP_SPACING                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356133
#define RSSMX_ATTR_LF_FREQUENCY_SWEEP_STEP_LIN                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 356134
#define RSSMX_ATTR_LF_FREQUENCY_SWEEP_STEP_LOG                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 356135
#define RSSMX_ATTR_RF_LEVEL_SWEEP_STATUS                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 356137
#define RSSMX_ATTR_RF_LEVEL_SWEEP_MODE                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 356138
#define RSSMX_ATTR_RF_LEVEL_SWEEP_TRIGGER_SOURCE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 356139
#define RSSMX_ATTR_RF_LEVEL_SWEEP_RETRACE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 356140
#define RSSMX_ATTR_RF_LEVEL_SWEEP_MANUAL_STEP                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356141
#define RSSMX_ATTR_RF_LEVEL_SWEEP_SHAPE                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 356142
#define RSSMX_ATTR_RF_LEVEL_SWEEP_DWELL_TIME                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 356143
#define RSSMX_ATTR_RF_LEVEL_SWEEP_EXECUTE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 356144
#define RSSMX_ATTR_TRIGGER_RF_LEVEL_SWEEP                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 356145
#define RSSMX_ATTR_RF_LEVEL_SWEEP_START_LEVEL                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356146
#define RSSMX_ATTR_RF_LEVEL_SWEEP_STOP_LEVEL                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 356147
#define RSSMX_ATTR_RF_LEVEL_SWEEP_POINTS                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 356148
#define RSSMX_ATTR_RF_LEVEL_SWEEP_STEP                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 356149
#define RSSMX_ATTR_RF_COMBINED_SWEEP_EXECUTE_SINGLE_SWEEP                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 356767
#define RSSMX_ATTR_RF_COMBINED_SWEEP_MODE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 356768
#define RSSMX_ATTR_RF_COMBINED_SWEEP_TRIGGER_SOURCE                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 356769
#define RSSMX_ATTR_RF_COMBINED_SWEEP_RETRACE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 356770
#define RSSMX_ATTR_RF_COMBINED_SWEEP_SHAPE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 356771
#define RSSMX_ATTR_RF_COMBINED_SWEEP_DWELL_TIME                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 356772
#define RSSMX_ATTR_RF_COMBINED_SWEEP_RANGE_FREQUENCY_START                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 356773
#define RSSMX_ATTR_RF_COMBINED_SWEEP_RANGE_FREQUENCY_STOP                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 356774
#define RSSMX_ATTR_RF_COMBINED_SWEEP_RANGE_POWER_START                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 356775
#define RSSMX_ATTR_RF_COMBINED_SWEEP_RANGE_POWER_STOP                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 356776
#define RSSMX_ATTR_RF_COMBINED_SWEEP_STEP_COUNT                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 356777
#define RSSMX_ATTR_SWEEP_MODE_OUTPUT_VOLTAGE_MODE                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 356450
#define RSSMX_ATTR_SWEEP_MODE_OUTPUT_MARKER_MODE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 356451
#define RSSMX_ATTR_SWEEP_MARKER_POLARITY                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 356452
#define RSSMX_ATTR_SWEEP_ACTIVE_MARKER                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 356453
#define RSSMX_ATTR_SWEEP_MARKER_ENABLED                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 356454
#define RSSMX_ATTR_SWEEP_MARKER_FREQUENCY                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 356455
#define RSSMX_ATTR_LIST_STATUS                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 356151
#define RSSMX_ATTR_LIST_INDEX                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356153
#define RSSMX_ATTR_LIST_MODE                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 356154
#define RSSMX_ATTR_LIST_TRIGGER_SOURCE                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 356155
#define RSSMX_ATTR_LIST_TRIGGER_EXECUTE                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 356156
#define RSSMX_ATTR_LIST_DWELL_TIME_MODE                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 356157
#define RSSMX_ATTR_LIST_DWELL_TIME                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 356158
#define RSSMX_ATTR_LIST_DWELL_TIME_POINTS                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 356160
#define RSSMX_ATTR_LIST_RUN_MODE                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 356161
#define RSSMX_ATTR_LIST_DELETE_LIST                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 356163
#define RSSMX_ATTR_LIST_DELETE_ALL                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 356165
#define RSSMX_ATTR_LIST_FREE_MEMORY                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 356166
#define RSSMX_ATTR_LIST_RESET_LIST                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 356167
#define RSSMX_ATTR_LIST_SELECT_LIST                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 356168
#define RSSMX_ATTR_LIST_START_INDEX                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 356169
#define RSSMX_ATTR_LIST_STOP_INDEX                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 356170
#define RSSMX_ATTR_LIST_FREQUENCY_POINTS                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 356172
#define RSSMX_ATTR_LIST_LEVEL_POINTS                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 356174
#define RSSMX_ATTR_LIST_IMPORT_EXPORT                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 356176
#define RSSMX_ATTR_LIST_IMPORT_EXPORT_FILE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 356177
#define RSSMX_ATTR_LIST_IMPORT_EXPORT_ASCII_FILE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 356178
#define RSSMX_ATTR_LIST_IMPORT_EXPORT_ASCII_EXTENSION                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 356179
#define RSSMX_ATTR_LIST_IMPORT_EXPORT_ASCII_COLUMN_SEPARATOR                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 356180
#define RSSMX_ATTR_LIST_IMPORT_EXPORT_ASCII_DECIMAL_SEPARATOR                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356181
#define RSSMX_ATTR_LIST_IMPORT_EXPORT_EXECUTE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356182
#define RSSMX_ATTR_USER_CORRECTION_DATA_ENABLED                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 356183
#define RSSMX_ATTR_USER_CORRECTION_DATA_TABLE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356184
#define RSSMX_ATTR_USER_CORRECTION_DATA_NUMBER_OF_FREQUENCY_VALUES                RS_SPECIFIC_PUBLIC_ATTR_BASE + 356186
#define RSSMX_ATTR_USER_CORRECTION_DATA_NUMBER_OF_LEVEL_VALUES                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 356188
#define RSSMX_ATTR_USER_CORRECTION_DATA_SENSOR_ZEROING                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 356189
#define RSSMX_ATTR_USER_CORRECTION_DATA_VALUE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356192
#define RSSMX_ATTR_USER_CORRECTION_DATA_DELETE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 356193
#define RSSMX_ATTR_USER_CORRECTION_DATA_EXCHANGE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 356196
#define RSSMX_ATTR_USER_CORRECTION_DATA_EXCHANGE_FILE                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 356197
#define RSSMX_ATTR_USER_CORRECTION_DATA_EXCHANGE_ASCII_FILE                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 356428
#define RSSMX_ATTR_USER_CORRECTION_DATA_EXCHANGE_EXTENSION                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 356198
#define RSSMX_ATTR_USER_CORRECTION_DATA_EXCHANGE_COLUMN_SEPARATOR                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 356199
#define RSSMX_ATTR_USER_CORRECTION_DATA_EXCHANGE_DECIMAL_SEPARATOR                RS_SPECIFIC_PUBLIC_ATTR_BASE + 356200
#define RSSMX_ATTR_CORRECTION_EXECUTE                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 356201
#define RSSMX_ATTR_VOR_ENABLED                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 356456
#define RSSMX_ATTR_VOR_SET_TO_DEFAULT                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 356457
#define RSSMX_ATTR_VOR_SETTINGS_CATALOG                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 356458
#define RSSMX_ATTR_VOR_DELETE_SETTINGS                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 356459
#define RSSMX_ATTR_VOR_RECALL_SETTINGS                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 356461
#define RSSMX_ATTR_VOR_SAVE_SETTINGS                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 356463
#define RSSMX_ATTR_VOR_CARRIER_FREQUENCY_MODE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356465
#define RSSMX_ATTR_VOR_USER_MODE_CARRIER_FREQUENCY                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 356466
#define RSSMX_ATTR_VOR_ICAO_CHANNEL                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 356467
#define RSSMX_ATTR_VOR_EXT_AM                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356468
#define RSSMX_ATTR_VOR_MODE                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 356469
#define RSSMX_ATTR_VOR_VAR_REF_FREQUENCY                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 356470
#define RSSMX_ATTR_VOR_VAR_DEPTH                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 356471
#define RSSMX_ATTR_VOR_SUBCARRIER_FREQUENCY                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 356472
#define RSSMX_ATTR_VOR_SUBCARRIER_DEPTH                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 356473
#define RSSMX_ATTR_VOR_REF_DEVIATION                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 356474
#define RSSMX_ATTR_VOR_BEARING_ANGLE                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 356475
#define RSSMX_ATTR_VOR_DIRECTION                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 356476
#define RSSMX_ATTR_VOR_COMID_STATE                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 356477
#define RSSMX_ATTR_VOR_CODE                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 356478
#define RSSMX_ATTR_VOR_PERIOD                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356479
#define RSSMX_ATTR_VOR_FREQUENCY                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 356480
#define RSSMX_ATTR_VOR_DEPTH                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 356481
#define RSSMX_ATTR_VOR_TIME_SCHEMA                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 356482
#define RSSMX_ATTR_VOR_DOT_LENGTH                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 356483
#define RSSMX_ATTR_VOR_DASH_LENGTH                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 356484
#define RSSMX_ATTR_VOR_SYMBOL_SPACE                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 356485
#define RSSMX_ATTR_VOR_LETTER_SPACE                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 356486
#define RSSMX_ATTR_ILS_ENABLED                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 356487
#define RSSMX_ATTR_ILS_SET_TO_DEFAULT                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 356488
#define RSSMX_ATTR_ILS_SETTINGS_CATALOG                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 356489
#define RSSMX_ATTR_ILS_DELETE_SETTINGS                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 356490
#define RSSMX_ATTR_ILS_RECALL_SETTINGS                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 356492
#define RSSMX_ATTR_ILS_SAVE_SETTINGS                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 356494
#define RSSMX_ATTR_ILS_COMPONENT                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 356496
#define RSSMX_ATTR_ILS_GS_SET_TO_DEFAULT                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 356497
#define RSSMX_ATTR_ILS_GS_CARRIER_FREQUENCY_MODE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 356498
#define RSSMX_ATTR_ILS_GS_CARRIER_FREQUENCY                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 356499
#define RSSMX_ATTR_ILS_GS_ICAO_CHANNEL                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 356500
#define RSSMX_ATTR_ILS_GS_SYNC_WITH_LOCALIZER                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356501
#define RSSMX_ATTR_ILS_GS_EXT_AM                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 356502
#define RSSMX_ATTR_ILS_GS_OPERATING_MODE                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 356503
#define RSSMX_ATTR_ILS_GS_DDM_POLARITY                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 356504
#define RSSMX_ATTR_ILS_GS_UP_FREQUENCY                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 356505
#define RSSMX_ATTR_ILS_GS_DOWN_FREQUENCY                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 356506
#define RSSMX_ATTR_ILS_GS_UPDOWN_PHASE                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 356507
#define RSSMX_ATTR_ILS_GS_FLY_MODE                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 356508
#define RSSMX_ATTR_ILS_GS_SUM_OF_DEPTH                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 356509
#define RSSMX_ATTR_ILS_GS_DDM_COUPLING                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 356510
#define RSSMX_ATTR_ILS_GS_DDM_STEP                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 356511
#define RSSMX_ATTR_ILS_GS_DDM_CURRENT                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 356512
#define RSSMX_ATTR_ILS_GS_DDM_DEPTH                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 356513
#define RSSMX_ATTR_ILS_GS_DDM_LOGARITHMIC                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 356514
#define RSSMX_ATTR_ILS_GS_DDM_PERCENT                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 356515
#define RSSMX_ATTR_ILS_LOCALIZER_SET_TO_DEFAULT                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 356516
#define RSSMX_ATTR_ILS_LOCALIZER_CARRIER_FREQUENCY_MODE                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 356517
#define RSSMX_ATTR_ILS_LOCALIZER_CARRIER_FREQUENCY                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 356518
#define RSSMX_ATTR_ILS_LOCALIZER_ICAO_CHANNEL                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356519
#define RSSMX_ATTR_ILS_LOCALIZER_SYNC_WITH_GS                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356520
#define RSSMX_ATTR_ILS_LOCALIZER_EXT_AM                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 356521
#define RSSMX_ATTR_ILS_LOCALIZER_OPERATING_MODE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 356522
#define RSSMX_ATTR_ILS_LOCALIZER_DDM_POLARITY                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356523
#define RSSMX_ATTR_ILS_LOCALIZER_LEFT_FREQUENCY                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 356524
#define RSSMX_ATTR_ILS_LOCALIZER_RIGHT_FREQUENCY                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 356525
#define RSSMX_ATTR_ILS_LOCALIZER_LEFT_RIGHT_PHASE                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 356526
#define RSSMX_ATTR_ILS_LOCALIZER_FLY_MODE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 356527
#define RSSMX_ATTR_ILS_LOCALIZER_SUM_OF_DEPTH                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356528
#define RSSMX_ATTR_ILS_LOCALIZER_DDM_COUPLING                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356529
#define RSSMX_ATTR_ILS_LOCALIZER_DDM_STEP                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 356530
#define RSSMX_ATTR_ILS_LOCALIZER_DDM_CURRENT                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 356531
#define RSSMX_ATTR_ILS_LOCALIZER_DDM_DEPTH                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 356532
#define RSSMX_ATTR_ILS_LOCALIZER_DDM_LOGARITHMIC                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 356533
#define RSSMX_ATTR_ILS_LOCALIZER_DDM_PERCENT                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 356534
#define RSSMX_ATTR_ILS_LOCALIZER_COMID_STATE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 356535
#define RSSMX_ATTR_ILS_LOCALIZER_CODE                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 356536
#define RSSMX_ATTR_ILS_LOCALIZER_FREQUENCY                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 356537
#define RSSMX_ATTR_ILS_LOCALIZER_PERIOD                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 356538
#define RSSMX_ATTR_ILS_LOCALIZER_DEPTH                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 356539
#define RSSMX_ATTR_ILS_LOCALIZER_TIME_SCHEMA                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 356540
#define RSSMX_ATTR_ILS_LOCALIZER_DOT_LENGTH                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 356541
#define RSSMX_ATTR_ILS_LOCALIZER_DASH_LENGTH                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 356542
#define RSSMX_ATTR_ILS_LOCALIZER_SYMBOL_SPACE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356543
#define RSSMX_ATTR_ILS_LOCALIZER_LETTER_SPACE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356544
#define RSSMX_ATTR_ILS_MB_SET_TO_DEFAULT                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 356545
#define RSSMX_ATTR_ILS_MB_CARRIER_FREQUENCY_MODE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 356546
#define RSSMX_ATTR_ILS_MB_CARRIER_FREQUENCY                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 356547
#define RSSMX_ATTR_ILS_MB_MARKER_FREQUENCY                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 356548
#define RSSMX_ATTR_ILS_MB_MARKER_DEPTH                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 356549
#define RSSMX_ATTR_ILS_MB_PULSED_MARKER                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 356550
#define RSSMX_ATTR_ILS_MB_COMID_STATE                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 356551
#define RSSMX_ATTR_ILS_MB_CODE                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 356552
#define RSSMX_ATTR_ILS_MB_PERIOD                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 356553
#define RSSMX_ATTR_ILS_MB_FREQUENCY                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 356554
#define RSSMX_ATTR_ILS_MB_DEPTH                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 356555
#define RSSMX_ATTR_ILS_MB_TIME_SCHEMA                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 356556
#define RSSMX_ATTR_ILS_MB_DOT_LENGTH                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 356557
#define RSSMX_ATTR_ILS_MB_DASH_LENGTH                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 356558
#define RSSMX_ATTR_ILS_MB_SYMBOL_SPACE                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 356559
#define RSSMX_ATTR_ILS_MB_LETTER_SPACE                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 356560
#define RSSMX_ATTR_ADF_ENABLED                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 356561
#define RSSMX_ATTR_ADF_SET_TO_DEFAULT                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 356562
#define RSSMX_ATTR_ADF_SETTINGS_CATALOG                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 356563
#define RSSMX_ATTR_ADF_DELETE_SETTINGS                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 356564
#define RSSMX_ATTR_ADF_RECALL_SETTINGS                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 356566
#define RSSMX_ATTR_ADF_SAVE_SETTINGS                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 356568
#define RSSMX_ATTR_ADF_COM_ID_ENABLED                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 356570
#define RSSMX_ATTR_ADF_CODE                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 356571
#define RSSMX_ATTR_ADF_PERIOD                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356572
#define RSSMX_ATTR_ADF_FREQUENCY                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 356573
#define RSSMX_ATTR_ADF_DEPTH                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 356574
#define RSSMX_ATTR_ADF_TIME_SCHEMA                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 356575
#define RSSMX_ATTR_ADF_DOT_LENGTH                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 356576
#define RSSMX_ATTR_ADF_DASH_LENGTH                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 356577
#define RSSMX_ATTR_ADF_SYMBOL_SPACE                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 356578
#define RSSMX_ATTR_ADF_LETTER_SPACE                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 356579
#define RSSMX_ATTR_REFERENCE_OSCILLATOR_PRESET                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 356202
#define RSSMX_ATTR_REFERENCE_OSCILLATOR_SOURCE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 356203
#define RSSMX_ATTR_REFERENCE_OSCILLATOR_EFC_ENABLED                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 356204
#define RSSMX_ATTR_REFERENCE_OSCILLATOR_VOLTAGE_TUNING_SENSITIVITY                RS_SPECIFIC_PUBLIC_ATTR_BASE + 356205
#define RSSMX_ATTR_REFERENCE_OSCILLATOR_RF_SWITCH_OFF_ACTIVE                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 356206
#define RSSMX_ATTR_REFERENCE_OSCILLATOR_EXTERNAL_FREQUENCY                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 356207
#define RSSMX_ATTR_REFERENCE_OSCILLATOR_USER_DEFINED_EXTERNAL_FREQUENCY           RS_SPECIFIC_PUBLIC_ATTR_BASE + 356208
#define RSSMX_ATTR_REFERENCE_OSCILLATOR_SYNCHRONIZATION_BANDWIDTH                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 356209
#define RSSMX_ATTR_REFERENCE_OSCILLATOR_NOMINAL_SYNCHRONIZATION_BANDWIDTH         RS_SPECIFIC_PUBLIC_ATTR_BASE + 356580
#define RSSMX_ATTR_REFERENCE_OSCILLATOR_OUTPUT_FREQUENCY_MODE                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356210
#define RSSMX_ATTR_REFERENCE_OSCILLATOR_OUTPUT_REFERENCE_FREQUENCY                RS_SPECIFIC_PUBLIC_ATTR_BASE + 356427
#define RSSMX_ATTR_REFERENCE_OSCILLATOR_ADJUSTMENT_DAC_ACTIVE                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356211
#define RSSMX_ATTR_REFERENCE_OSCILLATOR_ADJUSTMENT_DAC_FREQUENCY                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 356212
#define RSSMX_ATTR_REFERENCE_OSCILLATOR_MINIMUM_LOCKING_RANGE                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356581
#define RSSMX_ATTR_CLOCK_SYNTHESIS_ENABLED                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 356213
#define RSSMX_ATTR_CLOCK_SYNTHESIS_OUTPUT_TYPE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 356214
#define RSSMX_ATTR_CLOCK_SYNTHESIS_FREQUENCY                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 356215
#define RSSMX_ATTR_CLOCK_SYNTHESIS_POWER                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 356216
#define RSSMX_ATTR_CLOCK_SYNTHESIS_DC_OFFSET_STATE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 356217
#define RSSMX_ATTR_CLOCK_SYNTHESIS_DC_OFFSET                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 356218
#define RSSMX_ATTR_CLOCK_SYNTHESIS_VOLTAGE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 356219
#define RSSMX_ATTR_CLOCK_SYNTHESIS_PHASE                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 356220
#define RSSMX_ATTR_CLOCK_SYNTHESIS_RESET_DELTA_PHASE_VALUE                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 356221
#define RSSMX_ATTR_CLOCK_SYNTHESIS_FREQUENCY_VARIATION_ACTIVE                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356222
#define RSSMX_ATTR_CLOCK_SYNTHESIS_FREQUENCY_VARIATION_STEP                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 356223
#define RSSMX_ATTR_CLOCK_SYNTHESIS_LEVEL_VARIATION_ACTIVE                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 356224
#define RSSMX_ATTR_CLOCK_SYNTHESIS_LEVEL_VARIATION_STEP                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 356225
#define RSSMX_ATTR_POWER_CONTINUOUS                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 356226
#define RSSMX_ATTR_POWER_READ_POWER                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 356227
#define RSSMX_ATTR_POWER_UNIT                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356228
#define RSSMX_ATTR_POWER_DEFAULT_APERTURE_TIME_ENABLED                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 356229
#define RSSMX_ATTR_POWER_APERTURE_TIME                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 356230
#define RSSMX_ATTR_POWER_USE_SPARAMETERS                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 356231
#define RSSMX_ATTR_POWER_S_PARAMETER_DATA_SET                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356232
#define RSSMX_ATTR_POWER_S_PARAMETER_DATA_SET_LIST                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 356233
#define RSSMX_ATTR_POWER_DISPLAY_PERMANENT_ENABLED                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 356234
#define RSSMX_ATTR_POWER_DISPLAY_PERMANENT                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 356235
#define RSSMX_ATTR_POWER_FREQUENCY                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 356236
#define RSSMX_ATTR_POWER_LOGGING_STATE                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 356237
#define RSSMX_ATTR_POWER_OFFSET                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 356238
#define RSSMX_ATTR_POWER_OFFSET_ENABLED                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 356239
#define RSSMX_ATTR_POWER_SOURCE                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 356240
#define RSSMX_ATTR_POWER_ZERO                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356241
#define RSSMX_ATTR_POWER_FILTER_LENGTH_AUTO                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 356242
#define RSSMX_ATTR_POWER_FILTER_LENGTH                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 356243
#define RSSMX_ATTR_POWER_FILTER_MODE                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 356244
#define RSSMX_ATTR_POWER_FILTER_NS_RATIO                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 356245
#define RSSMX_ATTR_POWER_FILTER_NS_RATIO_TIMEOUT                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 356246
#define RSSMX_ATTR_POWER_FILTER_SEARCH_ONCE                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 356247
#define RSSMX_ATTR_POWER_SENSOR_DEVICE                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 356248
#define RSSMX_ATTR_POWER_SENSOR_TYPE                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 356249
#define RSSMX_ATTR_POWER_SENSOR_VERSION                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 356250
#define RSSMX_ATTR_POWER_SENSOR_SERIAL_NUMBER                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356251
#define RSSMX_ATTR_POWER_SENSOR_MAPPING_SCAN_ENABLED                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 356252
#define RSSMX_ATTR_POWER_SENSOR_MAPPING_LIST                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 356253
#define RSSMX_ATTR_POWER_SENSOR_MAPPING_ADD_LAN_SENSOR                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 356778
#define RSSMX_ATTR_POWER_SENSOR_MAPPING_ADD_USB_SENSOR                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 356779
#define RSSMX_ATTR_POWER_SENSOR_MAPPING_REMOVE_LAN_SENSORS                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 356781
#define RSSMX_ATTR_POWER_SENSOR_MAPPING_REMOVE_USB_SENSORS                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 356782
#define RSSMX_ATTR_POWER_SENSOR_MAPPING_REMOVE_ALL                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 356783
#define RSSMX_ATTR_POWER_ANALYSIS_INITIATE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 356583
#define RSSMX_ATTR_POWER_ANALYSIS_ABORT                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 356584
#define RSSMX_ATTR_POWER_ANALYSIS_TRACE_STATE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356585
#define RSSMX_ATTR_POWER_ANALYSIS_TRACE_REFERENCE                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 356586
#define RSSMX_ATTR_POWER_ANALYSIS_TRACE_SOURCE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 356587
#define RSSMX_ATTR_POWER_ANALYSIS_TRACE_COLOR                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356588
#define RSSMX_ATTR_POWER_ANALYSIS_FREQUENCY_SOURCE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 356794
#define RSSMX_ATTR_POWER_ANALYSIS_POWER_SOURCE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 356795
#define RSSMX_ATTR_POWER_ANALYSIS_TIME_SOURCE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356796
#define RSSMX_ATTR_GENERATE_POWER_ANALYSIS_FREQUENCY_REFERENCE                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 356590
#define RSSMX_ATTR_GENERATE_POWER_ANALYSIS_FREQUENCY_REFERENCE_POINTS             RS_SPECIFIC_PUBLIC_ATTR_BASE + 356591
#define RSSMX_ATTR_GENERATE_POWER_ANALYSIS_POWER_REFERENCE                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 356593
#define RSSMX_ATTR_GENERATE_POWER_ANALYSIS_POWER_REFERENCE_POINTS                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 356594
#define RSSMX_ATTR_GENERATE_POWER_ANALYSIS_TIME_REFERENCE                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 356596
#define RSSMX_ATTR_GENERATE_POWER_ANALYSIS_TIME_REFERENCE_POINTS                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 356597
#define RSSMX_ATTR_POWER_ANALYSIS_MATH_FREQUENCY_ENABLED                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 356598
#define RSSMX_ATTR_POWER_ANALYSIS_MATH_FREQUENCY_SUBTRACT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 356599
#define RSSMX_ATTR_POWER_ANALYSIS_MATH_FREQUENCY_X                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 356797
#define RSSMX_ATTR_POWER_ANALYSIS_MATH_FREQUENCY_Y                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 356798
#define RSSMX_ATTR_POWER_ANALYSIS_MATH_POWER_ENABLED                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 356600
#define RSSMX_ATTR_POWER_ANALYSIS_MATH_POWER_SUBTRACT                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 356601
#define RSSMX_ATTR_POWER_ANALYSIS_MATH_POWER_X                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 356799
#define RSSMX_ATTR_POWER_ANALYSIS_MATH_POWER_Y                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 356800
#define RSSMX_ATTR_POWER_ANALYSIS_MATH_TIME_ENABLED                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 356602
#define RSSMX_ATTR_POWER_ANALYSIS_MATH_TIME_SUBTRACT                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 356603
#define RSSMX_ATTR_POWER_ANALYSIS_MATH_TIME_X                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356801
#define RSSMX_ATTR_POWER_ANALYSIS_MATH_TIME_Y                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356802
#define RSSMX_ATTR_POWER_ANALYSIS_TRACE_DATA_POINTS                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 356604
#define RSSMX_ATTR_POWER_ANALYSIS_MEASUREMENT_MODE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 356608
#define RSSMX_ATTR_POWER_ANALYSIS_MEASUREMENT_EXECUTION                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 356609
#define RSSMX_ATTR_POWER_ANALYSIS_FREQUENCY_MIN                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 356611
#define RSSMX_ATTR_POWER_ANALYSIS_FREQUENCY_MAX                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 356612
#define RSSMX_ATTR_POWER_ANALYSIS_FREQUENCY_STEPS                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 356613
#define RSSMX_ATTR_POWER_ANALYSIS_FREQUENCY_TIMING                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 356614
#define RSSMX_ATTR_POWER_ANALYSIS_FREQUENCY_SPACING                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 356615
#define RSSMX_ATTR_POWER_ANALYSIS_FREQUENCY_EXECUTION                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 356616
#define RSSMX_ATTR_POWER_ANALYSIS_POWER_MIN                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 356617
#define RSSMX_ATTR_POWER_ANALYSIS_POWER_MAX                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 356618
#define RSSMX_ATTR_POWER_ANALYSIS_POWER_STEPS                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356619
#define RSSMX_ATTR_POWER_ANALYSIS_POWER_TIMING                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 356620
#define RSSMX_ATTR_POWER_ANALYSIS_POWER_EXECUTION                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 356621
#define RSSMX_ATTR_POWER_ANALYSIS_TIME_MIN                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 356622
#define RSSMX_ATTR_POWER_ANALYSIS_TIME_MAX                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 356623
#define RSSMX_ATTR_POWER_ANALYSIS_TIME_STEPS                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 356624
#define RSSMX_ATTR_POWER_ANALYSIS_TIME_AVERAGE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 356625
#define RSSMX_ATTR_POWER_ANALYSIS_TIME_EXECUTION                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 356626
#define RSSMX_ATTR_POWER_ANALYSIS_TIME_TRIGGER_EVENTS                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 356750
#define RSSMX_ATTR_POWER_ANALYSIS_DIAGRAM_GRID_ENABLED                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 356627
#define RSSMX_ATTR_POWER_ANALYSIS_DIAGRAM_BACKGROUND_COLOR                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 356628
#define RSSMX_ATTR_POWER_ANALYSIS_DIAGRAM_STANDARD_VIEW_ENABLED                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 356629
#define RSSMX_ATTR_POWER_ANALYSIS_DIAGRAM_MARKER_VIEW_ENABLED                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356630
#define RSSMX_ATTR_POWER_ANALYSIS_DIAGRAM_PULSE_DATA_VIEW_ENABLED                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 356631
#define RSSMX_ATTR_POWER_ANALYSIS_DIAGRAM_FULLSCREEN_ENABLED                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 356632
#define RSSMX_ATTR_POWER_ANALYSIS_DIAGRAM_FREQUENCY_MIN                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 356633
#define RSSMX_ATTR_POWER_ANALYSIS_DIAGRAM_FREQUENCY_MAX                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 356634
#define RSSMX_ATTR_POWER_ANALYSIS_DIAGRAM_FREQUENCY_AUTO_SCALE                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 356635
#define RSSMX_ATTR_POWER_ANALYSIS_DIAGRAM_FREQUENCY_AUTO_SCALE_RESET              RS_SPECIFIC_PUBLIC_ATTR_BASE + 356636
#define RSSMX_ATTR_POWER_ANALYSIS_DIAGRAM_POWER_MIN                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 356637
#define RSSMX_ATTR_POWER_ANALYSIS_DIAGRAM_POWER_MAX                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 356638
#define RSSMX_ATTR_POWER_ANALYSIS_DIAGRAM_POWER_AUTO_SCALE                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 356639
#define RSSMX_ATTR_POWER_ANALYSIS_DIAGRAM_POWER_AUTO_SCALE_RESET                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 356640
#define RSSMX_ATTR_POWER_ANALYSIS_DIAGRAM_TIME_MIN                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 356641
#define RSSMX_ATTR_POWER_ANALYSIS_DIAGRAM_TIME_MAX                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 356642
#define RSSMX_ATTR_POWER_ANALYSIS_DIAGRAM_TIME_AUTO_SCALE                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 356643
#define RSSMX_ATTR_POWER_ANALYSIS_DIAGRAM_TIME_AUTO_SCALE_RESET                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 356644
#define RSSMX_ATTR_POWER_ANALYSIS_GATE_INDICATION_ENABLED                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 356645
#define RSSMX_ATTR_POWER_ANALYSIS_GATE_TRACE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 356646
#define RSSMX_ATTR_POWER_ANALYSIS_GATE_ENABLED                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 356647
#define RSSMX_ATTR_POWER_ANALYSIS_GATE_AVERAGE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 356648
#define RSSMX_ATTR_POWER_ANALYSIS_GATE_MAXIMUM                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 356649
#define RSSMX_ATTR_POWER_ANALYSIS_GATE_START                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 356650
#define RSSMX_ATTR_POWER_ANALYSIS_GATE_STOP                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 356651
#define RSSMX_ATTR_POWER_ANALYSIS_SENSOR_FREQUENCY_OFFSET_ENABLED                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 356652
#define RSSMX_ATTR_POWER_ANALYSIS_SENSOR_FREQUENCY_OFFSET                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 356653
#define RSSMX_ATTR_POWER_ANALYSIS_SENSOR_FREQUENCY_USE_SEPARATE_FREQ              RS_SPECIFIC_PUBLIC_ATTR_BASE + 356654
#define RSSMX_ATTR_POWER_ANALYSIS_SENSOR_FREQUENCY_SEPARATE_FREQ_START            RS_SPECIFIC_PUBLIC_ATTR_BASE + 356655
#define RSSMX_ATTR_POWER_ANALYSIS_SENSOR_FREQUENCY_SEPARATE_FREQ_STOP             RS_SPECIFIC_PUBLIC_ATTR_BASE + 356656
#define RSSMX_ATTR_POWER_ANALYSIS_SENSOR_POWER_OFFSET_ENABLED                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356657
#define RSSMX_ATTR_POWER_ANALYSIS_SENSOR_POWER_OFFSET                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 356658
#define RSSMX_ATTR_POWER_ANALYSIS_SENSOR_POWER_USE_SEPARATE_FREQ                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 356659
#define RSSMX_ATTR_POWER_ANALYSIS_SENSOR_POWER_SEPARATE_FREQ                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 356660
#define RSSMX_ATTR_POWER_ANALYSIS_SENSOR_TIME_OFFSET_ENABLED                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 356661
#define RSSMX_ATTR_POWER_ANALYSIS_SENSOR_TIME_OFFSET                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 356662
#define RSSMX_ATTR_POWER_ANALYSIS_SENSOR_TIME_USE_SEPARATE_FREQ                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 356663
#define RSSMX_ATTR_POWER_ANALYSIS_SENSOR_TIME_SEPARATE_FREQ                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 356664
#define RSSMX_ATTR_POWER_ANALYSIS_PULSE_DATA_ALL_INDICATIONS_ENABLED              RS_SPECIFIC_PUBLIC_ATTR_BASE + 356665
#define RSSMX_ATTR_POWER_ANALYSIS_PULSE_DATA_ENABLED                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 356666
#define RSSMX_ATTR_POWER_ANALYSIS_PULSE_DATA_BASE                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 356667
#define RSSMX_ATTR_POWER_ANALYSIS_PULSE_DATA_BASE_CALCULATED                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 356668
#define RSSMX_ATTR_POWER_ANALYSIS_PULSE_DATA_DISTAL                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 356669
#define RSSMX_ATTR_POWER_ANALYSIS_PULSE_DATA_DISTAL_CALCULATED                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 356670
#define RSSMX_ATTR_POWER_ANALYSIS_PULSE_DATA_MESIAL                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 356671
#define RSSMX_ATTR_POWER_ANALYSIS_PULSE_DATA_MESIAL_CALCULATED                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 356672
#define RSSMX_ATTR_POWER_ANALYSIS_PULSE_DATA_PROXIMAL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 356673
#define RSSMX_ATTR_POWER_ANALYSIS_PULSE_DATA_PROXIMAL_CALCULATED                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 356674
#define RSSMX_ATTR_POWER_ANALYSIS_NOTIFICATION_DUTY_CYCLE                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 356675
#define RSSMX_ATTR_POWER_ANALYSIS_NOTIFICATION_DUTY_CYCLE_ENABLED                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 356676
#define RSSMX_ATTR_POWER_ANALYSIS_NOTIFICATION_PULSE_WIDTH                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 356677
#define RSSMX_ATTR_POWER_ANALYSIS_NOTIFICATION_PULSE_WIDTH_ENABLED                RS_SPECIFIC_PUBLIC_ATTR_BASE + 356678
#define RSSMX_ATTR_POWER_ANALYSIS_NOTIFICATION_PULSE_PERIOD                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 356679
#define RSSMX_ATTR_POWER_ANALYSIS_NOTIFICATION_PULSE_PERIOD_ENABLED               RS_SPECIFIC_PUBLIC_ATTR_BASE + 356680
#define RSSMX_ATTR_POWER_ANALYSIS_NOTIFICATION_PULSE_OFF_TIME                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356681
#define RSSMX_ATTR_POWER_ANALYSIS_NOTIFICATION_PULSE_OFF_TIME_ENABLED             RS_SPECIFIC_PUBLIC_ATTR_BASE + 356682
#define RSSMX_ATTR_POWER_ANALYSIS_NOTIFICATION_PULSE_STATE                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 356683
#define RSSMX_ATTR_POWER_ANALYSIS_PULSE_TRANSITION_RISE_TIME                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 356684
#define RSSMX_ATTR_POWER_ANALYSIS_PULSE_TRANSITION_RISE_TIME_ENABLED              RS_SPECIFIC_PUBLIC_ATTR_BASE + 356685
#define RSSMX_ATTR_POWER_ANALYSIS_PULSE_TRANSITION_FALL_TIME                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 356686
#define RSSMX_ATTR_POWER_ANALYSIS_PULSE_TRANSITION_FALL_TIME_ENABLED              RS_SPECIFIC_PUBLIC_ATTR_BASE + 356687
#define RSSMX_ATTR_POWER_ANALYSIS_PULSE_TRANSITION_START_TIME                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356688
#define RSSMX_ATTR_POWER_ANALYSIS_PULSE_TRANSITION_START_TIME_ENABLED             RS_SPECIFIC_PUBLIC_ATTR_BASE + 356689
#define RSSMX_ATTR_POWER_ANALYSIS_PULSE_TRANSITION_STOP_TIME                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 356690
#define RSSMX_ATTR_POWER_ANALYSIS_PULSE_TRANSITION_STOP_TIME_ENABLED              RS_SPECIFIC_PUBLIC_ATTR_BASE + 356691
#define RSSMX_ATTR_POWER_ANALYSIS_PULSE_TRANSITION_OVERSHOOT_RISING_EDGE          RS_SPECIFIC_PUBLIC_ATTR_BASE + 356692
#define RSSMX_ATTR_POWER_ANALYSIS_PULSE_TRANSITION_OVERSHOOT_RISING_EDGE_ENABLED  RS_SPECIFIC_PUBLIC_ATTR_BASE + 356693
#define RSSMX_ATTR_POWER_ANALYSIS_PULSE_TRANSITION_OVERSHOOT_FALLING_EDGE         RS_SPECIFIC_PUBLIC_ATTR_BASE + 356694
#define RSSMX_ATTR_POWER_ANALYSIS_PULSE_TRANSITION_OVERSHOOT_FALLING_EDGE_ENABLED RS_SPECIFIC_PUBLIC_ATTR_BASE + 356695
#define RSSMX_ATTR_POWER_ANALYSIS_POWER_MINIMAL                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 356696
#define RSSMX_ATTR_POWER_ANALYSIS_POWER_MINIMAL_ENABLED                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 356697
#define RSSMX_ATTR_POWER_ANALYSIS_POWER_PEAK                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 356698
#define RSSMX_ATTR_POWER_ANALYSIS_POWER_PEAK_ENABLED                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 356699
#define RSSMX_ATTR_POWER_ANALYSIS_POWER_AVERAGE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 356700
#define RSSMX_ATTR_POWER_ANALYSIS_POWER_AVERAGE_ENABLED                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 356701
#define RSSMX_ATTR_POWER_ANALYSIS_PULSE_POWER_TOP                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 356702
#define RSSMX_ATTR_POWER_ANALYSIS_PULSE_POWER_TOP_ENABLED                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 356703
#define RSSMX_ATTR_POWER_ANALYSIS_PULSE_POWER_BASE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 356704
#define RSSMX_ATTR_POWER_ANALYSIS_PULSE_POWER_BASE_ENABLED                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 356705
#define RSSMX_ATTR_POWER_ANALYSIS_PULSE_POWER_DISTAL                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 356706
#define RSSMX_ATTR_POWER_ANALYSIS_PULSE_POWER_DISTAL_ENABLED                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 356707
#define RSSMX_ATTR_POWER_ANALYSIS_PULSE_POWER_MESIAL                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 356708
#define RSSMX_ATTR_POWER_ANALYSIS_PULSE_POWER_MESIAL_ENABLED                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 356709
#define RSSMX_ATTR_POWER_ANALYSIS_PULSE_POWER_PROXIMAL                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 356710
#define RSSMX_ATTR_POWER_ANALYSIS_PULSE_POWER_PROXIMAL_ENABLED                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 356711
#define RSSMX_ATTR_POWER_ANALYSIS_TRIGGER_MODE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 356712
#define RSSMX_ATTR_POWER_ANALYSIS_TRIGGER_LEVEL                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 356713
#define RSSMX_ATTR_POWER_ANALYSIS_TRIGGER_SLOPE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 356714
#define RSSMX_ATTR_POWER_ANALYSIS_TRIGGER_HYSTERESIS                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 356715
#define RSSMX_ATTR_POWER_ANALYSIS_TRIGGER_DROP_OUT_TIME                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 356716
#define RSSMX_ATTR_POWER_ANALYSIS_TRIGGER_AUTO_SET                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 356717
#define RSSMX_ATTR_SCREENSHOT_FILE_NAME                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 356718
#define RSSMX_ATTR_SCREENSHOT_EXECUTE                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 356719
#define RSSMX_ATTR_SCREENSHOT_DEVICE                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 356721
#define RSSMX_ATTR_SCREENSHOT_SIZE                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 356722
#define RSSMX_ATTR_SCREENSHOT_FORMAT                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 356723
#define RSSMX_ATTR_SCREENSHOT_CSV_HEADER_ENABLED                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 356724
#define RSSMX_ATTR_SCREENSHOT_CSV_ORIENTATION                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356725
#define RSSMX_ATTR_SCREENSHOT_CSV_COLUMN_SEPARATOR                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 356726
#define RSSMX_ATTR_SCREENSHOT_CSV_DECIMAL_SEPARATOR                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 356727
#define RSSMX_ATTR_SCREENSHOT_AUTOMATIC_NAMING_ENABLED                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 356728
#define RSSMX_ATTR_SCREENSHOT_AUTOMATIC_NAMING_DIRECTORY                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 356729
#define RSSMX_ATTR_SCREENSHOT_AUTOMATIC_NAMING_CLEAR_DIRECTORY                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 356730
#define RSSMX_ATTR_SCREENSHOT_AUTOMATIC_NAMING_PREFIX_ENABLED                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356731
#define RSSMX_ATTR_SCREENSHOT_AUTOMATIC_NAMING_PREFIX                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 356732
#define RSSMX_ATTR_SCREENSHOT_AUTOMATIC_NAMING_YEAR_ENABLED                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 356733
#define RSSMX_ATTR_SCREENSHOT_AUTOMATIC_NAMING_YEAR                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 356734
#define RSSMX_ATTR_SCREENSHOT_AUTOMATIC_NAMING_MONTH_ENABLED                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 356735
#define RSSMX_ATTR_SCREENSHOT_AUTOMATIC_NAMING_MONTH                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 356736
#define RSSMX_ATTR_SCREENSHOT_AUTOMATIC_NAMING_DAY_ENABLED                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 356737
#define RSSMX_ATTR_SCREENSHOT_AUTOMATIC_NAMING_DAY                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 356738
#define RSSMX_ATTR_SCREENSHOT_AUTOMATIC_NAMING_NUMBER                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 356739
#define RSSMX_ATTR_SCREENSHOT_AUTOMATIC_NAMING_FILE_NAME                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 356740
#define RSSMX_ATTR_CALIBRATION_ALL                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 356255
#define RSSMX_ATTR_SAVE_IMMEDIATE                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 356741
#define RSSMX_ATTR_RECALL_IMMEDIATE                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 356743
#define RSSMX_ATTR_CALIBRATION_DATE                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 356256
#define RSSMX_ATTR_CALIBRATION_INTERNAL_REFERENCE_FREQUENCY                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 356257
#define RSSMX_ATTR_CALIBRATION_CONTINUE_ON_ERROR                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 356258
#define RSSMX_ATTR_CALIBRATION_CLOCK_SYNTHESIS                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 356259
#define RSSMX_ATTR_CALIBRATION_LEVEL_BANDWIDTH                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 356260
#define RSSMX_ATTR_CALIBRATION_INTERNAL_MODULATION_GENERATOR                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 356261
#define RSSMX_ATTR_CALIBRATION_LAST_FULL_ADJUSTMENT                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 356262
#define RSSMX_ATTR_CALIBRATION_TEMPERATURE_OFFSET_TO_LAST_FULL_ADJUSTMENT         RS_SPECIFIC_PUBLIC_ATTR_BASE + 356263
#define RSSMX_ATTR_CALIBRATION_LOGGING                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 356264
#define RSSMX_ATTR_CALIBRATION_FM_OFFSET                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 356265
#define RSSMX_ATTR_CALIBRATION_FREQUENCY                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 356266
#define RSSMX_ATTR_CALIBRATION_LEVEL_CALIBRATION_DATA                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 356267
#define RSSMX_ATTR_CALIBRATION_LEVEL_ENABLED                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 356269
#define RSSMX_ATTR_CALIBRATION_LEVEL                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 356270
#define RSSMX_ATTR_CALIBRATION_INFORMATION                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 356745
#define RSSMX_ATTR_CALIBRATION_TIME_SINCE_LAST_FULL_ADJUSTMENT                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 356746
#define RSSMX_ATTR_HARD_COPY_EXECUTE                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 356276
#define RSSMX_ATTR_HARD_COPY_GRAPHIC_FORMAT                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 356277
#define RSSMX_ATTR_HARD_COPY_REGION                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 356278
#define RSSMX_ATTR_HARD_COPY_FILE_NAME                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 356279
#define RSSMX_ATTR_HARD_COPY_GENERATED_AUTO_FILE_NAME                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 356280
#define RSSMX_ATTR_HARD_COPY_AUTO_FILE_DIRECTORY                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 356281
#define RSSMX_ATTR_CLEAR_HARD_COPY_FILE                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 356282
#define RSSMX_ATTR_HARD_COPY_AUTO_FILE_NAME                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 356283
#define RSSMX_ATTR_HARD_COPY_AUTO_NAMING_ENABLED                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 356284
#define RSSMX_ATTR_HARD_COPY_DAY_TO_FILE_NAME                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356285
#define RSSMX_ATTR_HARD_COPY_MONTH_TO_FILE_NAME                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 356287
#define RSSMX_ATTR_HARD_COPY_YEAR_TO_FILE_NAME                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 356289
#define RSSMX_ATTR_HARD_COPY_CURRENT_NUMBER                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 356291
#define RSSMX_ATTR_HARD_COPY_AUTO_PREFIX_ENABLED                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 356292
#define RSSMX_ATTR_HARD_COPY_FILE_NAME_PREFIX                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356293
#define RSSMX_ATTR_START_SELFTEST                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 356294
#define RSSMX_ATTR_SELFTEST_RESULT                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 356295
#define RSSMX_ATTR_KEYBOARD_LAYOUT                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 356296
#define RSSMX_ATTR_DISPLAY_BRIGHTNESS                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 356297
#define RSSMX_ATTR_DISPLAY_BUTTON_BRIGHTNESS                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 356298
#define RSSMX_ATTR_DISPLAY_UPDATE                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 356299
#define RSSMX_ATTR_DISPLAY_ANNOTATION_AMPLITUDE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 356300
#define RSSMX_ATTR_DISPLAY_ANNOTATION_FREQUENCY                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 356301
#define RSSMX_ATTR_DISPLAY_ANNOTATION_ALL                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 356302
#define RSSMX_ATTR_DISPLAY_DIALOG_ID                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 356303
#define RSSMX_ATTR_DISPLAY_DIALOG_OPEN                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 356305
#define RSSMX_ATTR_DISPLAY_DIALOG_CLOSE                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 356306
#define RSSMX_ATTR_DISPLAY_DIALOG_CLOSE_ALL                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 356307
#define RSSMX_ATTR_DISPLAY_POWER_SAVE_ENABLED                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356308
#define RSSMX_ATTR_DISPLAY_POWER_SAVE_HOLDOFF                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356309
#define RSSMX_ATTR_MMEMORY_CHANGE_DIRECTORY                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 356312
#define RSSMX_ATTR_MMEMORY_DELETE                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 356319
#define RSSMX_ATTR_MMEMORY_MAKE_DIRECTORY                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 356324
#define RSSMX_ATTR_MMEMORY_DRIVE                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 356326
#define RSSMX_ATTR_MMEMORY_REMOVE_DIRECTORY                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 356327
#define RSSMX_ATTR_SYSTEM_USER_INTERFACE_LOCK                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356330
#define RSSMX_ATTR_SYSTEM_DISPLAY_LOCK_ENABLED                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 356331
#define RSSMX_ATTR_SYSTEM_KEYBOARD_LOCK                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 356332
#define RSSMX_ATTR_SYSTEM_GPIB_RESOURCE                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 356334
#define RSSMX_ATTR_SYSTEM_REBOOT                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 356335
#define RSSMX_ATTR_SYSTEM_RESTART                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 356336
#define RSSMX_ATTR_SYSTEM_SHUTDOWN                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 356337
#define RSSMX_ATTR_SYSTEM_BEEPER                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 356338
#define RSSMX_ATTR_SYSTEM_USB_RESOURCE                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 356339
#define RSSMX_ATTR_SYSTEM_HISLIP_RESOURCE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 356340
#define RSSMX_ATTR_SYSTEM_NETWORK_RESOURCE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 356341
#define RSSMX_ATTR_SYSTEM_SOCKET_RESOURCE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 356342
#define RSSMX_ATTR_SYSTEM_SERIAL_RESOURCE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 356343
#define RSSMX_ATTR_SYSTEM_SERIAL_BAUD                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 356344
#define RSSMX_ATTR_SYSTEM_SERIAL_PARITY                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 356345
#define RSSMX_ATTR_SYSTEM_SERIAL_STOP_BITS                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 356346
#define RSSMX_ATTR_SYSTEM_EXPORT_HELP                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 356347
#define RSSMX_ATTR_SYSTEM_SCPI_INFORMATION                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 356348
#define RSSMX_ATTR_SYSTEM_OPTION_USER_DEFINED                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356349
#define RSSMX_ATTR_SYSTEM_ALL_DATA_SHEETS                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 356350
#define RSSMX_ATTR_SYSTEM_DATA_SHEET_FACTORY_VERSION                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 356351
#define RSSMX_ATTR_SYSTEM_DATA_SHEET_VERSION                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 356352
#define RSSMX_ATTR_SYSTEM_DATA_SHEET_IDS                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 356353
#define RSSMX_ATTR_SYSTEM_STARTUP_COMPLETE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 356357
#define RSSMX_ATTR_SYSTEM_DATE                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 356358
#define RSSMX_ATTR_SYSTEM_TIME_ZONE_CATALOG                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 356360
#define RSSMX_ATTR_SYSTEM_TIME_ZONE                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 356361
#define RSSMX_ATTR_SYSTEM_UPTIME                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 356362
#define RSSMX_ATTR_SYSTEM_BIOS_VERSION                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 356363
#define RSSMX_ATTR_SYSTEM_VERSION                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 356364
#define RSSMX_ATTR_SYSTEM_OS_VERSION                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 356365
#define RSSMX_ATTR_SYSTEM_USER_DIRECTORY                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 356366
#define RSSMX_ATTR_SYSTEM_WAIT                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 356368
#define RSSMX_ATTR_SYSTEM_DELETE_TEMPORARY_FILES                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 356747
#define RSSMX_ATTR_SYSTEM_NTP_ENABLED                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 356748
#define RSSMX_ATTR_SYSTEM_NTP_ADDRESS                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 356749
#define RSSMX_ATTR_SYSTEM_NETWORK_RESTART                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 356369
#define RSSMX_ATTR_SYSTEM_NETWORK_ENABLED                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 356370
#define RSSMX_ATTR_SYSTEM_NETWORK_DOMAIN                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 356371
#define RSSMX_ATTR_SYSTEM_NETWORK_MAC_ADDRESS                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356372
#define RSSMX_ATTR_SYSTEM_NETWORK_HOSTNAME                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 356373
#define RSSMX_ATTR_SYSTEM_NETWORK_WORKGROUP                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 356374
#define RSSMX_ATTR_SYSTEM_NETWORK_IP_ADDRESS                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 356375
#define RSSMX_ATTR_SYSTEM_NETWORK_IP_ADDRESS_MODE                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 356376
#define RSSMX_ATTR_SYSTEM_NETWORK_IP_ADDRESS_SUBNET_MASK                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 356377
#define RSSMX_ATTR_SYSTEM_NETWORK_IP_ADDRESS_GATEWAY                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 356378
#define RSSMX_ATTR_SYSTEM_NETWORK_DNS_SERVER                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 356379
#define RSSMX_ATTR_SYSTEM_ERROR_QUEUE_ALL                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 356380
#define RSSMX_ATTR_SYSTEM_ERROR_QUEUE_CODE_ALL                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 356381
#define RSSMX_ATTR_SYSTEM_ERROR_QUEUE_CODE_NEXT                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 356382
#define RSSMX_ATTR_SYSTEM_ERROR_QUEUE_COUNT                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 356383
#define RSSMX_ATTR_SYSTEM_ERROR                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 356384
#define RSSMX_ATTR_SYSTEM_ERROR_HISTORY                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 356385
#define RSSMX_ATTR_SYSTEM_CLEAR_HISTORY                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 356386
#define RSSMX_ATTR_SYSTEM_STATIC_ERRORS                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 356387
#define RSSMX_ATTR_SYSTEM_ERROR_OLDEST                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 356784
#define RSSMX_ATTR_STATUS_PRESET                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 356390
#define RSSMX_ATTR_UNIT_ANGLE                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356391
#define RSSMX_ATTR_UNIT_POWER                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 356392
#define RSSMX_ATTR_ID_QUERY_RESPONSE                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 356412
#define RSSMX_ATTR_PRESET_DEVICE                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 356415
#define RSSMX_ATTR_RESET_SIGNAL_PATH                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 356416
#define RSSMX_ATTR_FACTORY_PRESET                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 356417
#define RSSMX_ATTR_PROCESS_ALL_PREVIOUS_COMMANDS                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 356421
#define RSSMX_ATTR_INIT_SW_TRIGGER                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 356426

// RsSmx_rngFrequencySweepSpacing
#define RSSMX_VAL_FREQ_SWEEP_SPACING_LINEAR                                 0
#define RSSMX_VAL_FREQ_SWEEP_SPACING_LOGARITHMIC                            1

// RsSmx_rngReferenceOscillatorSource
#define RSSMX_VAL_ROSC_SOURCE_INTERNAL                                      0
#define RSSMX_VAL_ROSC_SOURCE_EXTERNAL                                      1

// RsSmx_rngReferenceOscillatorExternalFrequency
#define RSSMX_VAL_SMU_ROSC_EXTERNAL_FREQUENCY_VAR                           0
#define RSSMX_VAL_SMU_ROSC_EXTERNAL_FREQUENCY_10MHZ                         1
#define RSSMX_VAL_SMU_ROSC_EXTERNAL_FREQUENCY_100MHZ                        2
#define RSSMX_VAL_SMU_ROSC_EXTERNAL_FREQUENCY_1GHZ                          3

// RsSmx_rngPowerUnit
#define RSSMX_VAL_SENSE_POWER_UNIT_DBM                                      0
#define RSSMX_VAL_SENSE_POWER_UNIT_DBUV                                     1
#define RSSMX_VAL_SENSE_POWER_UNIT_WATT                                     2

// RsSmx_rngPowerSource
#define RSSMX_VAL_SENSE_POWER_SOURCE_A                                      0
#define RSSMX_VAL_SENSE_POWER_SOURCE_USER                                   1
#define RSSMX_VAL_SENSE_POWER_SOURCE_RF                                     2

// RsSmx_rngPowerFilterMode
#define RSSMX_VAL_SENSE_POWER_FILTER_TYPE_AUTO                              0
#define RSSMX_VAL_SENSE_POWER_FILTER_TYPE_USER                              1
#define RSSMX_VAL_SENSE_POWER_FILTER_TYPE_NSR                               2

// RsSmx_rngSlope
#define RSSMX_VAL_SLOPE_NEGATIVE                                            0
#define RSSMX_VAL_SLOPE_POSITIVE                                            1

// RsSmx_rngKeyboardLayout
#define RSSMX_VAL_SYSTEM_KEYBOARD_US                                        0
#define RSSMX_VAL_SYSTEM_KEYBOARD_DE                                        1
#define RSSMX_VAL_SYSTEM_KEYBOARD_ENGUK                                     2
#define RSSMX_VAL_SYSTEM_KEYBOARD_CHIN                                      3
#define RSSMX_VAL_SYSTEM_KEYBOARD_DAN                                       4
#define RSSMX_VAL_SYSTEM_KEYBOARD_DUTC                                      5
#define RSSMX_VAL_SYSTEM_KEYBOARD_FINN                                      6
#define RSSMX_VAL_SYSTEM_KEYBOARD_FREN                                      7
#define RSSMX_VAL_SYSTEM_KEYBOARD_FREB                                      8
#define RSSMX_VAL_SYSTEM_KEYBOARD_FREC                                      9
#define RSSMX_VAL_SYSTEM_KEYBOARD_ITAL                                      10
#define RSSMX_VAL_SYSTEM_KEYBOARD_JAP                                       11
#define RSSMX_VAL_SYSTEM_KEYBOARD_KOR                                       12
#define RSSMX_VAL_SYSTEM_KEYBOARD_NORW                                      13
#define RSSMX_VAL_SYSTEM_KEYBOARD_PORT                                      14
#define RSSMX_VAL_SYSTEM_KEYBOARD_RUSS                                      15
#define RSSMX_VAL_SYSTEM_KEYBOARD_SPAN                                      16
#define RSSMX_VAL_SYSTEM_KEYBOARD_SWED                                      17
#define RSSMX_VAL_SYSTEM_KEYBOARD_DUTB                                      18
#define RSSMX_VAL_SYSTEM_KEYBOARD_ENG                                       19

// RsSmx_rngUnitAngle
#define RSSMX_VAL_UNIT_ANGLE_DEG                                            0
#define RSSMX_VAL_UNIT_ANGLE_RAD                                            1

// RsSmx_rngUnitPower
#define RSSMX_VAL_UNIT_POWER_V                                              0
#define RSSMX_VAL_UNIT_POWER_DBM                                            1
#define RSSMX_VAL_UNIT_POWER_DBUV                                           2

// RsSmx_rngSystemIPAddressMode
#define RSSMX_VAL_SYSTEM_COMMUNICATE_IP_MODE_STATIC                         0
#define RSSMX_VAL_SYSTEM_COMMUNICATE_IP_MODE_P2P                            1
#define RSSMX_VAL_SYSTEM_COMMUNICATE_IP_MODE_AUTO                           2

// RsSmx_rngHardCopyLanguage
#define RSSMX_VAL_HCOPY_LANG_BMP                                            0
#define RSSMX_VAL_HCOPY_LANG_JPG                                            1
#define RSSMX_VAL_HCOPY_LANG_XPM                                            2
#define RSSMX_VAL_HCOPY_LANG_PNG                                            3

// RsSmx_rngTriggerSourceForAllSweeps
#define RSSMX_VAL_TRIG_SOURCE_SWEEP_AUTO                                    0
#define RSSMX_VAL_TRIG_SOURCE_SWEEP_SINGLE                                  1
#define RSSMX_VAL_TRIG_SOURCE_SWEEP_EXTERNAL                                2
#define RSSMX_VAL_TRIG_SOURCE_SWEEP_EAUT                                    3

// RsSmx_rngRFFrequencyonRecall
#define RSSMX_VAL_FREQUENCY_RCL_INCLUDE                                     0
#define RSSMX_VAL_FREQUENCY_RCL_EXCLUDE                                     1

// RsSmx_rngROSCSynchronizationBandwidth
#define RSSMX_VAL_ROSC_SBAN_WIDE                                            0
#define RSSMX_VAL_ROSC_SBAN_NARROW                                          1

// RsSmx_rngRFAttenuatorMode
#define RSSMX_VAL_OUTPUT_AMODE_AUTO                                         0
#define RSSMX_VAL_OUTPUT_AMODE_FIXED                                        1

// RsSmx_rngPowerOnState
#define RSSMX_VAL_OUTPUT_STATE_PON_OFF                                      0
#define RSSMX_VAL_OUTPUT_STATE_PON_UNCH                                     1

// RsSmx_rngALCEnabled
#define RSSMX_VAL_POWER_ALC_STATE_OFF                                       0
#define RSSMX_VAL_POWER_ALC_STATE_ON                                        1
#define RSSMX_VAL_POWER_ALC_STATE_AUTO                                      2
#define RSSMX_VAL_POWER_ALC_STATE_OFF_TABLE                                 3
#define RSSMX_VAL_POWER_ALC_STATE_ON_TABLE                                  4
#define RSSMX_VAL_POWER_ALC_STATE_PRESET                                    5
#define RSSMX_VAL_POWER_ALC_STATE_EALC                                      6

// RsSmx_rngSweepMode
#define RSSMX_VAL_SWEEP_MODE_AUTO                                           0
#define RSSMX_VAL_SWEEP_MODE_MANUAL                                         1
#define RSSMX_VAL_SWEEP_MODE_STEP                                           2

// RsSmx_rngFrequencySweepShape
#define RSSMX_VAL_SWEEP_SHAPE_SAW                                           0
#define RSSMX_VAL_SWEEP_SHAPE_TRI                                           1

// RsSmx_rngAMExternalCoupling
#define RSSMX_VAL_AM_EXTERNAL_COUPLING_AC                                   0
#define RSSMX_VAL_AM_EXTERNAL_COUPLING_DC                                   1

// RsSmx_rngPULMPolarity
#define RSSMX_VAL_PULM_POLARITY_NORMAL                                      0
#define RSSMX_VAL_PULM_POLARITY_INVERTED                                    1

// RsSmx_rngPULMSource
#define RSSMX_VAL_PULM_SOURCE_INTERNAL                                      0
#define RSSMX_VAL_PULM_SOURCE_EXTERNAL                                      1

// RsSmx_rngPULMTriggerMode
#define RSSMX_VAL_PULM_TRIGGER_MODE_AUTO                                    0
#define RSSMX_VAL_PULM_TRIGGER_MODE_EXTERNAL                                1
#define RSSMX_VAL_PULM_TRIGGER_MODE_EGATE                                   2
#define RSSMX_VAL_PULM_TRIGGER_MODE_ESINGLE                                 3

// RsSmx_rngFMMode
#define RSSMX_VAL_FM_MODE_HBAN                                              0
#define RSSMX_VAL_FM_MODE_LNOIS                                             1

// RsSmx_rngPMMode
#define RSSMX_VAL_PM_MODE_HBAN                                              0
#define RSSMX_VAL_PM_MODE_LNOIS                                             1
#define RSSMX_VAL_PM_MODE_HDEV                                              2

// RsSmx_rngCorrectionListMode
#define RSSMX_VAL_DEXC_MODE_IMPORT                                          0
#define RSSMX_VAL_DEXC_MODE_EXPORT                                          1

// RsSmx_rngCorrectionASCIIExtension
#define RSSMX_VAL_DEXC_ASCII_SOURCE_FILE_EXT_TXT                            0
#define RSSMX_VAL_DEXC_ASCII_SOURCE_FILE_EXT_CSV                            1

// RsSmx_rngASCIISeparator
#define RSSMX_VAL_COLUMN_SEP_TAB                                            0
#define RSSMX_VAL_COLUMN_SEP_SEM                                            1
#define RSSMX_VAL_COLUMN_SEP_COMMA                                          2
#define RSSMX_VAL_COLUMN_SEP_SPACE                                          3

// RsSmx_rngASCIIDecimalSeparator
#define RSSMX_VAL_DECIMAL_SEP_DOT                                           0
#define RSSMX_VAL_DECIMAL_SEP_COMMA                                         1

// RsSmx_rngListTriggerSource
#define RSSMX_VAL_LIST_TRIGGER_SOURCE_AUTO                                  0
#define RSSMX_VAL_LIST_TRIGGER_SOURCE_SINGLE                                1
#define RSSMX_VAL_LIST_TRIGGER_SOURCE_EXTERNAL                              2

// RsSmx_rngRFLevelVariationActive
#define RSSMX_VAL_RF_VARIATION_ACTIVE_DECIMAL                               0
#define RSSMX_VAL_RF_VARIATION_ACTIVE_USER                                  1

// RsSmx_rngExternalInputImpedance
#define RSSMX_VAL_EXT_INPUT_IMP_G50                                         0
#define RSSMX_VAL_EXT_INPUT_IMP_G600                                        1
#define RSSMX_VAL_EXT_INPUT_IMP_HIGH                                        2

// RsSmx_rngPULMTransitionType
#define RSSMX_VAL_PULM_TRANSITION_TYPE_SMOOTHED                             0
#define RSSMX_VAL_PULM_TRANSITION_TYPE_FAST                                 1

// RsSmx_rngPULMMode
#define RSSMX_VAL_PULSE_MODE_SINGLE                                         0
#define RSSMX_VAL_PULSE_MODE_DOUBLE                                         1
#define RSSMX_VAL_PULSE_MODE_TRAIN                                          2

// RsSmx_rngALCDetectorSensitivity
#define RSSMX_VAL_ALC_DETECTOR_AUTO                                         0
#define RSSMX_VAL_ALC_DETECTOR_FIXED                                        1

// RsSmx_rngRFAttRFOFFMode
#define RSSMX_VAL_RF_OFF_MODE_UNCH                                          0
#define RSSMX_VAL_RF_OFF_MODE_FULL                                          1

// RsSmx_rngLFGenShapeMode
#define RSSMX_VAL_LFG_SHAPE_SINE                                            0
#define RSSMX_VAL_LFG_SHAPE_PULSE                                           1
#define RSSMX_VAL_LFG_SHAPE_TRIANGLE                                        2
#define RSSMX_VAL_LFG_SHAPE_TRAP                                            3

// RsSmx_rngPGENOutputPolarity
#define RSSMX_VAL_PGEN_OUTPUT_POLARITY_NORMAL                               0
#define RSSMX_VAL_PGEN_OUTPUT_POLARITY_INVERTED                             1

// RsSmx_rngRFLevelMode
#define RSSMX_VAL_RF_LEVEL_MODE_NORMAL                                      0
#define RSSMX_VAL_RF_LEVEL_MODE_LOWNOISE                                    1
#define RSSMX_VAL_RF_LEVEL_MODE_LOWDIST                                     2

// RsSmx_rngRFLevelSettingCharacteristics
#define RSSMX_VAL_RF_LEVEL_SETTING_AUTO                                     0
#define RSSMX_VAL_RF_LEVEL_SETTING_UNINT                                    1
#define RSSMX_VAL_RF_LEVEL_SETTING_STRICTLY                                 2
#define RSSMX_VAL_RF_LEVEL_SETTING_VSWR                                     3
#define RSSMX_VAL_RF_LEVEL_SETTING_HDUN                                     4

// RsSmx_rngLFOutputSource
#define RSSMX_VAL_LFOUT_SOURCE_LF1                                          0
#define RSSMX_VAL_LFOUT_SOURCE_LF2                                          1
#define RSSMX_VAL_LFOUT_SOURCE_NOISE                                        2
#define RSSMX_VAL_LFOUT_SOURCE_EXT1                                         3
#define RSSMX_VAL_LFOUT_SOURCE_EXT2                                         4
#define RSSMX_VAL_LFOUT_SOURCE_AM                                           5
#define RSSMX_VAL_LFOUT_SOURCE_FMPM                                         6

// RsSmx_rngRFNoiseDistribution
#define RSSMX_VAL_RF_NOISE_DISTRIBUTION_GAUSS                               0
#define RSSMX_VAL_RF_NOISE_DISTRIBUTION_EQUAL                               1

// RsSmx_rngAMSource
#define RSSMX_VAL_AM_SOURCE_LF1                                             0
#define RSSMX_VAL_AM_SOURCE_LF2                                             1
#define RSSMX_VAL_AM_SOURCE_NOISE                                           2
#define RSSMX_VAL_AM_SOURCE_EXT1                                            3
#define RSSMX_VAL_AM_SOURCE_EXT2                                            4

// RsSmx_rngFMSource
#define RSSMX_VAL_FM_SOURCE_LF1                                             0
#define RSSMX_VAL_FM_SOURCE_LF2                                             1
#define RSSMX_VAL_FM_SOURCE_NOISE                                           2
#define RSSMX_VAL_FM_SOURCE_EXT1                                            3
#define RSSMX_VAL_FM_SOURCE_EXT2                                            4

// RsSmx_rngPMSource
#define RSSMX_VAL_PM_SOURCE_LF1                                             0
#define RSSMX_VAL_PM_SOURCE_LF2                                             1
#define RSSMX_VAL_PM_SOURCE_NOISE                                           2
#define RSSMX_VAL_PM_SOURCE_EXT1                                            3
#define RSSMX_VAL_PM_SOURCE_EXT2                                            4

// RsSmx_rngRefOscOutputFreqMode
#define RSSMX_VAL_REFERENCE_OSCILLATOR_FREQ_MODE_OFF                        0
#define RSSMX_VAL_REFERENCE_OSCILLATOR_FREQ_MODE_DER10M                     1
#define RSSMX_VAL_REFERENCE_OSCILLATOR_FREQ_MODE_DER100M                    2
#define RSSMX_VAL_REFERENCE_OSCILLATOR_FREQ_MODE_LOOPTHROUGH                3

// RsSmx_rngPULMImpedance
#define RSSMX_VAL_PULM_IMPEDANCE_G50                                        0
#define RSSMX_VAL_PULM_IMPEDANCE_G10K                                       1

// RsSmx_rngSystemSerialParity
#define RSSMX_VAL_SERIAL_PARITY_NONE                                        0
#define RSSMX_VAL_SERIAL_PARITY_ODD                                         1
#define RSSMX_VAL_SERIAL_PARITY_EVEN                                        2

// RsSmx_rngPowerDisplayPermanent
#define RSSMX_VAL_SENSE_POWER_DISPLAY_AVERAGE                               0
#define RSSMX_VAL_SENSE_POWER_DISPLAY_PEAK                                  1

// RsSmx_rngListRunMode
#define RSSMX_VAL_LIST_RUN_LEARNED                                          0
#define RSSMX_VAL_LIST_RUN_LIVE                                             1

// RsSmx_rngListDwellTimeMode
#define RSSMX_VAL_DWELL_MODE_GLOBAL                                         0
#define RSSMX_VAL_DWELL_MODE_LIST                                           1

// RsSmx_rngExternalInputCouplingMode
#define RSSMX_VAL_COUPLING_AC                                               0
#define RSSMX_VAL_COUPLING_DC                                               1

// RsSmx_rngHardCopyRegion
#define RSSMX_VAL_HCOPY_SNAPSHOT_ALL                                        0
#define RSSMX_VAL_HCOPY_SNAPSHOT_DIALOG                                     1

// RsSmx_rngRFFrequencyMode
#define RSSMX_VAL_RF_FREQUENCY_CW                                           0
#define RSSMX_VAL_RF_FREQUENCY_SWEEP                                        1
#define RSSMX_VAL_RF_FREQUENCY_LIST                                         2
#define RSSMX_VAL_RF_FREQUENCY_COMBINED                                     3

// RsSmx_rngSweepTriggerSource
#define RSSMX_VAL_SWEEP_TRIGGER_SOURCE_AUTO                                 0
#define RSSMX_VAL_SWEEP_TRIGGER_SOURCE_SINGLE                               1
#define RSSMX_VAL_SWEEP_TRIGGER_SOURCE_EXTERNAL                             2
#define RSSMX_VAL_SWEEP_TRIGGER_SOURCE_EXTERNAL_AUTO                        3

// RsSmx_rngMode
#define RSSMX_VAL_MODE_CW                                                   0
#define RSSMX_VAL_MODE_SWEEP                                                1

// RsSmx_rngRFPowerMode
#define RSSMX_VAL_RF_POWER_CW                                               0
#define RSSMX_VAL_RF_POWER_SWEEP                                            1

// RsSmx_rngOutputImpedance
#define RSSMX_VAL_IMPEDANCE_G50                                             0
#define RSSMX_VAL_IMPEDANCE_G1K                                             1
#define RSSMX_VAL_IMPEDANCE_G10K                                            2

// RsSmx_rngLFOutputSourcePath
#define RSSMX_PATH_A                                                        0
#define RSSMX_PATH_B                                                        1

// RsSmx_rngPowerSensorMapping
#define RSSMX_VAL_POWER_SENSOR_MAPPING_UNMAPPED                             0
#define RSSMX_VAL_POWER_SENSOR_MAPPING_SENSOR1                              1
#define RSSMX_VAL_POWER_SENSOR_MAPPING_SENSOR2                              2
#define RSSMX_VAL_POWER_SENSOR_MAPPING_SENSOR3                              3
#define RSSMX_VAL_POWER_SENSOR_MAPPING_SENSOR4                              4

// RsSmx_rngCalibrationLevelBandwidth
#define RSSMX_VAL_CAL_BWIDTH_LOW                                            0
#define RSSMX_VAL_CAL_BWIDTH_HIGH                                           1
#define RSSMX_VAL_CAL_BWIDTH_AUTO                                           2

// RsSmx_rngCalibrationLevelCalibrationData
#define RSSMX_VAL_LEVEL_CAL_FACTORY                                         0
#define RSSMX_VAL_LEVEL_CAL_CUSTOMER                                        1

// RsSmx_rngRFFrequencyMainPLLBandwidth
#define RSSMX_VAL_PLL_NORMAL                                                0
#define RSSMX_VAL_PLL_NARROW                                                1

// RsSmx_rngRFLevelHarmonicFilter
#define RSSMX_VAL_HARMONIC_FILTER_AUTO                                      0
#define RSSMX_VAL_HARMONIC_FILTER_ON                                        1

// RsSmx_rngClockSynthesisOutputType
#define RSSMX_VAL_CSYN_OTYPE_SES                                            0
#define RSSMX_VAL_CSYN_OTYPE_DSIN                                           1
#define RSSMX_VAL_CSYN_OTYPE_DSQ                                            2
#define RSSMX_VAL_CSYN_OTYPE_CMOS                                           3

// RsSmx_rngSelftestResult
#define RSSMX_VAL_SELFTEST_OK                                               0
#define RSSMX_VAL_SELFTEST_NOK                                              1
#define RSSMX_VAL_SELFTEST_RUNNING                                          2
#define RSSMX_VAL_SELFTEST_STOPPED                                          3

// RsSmx_rngReferenceOscillatorVoltageTuningSensitivity
#define RSSMX_VAL_SENSITIVITY_LOW                                           0
#define RSSMX_VAL_SENSITIVITY_HIGH                                          1

// RsSmx_rngSystemUserInterfaceLock
#define RSSMX_VAL_LOC_DISABLED                                              0
#define RSSMX_VAL_LOC_ENABLED                                               1
#define RSSMX_VAL_LOC_DISPLAY_ONLY                                          2
#define RSSMX_VAL_LOC_VNC_ONLY                                              3
#define RSSMX_VAL_LOC_TOUCH_OFF                                             4

// RsSmx_rngListMode
#define RSSMX_VAL_LIST_MODE_AUTO                                            0
#define RSSMX_VAL_LIST_MODE_STEP                                            1

// RsSmx_rngLFOutputExternalLFBandwidth
#define RSSMX_VAL_BWID_BW0M2                                                0
#define RSSMX_VAL_BWID_BW10M                                                1

// RsSmx_rngReferenceOscillatorOutputReferenceFrequency
#define RSSMX_VAL_ROSC_OUTPUT_FMODE_OFF                                     0
#define RSSMX_VAL_ROSC_OUTPUT_FMODE_DER1G                                   1
#define RSSMX_VAL_ROSC_OUTPUT_FMODE_LOOP                                    2

// RsSmx_rngPowerAnalysisTimingMode
#define RSSMX_VAL_TIMING_NORMAL                                             0
#define RSSMX_VAL_TIMING_FAST                                               1
#define RSSMX_VAL_TIMING_HIGH_PRECISION                                     2

// RsSmx_rngPowerAnalysisAutoScale
#define RSSMX_VAL_AUTO_SCALE_OFF                                            0
#define RSSMX_VAL_AUTO_SCALE_COARSE_EXPANDING                               1
#define RSSMX_VAL_AUTO_SCALE_FINE_EXPANDING                                 2
#define RSSMX_VAL_AUTO_SCALE_COARSE_FLOATING                                3
#define RSSMX_VAL_AUTO_SCALE_FINE_FLOATING                                  4

// RsSmx_rngDeviationMode
#define RSSMX_VAL_DEV_MODE_UNCOUPLED                                        0
#define RSSMX_VAL_DEV_MODE_TOTAL                                            1
#define RSSMX_VAL_DEV_MODE_RATIO                                            2

// RsSmx_rngPhaseContinuityMode
#define RSSMX_VAL_PHASE_CONT_NARROW                                         0
#define RSSMX_VAL_PHASE_CONT_WIDE                                           1

// RsSmx_rngILSLocalizerTimeSchema
#define RSSMX_VAL_ILS_LOCALIZER_TIME_SCHEMA_STD                             0
#define RSSMX_VAL_ILS_LOCALIZER_TIME_SCHEMA_USER                            1

// RsSmx_rngILSLocalizerFlyMode
#define RSSMX_VAL_ILS_LOCALIZER_FLY_MODE_LEFT                               0
#define RSSMX_VAL_ILS_LOCALIZER_FLY_MODE_RIGHT                              1

// RsSmx_rngILSLocalizerEXTAM
#define RSSMX_VAL_ILS_LOCALIZER_EXT_AM_INT                                  0
#define RSSMX_VAL_ILS_LOCALIZER_EXT_AM_EXT                                  1

// RsSmx_rngILSGSFlyMode
#define RSSMX_VAL_ILS_GS_FLY_MODE_UP                                        0
#define RSSMX_VAL_ILS_GS_FLY_MODE_DOWN                                      1

// RsSmx_rngILSGSOperatingMode
#define RSSMX_VAL_ILS_GS_OPERATING_MODE_NORM                                0
#define RSSMX_VAL_ILS_GS_OPERATING_MODE_ULOBE                               1
#define RSSMX_VAL_ILS_GS_OPERATING_MODE_LLOBE                               2

// RsSmx_rngILSGSEXTAM
#define RSSMX_VAL_ILS_GS_EXT_AM_INT                                         0
#define RSSMX_VAL_ILS_GS_EXT_AM_EXT                                         1

// RsSmx_rngSweepType
#define RSSMX_VAL_SWEEP_TYPE_STEPPED                                        0
#define RSSMX_VAL_SWEEP_TYPE_ANALOG                                         1

// RsSmx_rngMarkerPolarity
#define RSSMX_VAL_MARKER_POLARITY_NORMAL                                    0
#define RSSMX_VAL_MARKER_POLARITY_INVERTED                                  1

// RsSmx_rngVORTimeSchema
#define RSSMX_VAL_VOR_TIME_SCHEMA_STD                                       0
#define RSSMX_VAL_VOR_TIME_SCHEMA_USER                                      1

// RsSmx_rngVORMode
#define RSSMX_VAL_VOR_MODE_NORM                                             0
#define RSSMX_VAL_VOR_MODE_VAR                                              1
#define RSSMX_VAL_VOR_MODE_SUBCARRIER                                       2
#define RSSMX_VAL_VOR_MODE_FM_SUBCARRIER                                    3

// RsSmx_rngVOREXTAM
#define RSSMX_VAL_VOR_EXT_AM_INT                                            0
#define RSSMX_VAL_VOR_EXT_AM_EXT                                            1

// RsSmx_rngVORDirection
#define RSSMX_VAL_VOR_DIRECTION_FROM                                        0
#define RSSMX_VAL_VOR_DIRECTION_TO                                          1

// RsSmx_rngILSMBTimeSchema
#define RSSMX_VAL_ILS_MB_TIME_SCHEMA_STD                                    0
#define RSSMX_VAL_ILS_MB_TIME_SCHEMA_USER                                   1

// RsSmx_rngPowerAnalysisTraceState
#define RSSMX_VAL_TRACE_STATE_OFF                                           0
#define RSSMX_VAL_TRACE_STATE_ON                                            1
#define RSSMX_VAL_TRACE_STATE_HOLD                                          2

// RsSmx_rngPowerAnalysisTraceSource
#define RSSMX_VAL_TRACE_SOURCE_NONE                                         0
#define RSSMX_VAL_TRACE_SOURCE_SENS1                                        1
#define RSSMX_VAL_TRACE_SOURCE_SENS2                                        2
#define RSSMX_VAL_TRACE_SOURCE_SENS3                                        3
#define RSSMX_VAL_TRACE_SOURCE_SENS4                                        4

// RsSmx_rngPowerAnalysisTraceColor
#define RSSMX_VAL_TRACE_COLOR_INVERSE                                       0
#define RSSMX_VAL_TRACE_COLOR_GRAY                                          1
#define RSSMX_VAL_TRACE_COLOR_YELLOW                                        2
#define RSSMX_VAL_TRACE_COLOR_BLUE                                          3
#define RSSMX_VAL_TRACE_COLOR_GREEN                                         4
#define RSSMX_VAL_TRACE_COLOR_RED                                           5
#define RSSMX_VAL_TRACE_COLOR_MAGENTA                                       6

// RsSmx_rngPowerAnalysisMathFrequencySubtract
#define RSSMX_VAL_TRACE_SUBTRACT_T1T1                                       0
#define RSSMX_VAL_TRACE_SUBTRACT_T1T2                                       1
#define RSSMX_VAL_TRACE_SUBTRACT_T1T3                                       2
#define RSSMX_VAL_TRACE_SUBTRACT_T1T4                                       3
#define RSSMX_VAL_TRACE_SUBTRACT_T1REF                                      4
#define RSSMX_VAL_TRACE_SUBTRACT_T2T1                                       5
#define RSSMX_VAL_TRACE_SUBTRACT_T2T2                                       6
#define RSSMX_VAL_TRACE_SUBTRACT_T2T3                                       7
#define RSSMX_VAL_TRACE_SUBTRACT_T2T4                                       8
#define RSSMX_VAL_TRACE_SUBTRACT_T2REF                                      9
#define RSSMX_VAL_TRACE_SUBTRACT_T3T1                                       10
#define RSSMX_VAL_TRACE_SUBTRACT_T3T2                                       11
#define RSSMX_VAL_TRACE_SUBTRACT_T3T3                                       12
#define RSSMX_VAL_TRACE_SUBTRACT_T3T4                                       13
#define RSSMX_VAL_TRACE_SUBTRACT_T3REF                                      14
#define RSSMX_VAL_TRACE_SUBTRACT_T4T1                                       15
#define RSSMX_VAL_TRACE_SUBTRACT_T4T2                                       16
#define RSSMX_VAL_TRACE_SUBTRACT_T4T3                                       17
#define RSSMX_VAL_TRACE_SUBTRACT_T4T4                                       18
#define RSSMX_VAL_TRACE_SUBTRACT_T4REF                                      19

// RsSmx_rngPowerAnalysisMeasurementMode
#define RSSMX_VAL_MEASUREMENT_MODE_FREQ                                     0
#define RSSMX_VAL_MEASUREMENT_MODE_POWER                                    1
#define RSSMX_VAL_MEASUREMENT_MODE_TIME                                     2

// RsSmx_rngPowerAnalysisMeasurementExecution
#define RSSMX_VAL_EXECUTION_SINGLE                                          0
#define RSSMX_VAL_EXECUTION_CONTINUOUS                                      1

// RsSmx_rngPowerAnalysisTimeTriggerEvents
#define RSSMX_VAL_TIME_TRIGGER_AND                                          0
#define RSSMX_VAL_TIME_TRIGGER_OR                                           1

// RsSmx_rngPowerAnalysisDiagramBackgroundColor
#define RSSMX_VAL_BACKGROUND_BLACK                                          0
#define RSSMX_VAL_BACKGROUND_WHITE                                          1

// RsSmx_rngPowerAnalysisGateTrace
#define RSSMX_VAL_GATE_TRACE1                                               1
#define RSSMX_VAL_GATE_TRACE2                                               2
#define RSSMX_VAL_GATE_TRACE3                                               3
#define RSSMX_VAL_GATE_TRACE4                                               4

// RsSmx_rngPowerAnalysisPulseDataBase
#define RSSMX_VAL_POWER_ANALYSIS_PULSE_BASE_VOLTAGE                         0
#define RSSMX_VAL_POWER_ANALYSIS_PULSE_BASE_POWER                           1

// RsSmx_rngPowerAnalysisTriggerMode
#define RSSMX_VAL_POWER_ANALYSIS_TRIGGER_FREE                               0
#define RSSMX_VAL_POWER_ANALYSIS_TRIGGER_AUTO                               1
#define RSSMX_VAL_POWER_ANALYSIS_TRIGGER_INT                                2
#define RSSMX_VAL_POWER_ANALYSIS_TRIGGER_EXT                                3

// RsSmx_rngSweepModeOutputVoltageMode
#define RSSMX_VAL_SWEEP_OUTPUT_VOLTAGE_S0V25                                0
#define RSSMX_VAL_SWEEP_OUTPUT_VOLTAGE_S0V5                                 1
#define RSSMX_VAL_SWEEP_OUTPUT_VOLTAGE_S1V0                                 2
#define RSSMX_VAL_SWEEP_OUTPUT_VOLTAGE_XAXIS                                3

// RsSmx_rngSweepModeOutputMarkerMode
#define RSSMX_VAL_MARKER_MODE_MARKER                                        0
#define RSSMX_VAL_MARKER_MODE_USER                                          1

// RsSmx_rngAMMode
#define RSSMX_VAL_AM_MODE_NORMAL                                            0
#define RSSMX_VAL_AM_MODE_SCAN                                              1

// RsSmx_rngAMType
#define RSSMX_VAL_AM_TYPE_LINEAR                                            0
#define RSSMX_VAL_AM_TYPE_EXPONENTIAL                                       1

// RsSmx_rngSweepActiveMarker
#define RSSMX_VAL_MARKER_NONE                                               0
#define RSSMX_VAL_MARKER_M01                                                1
#define RSSMX_VAL_MARKER_M02                                                2
#define RSSMX_VAL_MARKER_M03                                                3
#define RSSMX_VAL_MARKER_M04                                                4
#define RSSMX_VAL_MARKER_M05                                                5
#define RSSMX_VAL_MARKER_M06                                                6
#define RSSMX_VAL_MARKER_M07                                                7
#define RSSMX_VAL_MARKER_M08                                                8
#define RSSMX_VAL_MARKER_M09                                                9
#define RSSMX_VAL_MARKER_M10                                                10

// RsSmx_rngScreenshotDevice
#define RSSMX_VAL_SCREENSHOT_DEVICE_FILE                                    0
#define RSSMX_VAL_SCREENSHOT_DEVICE_PRINTER                                 1

// RsSmx_rngScreenshotSize
#define RSSMX_VAL_SCREENSHOT_SIZE_320_240                                   0
#define RSSMX_VAL_SCREENSHOT_SIZE_640_480                                   1
#define RSSMX_VAL_SCREENSHOT_SIZE_800_600                                   2
#define RSSMX_VAL_SCREENSHOT_SIZE_1024_768                                  3

// RsSmx_rngScreenshotFormat
#define RSSMX_VAL_SCREENSHOT_FORMAT_BMP                                     0
#define RSSMX_VAL_SCREENSHOT_FORMAT_JPG                                     1
#define RSSMX_VAL_SCREENSHOT_FORMAT_XPM                                     2
#define RSSMX_VAL_SCREENSHOT_FORMAT_PNG                                     3
#define RSSMX_VAL_SCREENSHOT_FORMAT_CSV                                     4

// RsSmx_rngScreenshotCSVHeaderEnabled
#define RSSMX_VAL_CSV_HEADER_OFF                                            0
#define RSSMX_VAL_CSV_HEADER_STANDARD                                       1

// RsSmx_rngScreenshotCSVOrientation
#define RSSMX_VAL_CSV_ORIENTATION_VERTICAL                                  0
#define RSSMX_VAL_CSV_ORIENTATION_HORIZONTAL                                1

// RsSmx_rngScreenshotCSVColumnSeparator
#define RSSMX_VAL_CSV_COL_SEPARATOR_COMMA                                   0
#define RSSMX_VAL_CSV_COL_SEPARATOR_TAB                                     1
#define RSSMX_VAL_CSV_COL_SEPARATOR_SEMICOLON                               2
#define RSSMX_VAL_CSV_COL_SEPARATOR_BLANK                                   3

// RsSmx_rngScreenshotCSVDecimalSeparator
#define RSSMX_VAL_CSV_DEC_SEPARATOR_COMMA                                   0
#define RSSMX_VAL_CSV_DEC_SEPARATOR_DOT                                     1

// RsSmx_rngVORCarrierFrequencyMode
#define RSSMX_VAL_VOR_FREQ_MODE_USER                                        0
#define RSSMX_VAL_VOR_FREQ_MODE_ICAO                                        1

// RsSmx_rngVORICAOChannel
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH17X                                    0
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH17Y                                    1
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH19X                                    2
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH19Y                                    3
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH21X                                    4
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH21Y                                    5
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH23X                                    6
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH23Y                                    7
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH25X                                    8
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH25Y                                    9
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH27X                                    10
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH27Y                                    11
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH29X                                    12
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH29Y                                    13
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH31X                                    14
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH31Y                                    15
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH33X                                    16
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH33Y                                    17
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH35X                                    18
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH35Y                                    19
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH37X                                    20
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH37Y                                    21
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH39X                                    22
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH39Y                                    23
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH41X                                    24
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH41Y                                    25
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH43X                                    26
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH43Y                                    27
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH45X                                    28
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH45Y                                    29
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH47X                                    30
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH47Y                                    31
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH49X                                    32
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH49Y                                    33
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH51X                                    34
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH51Y                                    35
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH53X                                    36
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH53Y                                    37
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH55X                                    38
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH55Y                                    39
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH57X                                    40
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH57Y                                    41
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH58X                                    42
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH58Y                                    43
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH59X                                    44
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH59Y                                    45
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH70X                                    46
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH70Y                                    47
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH71X                                    48
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH71Y                                    49
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH72X                                    50
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH72Y                                    51
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH73X                                    52
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH73Y                                    53
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH74X                                    54
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH74Y                                    55
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH75X                                    56
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH75Y                                    57
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH76X                                    58
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH76Y                                    59
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH77X                                    60
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH77Y                                    61
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH78X                                    62
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH78Y                                    63
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH79X                                    64
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH79Y                                    65
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH80X                                    66
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH80Y                                    67
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH81X                                    68
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH81Y                                    69
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH82X                                    70
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH82Y                                    71
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH83X                                    72
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH83Y                                    73
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH84X                                    74
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH84Y                                    75
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH85X                                    76
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH85Y                                    77
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH86X                                    78
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH86Y                                    79
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH87X                                    80
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH87Y                                    81
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH88X                                    82
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH88Y                                    83
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH89X                                    84
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH89Y                                    85
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH90X                                    86
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH90Y                                    87
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH91X                                    88
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH91Y                                    89
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH92X                                    90
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH92Y                                    91
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH93X                                    92
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH93Y                                    93
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH94X                                    94
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH94Y                                    95
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH95X                                    96
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH95Y                                    97
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH96X                                    98
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH96Y                                    99
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH97X                                    100
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH97Y                                    101
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH98X                                    102
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH98Y                                    103
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH99X                                    104
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH99Y                                    105
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH100X                                   106
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH100Y                                   107
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH101X                                   108
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH101Y                                   109
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH102X                                   110
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH102Y                                   111
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH103X                                   112
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH103Y                                   113
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH104X                                   114
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH104Y                                   115
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH105X                                   116
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH105Y                                   117
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH106X                                   118
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH106Y                                   119
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH107X                                   120
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH107Y                                   121
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH108X                                   122
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH108Y                                   123
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH109X                                   124
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH109Y                                   125
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH110X                                   126
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH110Y                                   127
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH111X                                   128
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH111Y                                   129
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH112X                                   130
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH112Y                                   131
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH113X                                   132
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH113Y                                   133
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH114X                                   134
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH114Y                                   135
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH115X                                   136
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH115Y                                   137
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH116X                                   138
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH116Y                                   139
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH117X                                   140
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH117Y                                   141
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH118X                                   142
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH118Y                                   143
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH119X                                   144
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH119Y                                   145
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH120X                                   146
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH120Y                                   147
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH121X                                   148
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH121Y                                   149
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH122X                                   150
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH122Y                                   151
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH123X                                   152
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH123Y                                   153
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH124X                                   154
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH124Y                                   155
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH125X                                   156
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH125Y                                   157
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH126X                                   158
#define RSSMX_VAL_VOR_ICAO_CHANNEL_CH126Y                                   159

// RsSmx_rngILSComponent
#define RSSMX_VAL_ILS_COMPONENT_GLIDE_SLOPE                                 0
#define RSSMX_VAL_ILS_COMPONENT_LOCALIZER                                   1
#define RSSMX_VAL_ILS_COMPONENT_MARKER_BEACONS                              2

// RsSmx_rngILSGSCarrierFrequencyMode
#define RSSMX_VAL_ILS_GS_FREQ_MODE_USER                                     0
#define RSSMX_VAL_ILS_GS_FREQ_MODE_ICAO                                     1

// RsSmx_rngILSGSICAOChannel
#define RSSMX_VAL_ILS_GS_ICAO_CHANNEL_CH18X                                 0
#define RSSMX_VAL_ILS_GS_ICAO_CHANNEL_CH18Y                                 1
#define RSSMX_VAL_ILS_GS_ICAO_CHANNEL_CH20X                                 2
#define RSSMX_VAL_ILS_GS_ICAO_CHANNEL_CH20Y                                 3
#define RSSMX_VAL_ILS_GS_ICAO_CHANNEL_CH22X                                 4
#define RSSMX_VAL_ILS_GS_ICAO_CHANNEL_CH22Y                                 5
#define RSSMX_VAL_ILS_GS_ICAO_CHANNEL_CH24X                                 6
#define RSSMX_VAL_ILS_GS_ICAO_CHANNEL_CH24Y                                 7
#define RSSMX_VAL_ILS_GS_ICAO_CHANNEL_CH26X                                 8
#define RSSMX_VAL_ILS_GS_ICAO_CHANNEL_CH26Y                                 9
#define RSSMX_VAL_ILS_GS_ICAO_CHANNEL_CH28X                                 10
#define RSSMX_VAL_ILS_GS_ICAO_CHANNEL_CH28Y                                 11
#define RSSMX_VAL_ILS_GS_ICAO_CHANNEL_CH30X                                 12
#define RSSMX_VAL_ILS_GS_ICAO_CHANNEL_CH30Y                                 13
#define RSSMX_VAL_ILS_GS_ICAO_CHANNEL_CH32X                                 14
#define RSSMX_VAL_ILS_GS_ICAO_CHANNEL_CH32Y                                 15
#define RSSMX_VAL_ILS_GS_ICAO_CHANNEL_CH34X                                 16
#define RSSMX_VAL_ILS_GS_ICAO_CHANNEL_CH34Y                                 17
#define RSSMX_VAL_ILS_GS_ICAO_CHANNEL_CH36X                                 18
#define RSSMX_VAL_ILS_GS_ICAO_CHANNEL_CH36Y                                 19
#define RSSMX_VAL_ILS_GS_ICAO_CHANNEL_CH38X                                 20
#define RSSMX_VAL_ILS_GS_ICAO_CHANNEL_CH38Y                                 21
#define RSSMX_VAL_ILS_GS_ICAO_CHANNEL_CH40X                                 22
#define RSSMX_VAL_ILS_GS_ICAO_CHANNEL_CH40Y                                 23
#define RSSMX_VAL_ILS_GS_ICAO_CHANNEL_CH42X                                 24
#define RSSMX_VAL_ILS_GS_ICAO_CHANNEL_CH42Y                                 25
#define RSSMX_VAL_ILS_GS_ICAO_CHANNEL_CH44X                                 26
#define RSSMX_VAL_ILS_GS_ICAO_CHANNEL_CH44Y                                 27
#define RSSMX_VAL_ILS_GS_ICAO_CHANNEL_CH46X                                 28
#define RSSMX_VAL_ILS_GS_ICAO_CHANNEL_CH46Y                                 29
#define RSSMX_VAL_ILS_GS_ICAO_CHANNEL_CH48X                                 30
#define RSSMX_VAL_ILS_GS_ICAO_CHANNEL_CH48Y                                 31
#define RSSMX_VAL_ILS_GS_ICAO_CHANNEL_CH50X                                 32
#define RSSMX_VAL_ILS_GS_ICAO_CHANNEL_CH50Y                                 33
#define RSSMX_VAL_ILS_GS_ICAO_CHANNEL_CH52X                                 34
#define RSSMX_VAL_ILS_GS_ICAO_CHANNEL_CH52Y                                 35
#define RSSMX_VAL_ILS_GS_ICAO_CHANNEL_CH54X                                 36
#define RSSMX_VAL_ILS_GS_ICAO_CHANNEL_CH54Y                                 37
#define RSSMX_VAL_ILS_GS_ICAO_CHANNEL_CH56X                                 38
#define RSSMX_VAL_ILS_GS_ICAO_CHANNEL_CH56Y                                 39

// RsSmx_rngILSGSDDMPolarity
#define RSSMX_VAL_ILS_GS_DDM_POLARITY_P90_150                               0
#define RSSMX_VAL_ILS_GS_DDM_POLARITY_P150_90                               1

// RsSmx_rngILSGSDDMStep
#define RSSMX_VAL_ILS_GS_DDM_STEP_DEC                                       0
#define RSSMX_VAL_ILS_GS_DDM_STEP_PRED                                      1

// RsSmx_rngILSGSDDMCoupling
#define RSSMX_VAL_ILS_GS_DDM_COUPLING_FIX                                   0
#define RSSMX_VAL_ILS_GS_DDM_COUPLING_SDM                                   1

// RsSmx_rngILSLocalizerCarrierFrequencyMode
#define RSSMX_VAL_ILS_LOCALIZER_FREQ_MODE_USER                              0
#define RSSMX_VAL_ILS_LOCALIZER_FREQ_MODE_ICAO                              1

// RsSmx_rngILSLocalizerICAOChannel
#define RSSMX_VAL_ILS_LOCALIZER_ICAO_CHANNEL_CH18X                          0
#define RSSMX_VAL_ILS_LOCALIZER_ICAO_CHANNEL_CH18Y                          1
#define RSSMX_VAL_ILS_LOCALIZER_ICAO_CHANNEL_CH20X                          2
#define RSSMX_VAL_ILS_LOCALIZER_ICAO_CHANNEL_CH20Y                          3
#define RSSMX_VAL_ILS_LOCALIZER_ICAO_CHANNEL_CH22X                          4
#define RSSMX_VAL_ILS_LOCALIZER_ICAO_CHANNEL_CH22Y                          5
#define RSSMX_VAL_ILS_LOCALIZER_ICAO_CHANNEL_CH24X                          6
#define RSSMX_VAL_ILS_LOCALIZER_ICAO_CHANNEL_CH24Y                          7
#define RSSMX_VAL_ILS_LOCALIZER_ICAO_CHANNEL_CH26X                          8
#define RSSMX_VAL_ILS_LOCALIZER_ICAO_CHANNEL_CH26Y                          9
#define RSSMX_VAL_ILS_LOCALIZER_ICAO_CHANNEL_CH28X                          10
#define RSSMX_VAL_ILS_LOCALIZER_ICAO_CHANNEL_CH28Y                          11
#define RSSMX_VAL_ILS_LOCALIZER_ICAO_CHANNEL_CH30X                          12
#define RSSMX_VAL_ILS_LOCALIZER_ICAO_CHANNEL_CH30Y                          13
#define RSSMX_VAL_ILS_LOCALIZER_ICAO_CHANNEL_CH32X                          14
#define RSSMX_VAL_ILS_LOCALIZER_ICAO_CHANNEL_CH32Y                          15
#define RSSMX_VAL_ILS_LOCALIZER_ICAO_CHANNEL_CH34X                          16
#define RSSMX_VAL_ILS_LOCALIZER_ICAO_CHANNEL_CH34Y                          17
#define RSSMX_VAL_ILS_LOCALIZER_ICAO_CHANNEL_CH36X                          18
#define RSSMX_VAL_ILS_LOCALIZER_ICAO_CHANNEL_CH36Y                          19
#define RSSMX_VAL_ILS_LOCALIZER_ICAO_CHANNEL_CH38X                          20
#define RSSMX_VAL_ILS_LOCALIZER_ICAO_CHANNEL_CH38Y                          21
#define RSSMX_VAL_ILS_LOCALIZER_ICAO_CHANNEL_CH40X                          22
#define RSSMX_VAL_ILS_LOCALIZER_ICAO_CHANNEL_CH40Y                          23
#define RSSMX_VAL_ILS_LOCALIZER_ICAO_CHANNEL_CH42X                          24
#define RSSMX_VAL_ILS_LOCALIZER_ICAO_CHANNEL_CH42Y                          25
#define RSSMX_VAL_ILS_LOCALIZER_ICAO_CHANNEL_CH44X                          26
#define RSSMX_VAL_ILS_LOCALIZER_ICAO_CHANNEL_CH44Y                          27
#define RSSMX_VAL_ILS_LOCALIZER_ICAO_CHANNEL_CH46X                          28
#define RSSMX_VAL_ILS_LOCALIZER_ICAO_CHANNEL_CH46Y                          29
#define RSSMX_VAL_ILS_LOCALIZER_ICAO_CHANNEL_CH48X                          30
#define RSSMX_VAL_ILS_LOCALIZER_ICAO_CHANNEL_CH48Y                          31
#define RSSMX_VAL_ILS_LOCALIZER_ICAO_CHANNEL_CH50X                          32
#define RSSMX_VAL_ILS_LOCALIZER_ICAO_CHANNEL_CH50Y                          33
#define RSSMX_VAL_ILS_LOCALIZER_ICAO_CHANNEL_CH52X                          34
#define RSSMX_VAL_ILS_LOCALIZER_ICAO_CHANNEL_CH52Y                          35
#define RSSMX_VAL_ILS_LOCALIZER_ICAO_CHANNEL_CH54X                          36
#define RSSMX_VAL_ILS_LOCALIZER_ICAO_CHANNEL_CH54Y                          37
#define RSSMX_VAL_ILS_LOCALIZER_ICAO_CHANNEL_CH56X                          38
#define RSSMX_VAL_ILS_LOCALIZER_ICAO_CHANNEL_CH56Y                          39

// RsSmx_rngILSLocalizerOperatingMode
#define RSSMX_VAL_ILS_LOCALIZER_MODE_NORM                                   0
#define RSSMX_VAL_ILS_LOCALIZER_MODE_LLOB                                   1
#define RSSMX_VAL_ILS_LOCALIZER_MODE_RLOB                                   2

// RsSmx_rngILSLocalizerDDMPolarity
#define RSSMX_VAL_ILS_LOCALIZER_DDM_POLARITY_P90_150                        0
#define RSSMX_VAL_ILS_LOCALIZER_DDM_POLARITY_P150_90                        1

// RsSmx_rngILSLocalizerDDMCoupling
#define RSSMX_VAL_ILS_LOCALIZER_DDM_COUPLING_FIX                            0
#define RSSMX_VAL_ILS_LOCALIZER_DDM_COUPLING_SDM                            1

// RsSmx_rngILSLocalizerDDMStep
#define RSSMX_VAL_ILS_LOCALIZER_DDM_STEP_DEC                                0
#define RSSMX_VAL_ILS_LOCALIZER_DDM_STEP_PRED                               1

// RsSmx_rngILSMBCarrierFrequencyMode
#define RSSMX_ILS_MB_CARRIER_FREQ_MODE_USER                                 0
#define RSSMX_ILS_MB_CARRIER_FREQ_MODE_PRED                                 1

// RsSmx_rngILSMBMarkerFrequency
#define RSSMX_VAL_ILS_MB_MARKER_FREQ_400                                    0
#define RSSMX_VAL_ILS_MB_MARKER_FREQ_1300                                   1
#define RSSMX_VAL_ILS_MB_MARKER_FREQ_3000                                   2

// RsSmx_rngADFTimeSchema
#define RSSMX_VAL_ADF_TIME_SCHEMA_STD                                       0
#define RSSMX_VAL_ADF_TIME_SCHEMA_USER                                      1

// RsSmx_rngChirpModulationDirection
#define RSSMX_VAL_CHIRP_MODULATION_DOWN                                     0
#define RSSMX_VAL_CHIRP_MODULATION_UP                                       1

// RsSmx_rngChirpModulationTriggerMode
#define RSSMX_VAL_CHIRP_MODULATION_TRIGGER_MODE_AUTO                        0
#define RSSMX_VAL_CHIRP_MODULATION_TRIGGER_MODE_EXTERNAL                    1
#define RSSMX_VAL_CHIRP_MODULATION_TRIGGER_MODE_EGATE                       2
#define RSSMX_VAL_CHIRP_MODULATION_TRIGGER_MODE_SINGLE                      3
#define RSSMX_VAL_CHIRP_MODULATION_TRIGGER_MODE_ESINGLE                     4

// RsSmx_rngRFCombinedSweepTriggerSource
#define RSSMX_VAL_RF_COMBINED_SWEEP_TRIGGER_SOURCE_AUTO                     0
#define RSSMX_VAL_RF_COMBINED_SWEEP_TRIGGER_SOURCE_SINGLE                   1
#define RSSMX_VAL_RF_COMBINED_SWEEP_TRIGGER_SOURCE_EXTERNAL                 2
#define RSSMX_VAL_RF_COMBINED_SWEEP_TRIGGER_SOURCE_EAUTO                    3

// RsSmx_rngRFCombinedSweepMode
#define RSSMX_VAL_RF_COMBINED_SWEEP_MODE_AUTO                               0
#define RSSMX_VAL_RF_COMBINED_SWEEP_MODE_MANUAL                             1
#define RSSMX_VAL_RF_COMBINED_SWEEP_MODE_STEP                               2

// RsSmx_rngRFCombinedSweepShape
#define RSSMX_VAL_RF_COMBINED_SWEEP_SHAPE_SAWTOOTH                          0
#define RSSMX_VAL_RF_COMBINED_SWEEP_SHAPE_TRIANGLE                          1

// RsSmx_rngPowerAnalysisSource
#define RSSMX_VAL_POWER_ANALYSIS_SOURCE_OFF                                 0
#define RSSMX_VAL_POWER_ANALYSIS_SOURCE_ON                                  1
#define RSSMX_VAL_POWER_ANALYSIS_SOURCE_SEN1                                2
#define RSSMX_VAL_POWER_ANALYSIS_SOURCE_SEN2                                3
#define RSSMX_VAL_POWER_ANALYSIS_SOURCE_SEN3                                4
#define RSSMX_VAL_POWER_ANALYSIS_SOURCE_SEN4                                5
#define RSSMX_VAL_POWER_ANALYSIS_SOURCE_HOLD                                6
#define RSSMX_VAL_POWER_ANALYSIS_SOURCE_REF                                 7

//repcap ExternalTrigger
#define RSSMX_REPCAP_EXTERNALTRIGGER_EXT1                                   1
#define RSSMX_REPCAP_EXTERNALTRIGGER_EXT2                                   2

//repcap Sense
#define RSSMX_REPCAP_SENSE_S1                                               1
#define RSSMX_REPCAP_SENSE_S2                                               2
#define RSSMX_REPCAP_SENSE_S3                                               3
#define RSSMX_REPCAP_SENSE_S4                                               4

//repcap LF
#define RSSMX_REPCAP_LF_LF1                                                 1
#define RSSMX_REPCAP_LF_LF2                                                 2

//repcap Path
#define RSSMX_REPCAP_PATH_PATH1                                             1
#define RSSMX_REPCAP_PATH_PATH2                                             2

//repcap Trace
#define RSSMX_REPCAP_TRACE_TR1                                              1
#define RSSMX_REPCAP_TRACE_TR2                                              2
#define RSSMX_REPCAP_TRACE_TR3                                              3
#define RSSMX_REPCAP_TRACE_TR4                                              4

//repcap Gate
#define RSSMX_REPCAP_GATE_GATE1                                             1
#define RSSMX_REPCAP_GATE_GATE2                                             2

//repcap Marker
#define RSSMX_REPCAP_MARKER_M1                                              1
#define RSSMX_REPCAP_MARKER_M2                                              2
#define RSSMX_REPCAP_MARKER_M3                                              3
#define RSSMX_REPCAP_MARKER_M4                                              4
#define RSSMX_REPCAP_MARKER_M5                                              5
#define RSSMX_REPCAP_MARKER_M6                                              6
#define RSSMX_REPCAP_MARKER_M7                                              7
#define RSSMX_REPCAP_MARKER_M8                                              8
#define RSSMX_REPCAP_MARKER_M9                                              9
#define RSSMX_REPCAP_MARKER_M10                                             10
/* END GENERATE */

/*****************************************************************************
 *- Hidden Attribute Declarations -------------------------------------------*
 *****************************************************************************/

#define RSSMX_ATTR_OPC_TIMEOUT               (RS_ATTR_OPC_TIMEOUT)           /* ViInt32 */
#define RSSMX_ATTR_IO_SESSION                (RS_ATTR_IO_SESSION)            /* ViSession */
#define RSSMX_ATTR_OPC_CALLBACK              (RS_ATTR_OPC_CALLBACK)          /* ViAddr */
#define RSSMX_ATTR_CHECK_STATUS_CALLBACK     (RS_ATTR_CHECK_STATUS_CALLBACK) /* ViAddr */

/****************************************************************************
 *---------------- Constant definition  ------------------------------------*
 ****************************************************************************/
// GG System
#define RSSMX_VAL_SATNAV_SYSTEM_GPS                                    0
#define RSSMX_VAL_SATNAV_SYSTEM_GAL                                    1
#define RSSMX_VAL_SATNAV_SYSTEM_GLON                                   2
#define RSSMX_VAL_SATNAV_SYSTEM_BEID                                   3

// Subsystem
#define RSSMX_VAL_SUBSYSTEM_GSM                                        0
#define RSSMX_VAL_SUBSYSTEM_BTO                                        1
#define RSSMX_VAL_SUBSYSTEM_W3GPP                                      2
#define RSSMX_VAL_SUBSYSTEM_C2K                                        3
#define RSSMX_VAL_SUBSYSTEM_TDSCDMA                                    4
#define RSSMX_VAL_SUBSYSTEM_WLAN                                       5
#define RSSMX_VAL_SUBSYSTEM_WLANN                                      6
#define RSSMX_VAL_SUBSYSTEM_WIMAX                                      7
#define RSSMX_VAL_SUBSYSTEM_DVB                                        8
#define RSSMX_VAL_SUBSYSTEM_DAB                                        9
#define RSSMX_VAL_SUBSYSTEM_DM                                         10
#define RSSMX_VAL_SUBSYSTEM_GPS                                        11
#define RSSMX_VAL_SUBSYSTEM_STEREO                                     12
#define RSSMX_VAL_SUBSYSTEM_XMR                                        13
#define RSSMX_VAL_SUBSYSTEM_SIRIUS                                     14
#define RSSMX_VAL_SUBSYSTEM_EUTRA                                      15
#define RSSMX_VAL_SUBSYSTEM_TETRA                                      16
#define RSSMX_VAL_SUBSYSTEM_FADING                                     17
#define RSSMX_VAL_SUBSYSTEM_EVDO                                       18
#define RSSMX_VAL_SUBSYSTEM_C5G                                        19
#define RSSMX_VAL_SUBSYSTEM_WLANAD                                     20

// File Action
#define RSSMX_VAL_FILE_ACTION_DELETE                                   0
#define RSSMX_VAL_FILE_ACTION_RECALL                                   1
#define RSSMX_VAL_FILE_ACTION_SAVE                                     2

// Frequency Type (up and down)
#define RSSMX_VAL_UP                                                   0
#define RSSMX_VAL_DOWN                                                   1

// Frequency Type (left and right)
#define RSSMX_VAL_LEFT                                                   0
#define RSSMX_VAL_RIGHT                                                   1

//Scaling
#define RSSMX_VAL_LF_FREQUENCY_STEP_SCALING_LINEAR                     0
#define RSSMX_VAL_LF_FREQUENCY_STEP_SCALING_LOGARITHMIC                1

//Sweep Mode
#define RSSMX_VAL_SWEEP_MODE_FREQUENCY_SWEEP                           0
#define RSSMX_VAL_SWEEP_MODE_POWER_SWEEP                               1

#define RSSMX_VAL_FSIM_COOR_RX                                         0
#define RSSMX_VAL_FSIM_COOR_TX                                         1

// SatNav Standard
#define RSSMX_VAL_SATNAV_STANDARD_GPS                                  0
#define RSSMX_VAL_SATNAV_STANDARD_GALILEO                              1
#define RSSMX_VAL_SATNAV_STANDARD_GLONASS                              2
#define RSSMX_VAL_SATNAV_STANDARD_BEIDOU                               3
#define RSSMX_VAL_SATNAV_STANDARD_QZSS                                 4

#define RSSMX_VAL_STAT_REG_PRESET                                      0
#define RSSMX_VAL_STAT_REG_QUESTIONABLE                                1
#define RSSMX_VAL_STAT_REG_OPERATION                                   2

#define RSSMX_VAL_STAT_REG_MAIN                                        0

#define RSSMX_VAL_STAT_REG_QUERY_COND                                  0
#define RSSMX_VAL_STAT_REG_QUERY_EVENT                                 1

#define RSSMX_VAL_SWEEP_MODE_SINGLE                                    1
#define RSSMX_VAL_SWEEP_MODE_EXT_SINGLE                                3
#define RSSMX_VAL_SWEEP_MODE_EXT_STEP                                  4

#define RSSMX_VAL_LIST_TRIGGER_SOURCE_HOP                              3
#define RSSMX_VAL_LIST_TRIGGER_SOURCE_DHOP                             4

// Fast Sweep
#define RSSMX_VAL_FAST_SWEEP_NORMAL                                    0
#define RSSMX_VAL_FAST_SWEEP_FAST                                      1

#define RSSMX_VAL_ON_WITHOUT_WAIT                                      2

#define RSSMX_VAL_ILS_AMPLITUDE_CURRENT     0
#define RSSMX_VAL_ILS_AMPLITUDE_DEPTH       1
#define RSSMX_VAL_ILS_AMPLITUDE_LOGARITHMIC 2
#define RSSMX_VAL_ILS_AMPLITUDE_PERCENT     3

#define RSSMX_VAL_AM_MODE_TYPE_NORM_LIN 0
#define RSSMX_VAL_AM_MODE_TYPE_NORM_EXP 1
#define RSSMX_VAL_AM_MODE_TYPE_SCAN_EXP 2

#define RSSMX_VAL_ANALYSIS_MODE_FREQUENCY                               0
#define RSSMX_VAL_ANALYSIS_MODE_POWER                                   1
#define RSSMX_VAL_ANALYSIS_MODE_TIME                                   2

/****************************************************************************
 *---------------- Instrument Driver Function Declarations -----------------*
 ****************************************************************************/
ViStatus _VI_FUNC rssmx_init(ViRsrc     resourceName,
                             ViBoolean  IDQuery,
                             ViBoolean  resetDevice,
                             ViSession* instrSession);

ViStatus _VI_FUNC rssmx_InitWithOptions(ViRsrc     resourceName,
                                        ViBoolean  IDQuery,
                                        ViBoolean  resetDevice,
                                        ViString   optionString,
                                        ViSession* instrSession);

ViStatus _VI_FUNC rssmx_ConfigureOutputEnabled(ViSession instrSession,
                                               ViBoolean outputEnabled);

ViStatus _VI_FUNC rssmx_ConfigureRF(ViSession instrSession,
                                    ViReal64  frequency,
                                    ViReal64  powerLevel);

ViStatus _VI_FUNC rssmx_ConfigureRFFrequencyMode(ViSession instrSession,
                                                 ViInt32   RFFrequencyMode);

ViStatus _VI_FUNC rssmx_ConfigurePowerMode(ViSession instrSession,
                                           ViInt32   powerMode);

ViStatus _VI_FUNC rssmx_SendSoftwareTrigger(ViSession instrSession);

ViStatus _VI_FUNC rssmx_ConfigureRFFrequency(ViSession instrSession,
                                             ViReal64  frequency);

ViStatus _VI_FUNC rssmx_ConfigureRFFrequencySettings(ViSession instrSession,
                                                     ViInt32   mainPLLBandwidth,
                                                     ViReal64  offset,
                                                     ViReal64  multiplier);

ViStatus _VI_FUNC rssmx_ConfigureRFFrequencyPhaseContinuity(ViSession instrSession,
                                                            ViBoolean enabled,
                                                            ViInt32   mode);

ViStatus _VI_FUNC rssmx_QueryRFFrequencyPhaseContinuityMaxMin(
                            ViSession instrSession,
                            ViReal64* max,
                            ViReal64* min);

ViStatus _VI_FUNC rssmx_ConfigureRFAmplitude(ViSession instrSession,
                                             ViReal64  powerLevel);

ViStatus _VI_FUNC rssmx_ConfigureRFLevel(ViSession instrSession,
                                         ViReal64  RFAmplitude,
                                         ViInt32   mode,
                                         ViReal64  limit,
                                         ViReal64  offset,
                                         ViInt32   settingCharacteristics,
                                         ViInt32   harmonicFilter);

ViStatus _VI_FUNC rssmx_ConfigureRFLevelAttenuator(ViSession instrSession,
                                                   ViInt32   mode,
                                                   ViInt32   RFAttenuatorRFOFFMode);

ViStatus _VI_FUNC rssmx_OvervoltageProtectionTripped(ViSession  instrSession,
                                                     ViBoolean* overvoltageProtectionTripped);

ViStatus _VI_FUNC rssmx_OvervoltageProtectionClear(ViSession instrSession);

ViStatus _VI_FUNC rssmx_ConfigureRFPowerOnState(ViSession instrSession,
                                                ViInt32   powerOnState);

ViStatus _VI_FUNC rssmx_ConfigureRFLevelOnRecall(ViSession instrSession,
                                                 ViInt32   RFLevelOnRecall);

ViStatus _VI_FUNC rssmx_QueryRFLevelUninterruptedRange(ViSession instrSession,
                                                       ViReal64* lowerRange,
                                                       ViReal64* upperRange);

ViStatus _VI_FUNC rssmx_QueryRFLevelRange(ViSession instrSession,
                                          ViReal64* rangeMin,
                                          ViReal64* rangeMax);

ViStatus _VI_FUNC rssmx_ConfigureALCEnabled(ViSession instrSession,
                                            ViInt32   ALCEnabled);

ViStatus _VI_FUNC rssmx_QueryALCMode(ViSession instrSession,
                                     ViInt32*  ALCMode);

ViStatus _VI_FUNC rssmx_ConfigureALCDetectorSensitivity(ViSession instrSession,
                                                        ViInt32   detectorSensitivity);

ViStatus _VI_FUNC rssmx_ALCRecalculate(ViSession instrSession);

ViStatus _VI_FUNC rssmx_ALCReadjust(ViSession instrSession);

ViStatus _VI_FUNC rssmx_ALCLevelControlMode(ViSession instrSession);

ViStatus _VI_FUNC rssmx_ConfigureALCExternalDetector(ViSession instrSession,
                                                     ViReal64  couplingFactor,
                                                     ViReal64  generatorLevel);

ViStatus _VI_FUNC rssmx_ConfigureRFDeltaPhase(ViSession instrSession,
                                              ViReal64  RFDeltaPhase);

ViStatus _VI_FUNC rssmx_ResetRFDeltaPhaseDisplay(ViSession instrSession);

ViStatus _VI_FUNC rssmx_ConfigureAllModulationsState(ViSession instrSession,
                                                     ViBoolean allModulationsEnabled);

ViStatus _VI_FUNC rssmx_ConfigureExternalInput(ViSession instrSession,
                                               ViInt32   external,
                                               ViInt32   coupling,
                                               ViInt32   impedance);

ViStatus _VI_FUNC rssmx_ConfigureNoiseGenerator(ViSession instrSession,
                                                ViInt32   distribution,
                                                ViBoolean bandwidthState,
                                                ViReal64  bandwidth);

ViStatus _VI_FUNC rssmx_ConfigureAMPath(ViSession instrSession,
                                        ViInt32   path,
                                        ViBoolean enabled,
                                        ViInt32   source,
                                        ViReal64  depth);

ViStatus _VI_FUNC rssmx_ConfigureAMSettings(ViSession instrSession,
                                            ViInt32   path,
                                            ViInt32   modeAndType,
                                            ViReal64  depth,
                                            ViReal64  sensitivity);

ViStatus _VI_FUNC rssmx_ConfigureAMDeviationMode(ViSession instrSession,
                                                 ViInt32   deviationMode);

ViStatus _VI_FUNC rssmx_ConfigureAMDeviationRatio(ViSession instrSession,
                                                  ViReal64  deviationRatio);

ViStatus _VI_FUNC rssmx_ConfigureAMDepthTotal(ViSession instrSession,
                                              ViReal64  depthTotal);

ViStatus _VI_FUNC rssmx_ConfigureAMExternalCoupling(ViSession instrSession,
                                                    ViInt32   AMExternalCoupling);

ViStatus _VI_FUNC rssmx_ConfigureFMPath(ViSession instrSession,
                                        ViInt32   path,
                                        ViBoolean enabled,
                                        ViInt32   source,
                                        ViReal64  deviation);

ViStatus _VI_FUNC rssmx_ConfigureFMSettings(ViSession instrSession,
                                            ViReal64  ratio,
                                            ViInt32   mode);

ViStatus _VI_FUNC rssmx_ConfigureFMDeviationMode(ViSession instrSession,
                                                 ViInt32   deviationMode);

ViStatus _VI_FUNC rssmx_ConfigureFMDeviationTotal(ViSession instrSession,
                                                  ViReal64  deviationTotal);

ViStatus _VI_FUNC rssmx_ConfigurePMPath(ViSession instrSession,
                                        ViInt32   path,
                                        ViBoolean enabled,
                                        ViInt32   source,
                                        ViReal64  deviation);

ViStatus _VI_FUNC rssmx_ConfigurePMSettings(ViSession instrSession,
                                            ViReal64  ratio,
                                            ViInt32   mode);

ViStatus _VI_FUNC rssmx_ConfigurePMDeviationMode(ViSession instrSession,
                                                 ViInt32   deviationMode);

ViStatus _VI_FUNC rssmx_ConfigurePMDeviationTotal(ViSession instrSession,
                                                  ViReal64  deviationTotal);

ViStatus _VI_FUNC rssmx_ConfigurePulseModulationEnabled(ViSession instrSession,
                                                        ViBoolean pulseModulationEnabled);

ViStatus _VI_FUNC rssmx_ConfigurePulseModulationSettings(ViSession instrSession,
                                                         ViInt32   source,
                                                         ViInt32   transitionType,
                                                         ViInt32   videoPolarity);

ViStatus _VI_FUNC rssmx_ConfigurePulseModulationExternalTrigger(
                            ViSession instrSession,
                            ViInt32   polarity,
                            ViInt32   impedance,
                            ViReal64  threshold);

ViStatus _VI_FUNC rssmx_ConfigurePulseGeneratorState(ViSession instrSession,
                                                     ViBoolean pulseGeneratorEnabled);

ViStatus _VI_FUNC rssmx_ConfigurePulseGeneratorMode(ViSession instrSession,
                                                    ViInt32   mode);

ViStatus _VI_FUNC rssmx_ConfigurePulseGeneratorPulseParameters(
                            ViSession instrSession,
                            ViReal64  period,
                            ViReal64  width);

ViStatus _VI_FUNC rssmx_ConfigurePulseGeneratorDoublePulseParameters(
                            ViSession instrSession,
                            ViReal64  doubleDelay,
                            ViReal64  doubleWidth);

ViStatus _VI_FUNC rssmx_ConfigurePulseGeneratorDelay(ViSession instrSession,
                                                     ViReal64  delay);

ViStatus _VI_FUNC rssmx_ConfigurePulseGeneratorTriggerMode(ViSession instrSession,
                                                           ViInt32   triggerMode);

ViStatus _VI_FUNC rssmx_TriggerPulseGenerator(ViSession instrSession);

ViStatus _VI_FUNC rssmx_ConfigurePulseGeneratorOutputState(ViSession instrSession,
                                                           ViBoolean pulseOutputState);

ViStatus _VI_FUNC rssmx_ConfigurePulseGeneratorOutputPolarity(
                            ViSession instrSession,
                            ViInt32   pulseOutputPolarity);

ViStatus _VI_FUNC rssmx_QueryPulseModulationTrainCatalog(ViSession instrSession,
                                                         ViUInt32  arraySize,
                                                         ViChar    catalog[],
                                                         ViUInt32* bytesRead);

ViStatus _VI_FUNC rssmx_SelectPulseModulationTrain(ViSession instrSession,
                                                   ViString  fileName);

ViStatus _VI_FUNC rssmx_ConfigurePulseModulationTrainOffTimes(
                            ViSession instrSession,
                            ViUInt32  arraySize,
                            ViReal64  offTimes[]);

ViStatus _VI_FUNC rssmx_ConfigurePulseModulationTrainOnTimes(ViSession instrSession,
                                                             ViUInt32  arraySize,
                                                             ViReal64  onTimes[]);

ViStatus _VI_FUNC rssmx_ConfigurePulseModulationTrainRepetitions(
                            ViSession instrSession,
                            ViUInt32  arraySize,
                            ViReal64  repetitions[]);

ViStatus _VI_FUNC rssmx_DeletePulseModulationTrain(ViSession instrSession,
                                                   ViString  fileName);

ViStatus _VI_FUNC rssmx_QueryPulseTrainDataExchangeCatalog(ViSession instrSession,
                                                           ViUInt32  arraySize,
                                                           ViChar    catalog[],
                                                           ViUInt32* bytesRead);

ViStatus _VI_FUNC rssmx_SelectPulseTrainDataExchangeFile(ViSession instrSession,
                                                         ViString  fileName);

ViStatus _VI_FUNC rssmx_SelectPulseTrainDataExchangeASCIIFile(
                            ViSession instrSession,
                            ViString  ASCIIFile);

ViStatus _VI_FUNC rssmx_ConfigurePulseTrainDataExchangeASCIIFile(
                            ViSession instrSession,
                            ViInt32   ASCIIExtension,
                            ViInt32   columnSeparator,
                            ViInt32   decimalSeparator);

ViStatus _VI_FUNC rssmx_ConfigurePulseTrainDataExchangeImportExport(
                            ViSession instrSession,
                            ViInt32   importExport);

ViStatus _VI_FUNC rssmx_PulseTrainDataExchangeExecute(ViSession instrSession);

ViStatus _VI_FUNC rssmx_ConfigureLFOutputState(ViSession instrSession,
                                               ViBoolean LFOutputEnabled);

ViStatus _VI_FUNC rssmx_ConfigureLFOutputSettings(ViSession instrSession,
                                                  ViInt32   source,
                                                  ViReal64  outputVoltage,
                                                  ViReal64  DCOffset);

ViStatus _VI_FUNC rssmx_ConfigureLFGeneratorShape(ViSession instrSession,
                                                  ViInt32   LFGenerator,
                                                  ViInt32   shape);

ViStatus _VI_FUNC rssmx_ConfigureLFGeneratorVoltage(ViSession instrSession,
                                                    ViInt32   LFGenerator,
                                                    ViReal64  voltage);

ViStatus _VI_FUNC rssmx_ConfigureLFGeneratorSineFrequency(ViSession instrSession,
                                                          ViInt32   LFGenerator,
                                                          ViReal64  frequency);

ViStatus _VI_FUNC rssmx_ConfigureLFGeneratorPulse(ViSession instrSession,
                                                  ViInt32   LFGenerator,
                                                  ViReal64  period,
                                                  ViReal64  width);

ViStatus _VI_FUNC rssmx_ConfigureLFGeneratorTrapezoid(ViSession instrSession,
                                                      ViInt32   LFGenerator,
                                                      ViReal64  period,
                                                      ViReal64  rise,
                                                      ViReal64  fall,
                                                      ViReal64  high);

ViStatus _VI_FUNC rssmx_ConfigureLFGeneratorTriangle(ViSession instrSession,
                                                     ViInt32   LFGenerator,
                                                     ViReal64  period,
                                                     ViReal64  rise);

ViStatus _VI_FUNC rssmx_ConfigureChirpModulationEnabled(ViSession instrSession,
                                                        ViBoolean chirpModulationEnabled);

ViStatus _VI_FUNC rssmx_ConfigureChirpModulationSettings(ViSession instrSession,
                                                         ViInt32   direction,
                                                         ViInt32   triggerMode,
                                                         ViReal64  pulsePeriod,
                                                         ViReal64  pulseWidth,
                                                         ViReal64  bandwidth);

ViStatus _VI_FUNC rssmx_QueryChirpModulationCompressionRatio(ViSession instrSession,
                                                             ViReal64* compressionRatio);

ViStatus _VI_FUNC rssmx_ExecuteChirpModulationSingleTrigger(ViSession instrSession);

ViStatus _VI_FUNC rssmx_ConfigureSweepType(ViSession instrSession,
                                           ViInt32   sweepType);

ViStatus _VI_FUNC rssmx_ResetAllSweeps(ViSession instrSession);

ViStatus _VI_FUNC rssmx_TriggerAllSweeps(ViSession instrSession);

ViStatus _VI_FUNC rssmx_ConfigureSweepTrigger(ViSession instrSession,
                                              ViInt32   source,
                                              ViInt32   slope);

ViStatus _VI_FUNC rssmx_ConfigureRFFrequencySweepState(ViSession instrSession,
                                                       ViBoolean sweepState);

ViStatus _VI_FUNC rssmx_RFFrequencySweepExecute(ViSession instrSession);

ViStatus _VI_FUNC rssmx_TriggerRFFrequencySweep(ViSession instrSession);

ViStatus _VI_FUNC rssmx_QueryRFFrequencySweepBlankPoints(ViSession instrSession,
                                                         ViReal64  blankPoints[],
                                                         ViInt32*  returnedValues);

ViStatus _VI_FUNC rssmx_ConfigureRFFrequencySweepMode(ViSession instrSession,
                                                      ViInt32   sweepMode);

ViStatus _VI_FUNC rssmx_ConfigureRFFrequencySweepTriggerSource(
                            ViSession instrSession,
                            ViInt32   triggerSource);

ViStatus _VI_FUNC rssmx_ConfigureRFFrequencySweepRetrace(ViSession instrSession,
                                                         ViBoolean retrace);

ViStatus _VI_FUNC rssmx_ConfigureRFFrequencySweepManualStep(ViSession instrSession,
                                                            ViReal64  manualStep);

ViStatus _VI_FUNC rssmx_ConfigureRFFrequencySweepDwellTime(ViSession instrSession,
                                                           ViReal64  dwellTime);

ViStatus _VI_FUNC rssmx_ConfigureRFFrequencySweepShape(ViSession instrSession,
                                                       ViInt32   shape);

ViStatus _VI_FUNC rssmx_ConfigureRFFrequencySweepTime(ViSession instrSession,
                                                      ViReal64  frequencySweepTime);

ViStatus _VI_FUNC rssmx_ConfigureRFFrequencySweepStartStop(ViSession instrSession,
                                                           ViReal64  startFrequency,
                                                           ViReal64  stopFrequency);

ViStatus _VI_FUNC rssmx_ConfigureRFFrequencySweepCenterSpan(ViSession instrSession,
                                                            ViReal64  centerFrequency,
                                                            ViReal64  span);

ViStatus _VI_FUNC rssmx_ConfigureRFFrequencySweepPoints(ViSession instrSession,
                                                        ViInt32   sweepPoints);

ViStatus _VI_FUNC rssmx_ConfigureRFFrequencySweepSpacing(ViSession instrSession,
                                                         ViInt32   spacing,
                                                         ViReal64  step);

ViStatus _VI_FUNC rssmx_ConfigureLFFrequencySweepState(ViSession instrSession,
                                                       ViBoolean sweepState);

ViStatus _VI_FUNC rssmx_LFFrequencySweepExecute(ViSession instrSession);

ViStatus _VI_FUNC rssmx_TriggerLFFrequencySweep(ViSession instrSession);

ViStatus _VI_FUNC rssmx_TriggerLFFrequencySweepAuto(ViSession instrSession);

ViStatus _VI_FUNC rssmx_ConfigureLFFrequencyMode(ViSession instrSession,
                                                 ViInt32   LFMode);

ViStatus _VI_FUNC rssmx_ConfigureLFFrequencySweepMode(ViSession instrSession,
                                                      ViInt32   sweepMode);

ViStatus _VI_FUNC rssmx_ConfigureLFFrequencySweepTriggerSource(
                            ViSession instrSession,
                            ViInt32   triggerSource);

ViStatus _VI_FUNC rssmx_ConfigureLFFrequencySweepRetrace(ViSession instrSession,
                                                         ViBoolean retrace);

ViStatus _VI_FUNC rssmx_ConfigureLFFrequencySweepManualStep(ViSession instrSession,
                                                            ViReal64  manualStep);

ViStatus _VI_FUNC rssmx_ConfigureLFFrequencySweepShape(ViSession instrSession,
                                                       ViInt32   shape);

ViStatus _VI_FUNC rssmx_ConfigureLFFrequencySweepDwellTime(ViSession instrSession,
                                                           ViReal64  dwellTime);

ViStatus _VI_FUNC rssmx_ConfigureLFFrequencySweepStartStop(ViSession instrSession,
                                                           ViReal64  startFrequency,
                                                           ViReal64  stopFrequency);

ViStatus _VI_FUNC rssmx_ConfigureLFFrequencySweepPoints(ViSession instrSession,
                                                        ViInt32   sweepPoints);

ViStatus _VI_FUNC rssmx_ConfigureLFFrequencySweepSpacing(ViSession instrSession,
                                                         ViInt32   spacing,
                                                         ViReal64  step);

ViStatus _VI_FUNC rssmx_ConfigureRFLevelSweepState(ViSession instrSession,
                                                   ViBoolean sweepState);

ViStatus _VI_FUNC rssmx_RFLevelSweepExecute(ViSession instrSession);

ViStatus _VI_FUNC rssmx_TriggerRFLevelSweep(ViSession instrSession);

ViStatus _VI_FUNC rssmx_ConfigureRFLevelMode(ViSession instrSession,
                                             ViInt32   levelMode);

ViStatus _VI_FUNC rssmx_ConfigureRFLevelSweepMode(ViSession instrSession,
                                                  ViInt32   sweepMode);

ViStatus _VI_FUNC rssmx_ConfigureRFLevelSweepTriggerSource(ViSession instrSession,
                                                           ViInt32   triggerSource);

ViStatus _VI_FUNC rssmx_ConfigureRFLevelSweepRetrace(ViSession instrSession,
                                                     ViBoolean retrace);

ViStatus _VI_FUNC rssmx_ConfigureRFLevelSweepManualStep(ViSession instrSession,
                                                        ViReal64  manualStep);

ViStatus _VI_FUNC rssmx_ConfigureRFLevelSweepShape(ViSession instrSession,
                                                   ViInt32   shape);

ViStatus _VI_FUNC rssmx_ConfigureRFLevelSweepDwellTime(ViSession instrSession,
                                                       ViReal64  dwellTime);

ViStatus _VI_FUNC rssmx_ConfigureRFLevelSweepStartStop(ViSession instrSession,
                                                       ViReal64  startLevel,
                                                       ViReal64  stopLevel);

ViStatus _VI_FUNC rssmx_ConfigureRFLevelSweepPoints(ViSession instrSession,
                                                    ViInt32   sweepPoints);

ViStatus _VI_FUNC rssmx_ConfigureRFLevelSweepStep(ViSession instrSession,
                                                  ViReal64  step);

ViStatus _VI_FUNC rssmx_ConfigureRFCombinedSweepState(ViSession instrSession,
                                                      ViBoolean RFCombinedSweepState);

ViStatus _VI_FUNC rssmx_ConfigureRFCombinedSweepMode(ViSession instrSession,
                                                     ViInt32   mode);

ViStatus _VI_FUNC rssmx_ConfigureRFCombinedSweepTriggerSource(
                            ViSession instrSession,
                            ViInt32   triggerSource);

ViStatus _VI_FUNC rssmx_ConfigureRFCombinedSweepRetrace(ViSession instrSession,
                                                        ViBoolean retrace);

ViStatus _VI_FUNC rssmx_ExecuteRFCombinedSingleSweep(ViSession instrSession);

ViStatus _VI_FUNC rssmx_ConfigureRFCombinedSweepShape(ViSession instrSession,
                                                      ViInt32   shape);

ViStatus _VI_FUNC rssmx_ConfigureRFCombinedSweepDwellTime(ViSession instrSession,
                                                          ViReal64  dwellTime);

ViStatus _VI_FUNC rssmx_ConfigureRFCombinedSweepFrequencyStartStop(
                            ViSession instrSession,
                            ViReal64  startFrequency,
                            ViReal64  stopFrequency);

ViStatus _VI_FUNC rssmx_ConfigureRFCombinedSweepLevelStartStop(
                            ViSession instrSession,
                            ViReal64  startLevel,
                            ViReal64  stopLevel);

ViStatus _VI_FUNC rssmx_ConfigureRFCombinedSweepStepCount(ViSession instrSession,
                                                          ViInt32   stepCount);

ViStatus _VI_FUNC rssmx_ConfigureListState(ViSession instrSession,
                                           ViBoolean listState);

ViStatus _VI_FUNC rssmx_ConfigureListGeneralSettings(ViSession instrSession,
                                                     ViInt32   listMode,
                                                     ViInt32   dwellTimeMode);

ViStatus _VI_FUNC rssmx_ConfigureListIndex(ViSession instrSession,
                                           ViInt32   listIndex);

ViStatus _VI_FUNC rssmx_ConfigureListGlobalDwellTime(ViSession instrSession,
                                                     ViReal64  dwellTime);

ViStatus _VI_FUNC rssmx_ConfigureDwellTimesList(ViSession instrSession,
                                                ViUInt32  arraySize,
                                                ViReal64  dwellTimes[]);

ViStatus _VI_FUNC rssmx_ConfigureListTriggerSource(ViSession instrSession,
                                                   ViInt32   listTriggerSource);

ViStatus _VI_FUNC rssmx_ListTriggerExecute(ViSession instrSession);

ViStatus _VI_FUNC rssmx_QueryListCatalog(ViSession instrSession,
                                         ViUInt32  arraySize,
                                         ViChar    catalog[],
                                         ViUInt32* bytesRead);

ViStatus _VI_FUNC rssmx_SelectList(ViSession instrSession,
                                   ViString  fileName);

ViStatus _VI_FUNC rssmx_ConfigureListFrequencies(ViSession instrSession,
                                                 ViUInt32  arraySize,
                                                 ViReal64  frequencies[]);

ViStatus _VI_FUNC rssmx_ConfigureListLevels(ViSession instrSession,
                                            ViUInt32  arraySize,
                                            ViReal64  levels[]);

ViStatus _VI_FUNC rssmx_ConfigureListStartStopIndex(ViSession instrSession,
                                                    ViInt32   listStartIndex,
                                                    ViInt32   listStopIndex);

ViStatus _VI_FUNC rssmx_ResetList(ViSession instrSession);

ViStatus _VI_FUNC rssmx_DeleteList(ViSession instrSession,
                                   ViString  fileName);

ViStatus _VI_FUNC rssmx_DeleteAllLists(ViSession instrSession);

ViStatus _VI_FUNC rssmx_QueryListImportExportCatalog(ViSession instrSession,
                                                     ViUInt32  arraySize,
                                                     ViChar    catalog[],
                                                     ViUInt32* bytesRead);

ViStatus _VI_FUNC rssmx_SelectListImportExportFile(ViSession instrSession,
                                                   ViString  listImportExportFile);

ViStatus _VI_FUNC rssmx_SelectListImportExportASCIIFile(ViSession instrSession,
                                                        ViString  listImportExportASCIIFile);

ViStatus _VI_FUNC rssmx_ConfigureListImportExportASCIIFile(ViSession instrSession,
                                                           ViInt32   extension,
                                                           ViInt32   columnSeparator,
                                                           ViInt32   decimalSeparator);

ViStatus _VI_FUNC rssmx_ConfigureListImportExport(ViSession instrSession,
                                                  ViInt32   listImportExport);

ViStatus _VI_FUNC rssmx_ListImportExportExecute(ViSession instrSession);

ViStatus _VI_FUNC rssmx_ConfigureUserCorrectionState(ViSession instrSession,
                                                     ViBoolean userCorrectionDataEnabled);

ViStatus _VI_FUNC rssmx_QueryUserCorrectionCatalog(ViSession instrSession,
                                                   ViUInt32  arraySize,
                                                   ViChar    catalog[],
                                                   ViUInt32* bytesRead);

ViStatus _VI_FUNC rssmx_SelectUserCorrectionTable(ViSession instrSession,
                                                  ViString  fileName);

ViStatus _VI_FUNC rssmx_ConfigureUserCorrectionFrequencies(ViSession instrSession,
                                                           ViUInt32  arraySize,
                                                           ViReal64  frequencies[]);

ViStatus _VI_FUNC rssmx_ConfigureUserCorrectionLevels(ViSession instrSession,
                                                      ViUInt32  arraySize,
                                                      ViReal64  levels[]);

ViStatus _VI_FUNC rssmx_UserCorrectionFillWithSensor(ViSession instrSession,
                                                     ViInt32   sensor,
                                                     ViUInt32  maxTime);

ViStatus _VI_FUNC rssmx_DeleteUserCorrection(ViSession instrSession,
                                             ViString  fileName);

ViStatus _VI_FUNC rssmx_QueryUserCorrectionDataExchangeCatalog(
                            ViSession instrSession,
                            ViUInt32  arraySize,
                            ViChar    catalog[],
                            ViUInt32* bytesRead);

ViStatus _VI_FUNC rssmx_SelectUserCorrectionDataExchangeFile(ViSession instrSession,
                                                             ViString  fileName);

ViStatus _VI_FUNC rssmx_SelectUserCorrectionDataExchangeASCIIFile(
                            ViSession instrSession,
                            ViString  fileName);

ViStatus _VI_FUNC rssmx_ConfigureUserCorrectionDataExchangeFile(
                            ViSession instrSession,
                            ViInt32   extension,
                            ViInt32   columnSeparator,
                            ViInt32   decimalSeparator);

ViStatus _VI_FUNC rssmx_ConfigureUserCorrectionDataExchange(ViSession instrSession,
                                                            ViInt32   userCorrectionDataExchange);

ViStatus _VI_FUNC rssmx_UserCorrectionDataExchangeExecute(ViSession instrSession);

ViStatus _VI_FUNC rssmx_ConfigureVOREnabled(ViSession instrSession,
                                            ViInt32   path,
                                            ViBoolean VOREnabled);

ViStatus _VI_FUNC rssmx_SetVORToDefault(ViSession instrSession,
                                        ViInt32   path);

ViStatus _VI_FUNC rssmx_QueryVORSettingsCatalog(ViSession instrSession,
                                                ViInt32   path,
                                                ViUInt32  arraySize,
                                                ViChar    settingsCatalog[],
                                                ViUInt32* bytesRead);

ViStatus _VI_FUNC rssmx_ManageVORSettings(ViSession instrSession,
                                          ViInt32   path,
                                          ViInt32   action,
                                          ViString  filename);

ViStatus _VI_FUNC rssmx_ConfigureVORCarrierFrequencySettings(ViSession instrSession,
                                                             ViInt32   path,
                                                             ViInt32   carrierFrequencyMode,
                                                             ViReal64  carrierFrequency,
                                                             ViInt32   ICAOChannel,
                                                             ViInt32   EXTAM);

ViStatus _VI_FUNC rssmx_ConfigureVORSignalSettings(ViSession instrSession,
                                                   ViInt32   path,
                                                   ViInt32   operatingMode,
                                                   ViReal64  VARREFFrequency,
                                                   ViReal64  VARDepth,
                                                   ViReal64  subcarrierFrequency,
                                                   ViReal64  subcarrierDepth,
                                                   ViInt32   REFDeviation);

ViStatus _VI_FUNC rssmx_ConfigureVORPosition(ViSession instrSession,
                                             ViInt32   path,
                                             ViReal64  bearingAngle,
                                             ViInt32   direction);

ViStatus _VI_FUNC rssmx_ConfigureVORCOMID(ViSession instrSession,
                                          ViInt32   path,
                                          ViBoolean COMIDState,
                                          ViString  code,
                                          ViReal64  period,
                                          ViReal64  frequency,
                                          ViReal64  depth);

ViStatus _VI_FUNC rssmx_ConfigureVORCOMIDTimeSchema(ViSession instrSession,
                                                    ViInt32   path,
                                                    ViInt32   timeSchema,
                                                    ViReal64  dotLength,
                                                    ViReal64  dashLength,
                                                    ViReal64  symbolSpace,
                                                    ViReal64  letterSpace);

ViStatus _VI_FUNC rssmx_ConfigureILSEnabled(ViSession instrSession,
                                            ViInt32   path,
                                            ViBoolean ILSEnabled);

ViStatus _VI_FUNC rssmx_SetILSToDefault(ViSession instrSession,
                                        ViInt32   path);

ViStatus _VI_FUNC rssmx_QueryILSSettingsCatalog(ViSession instrSession,
                                                ViUInt32  arraySize,
                                                ViChar    settingsCatalog[],
                                                ViUInt32* bytesRead,
                                                ViInt32   path);

ViStatus _VI_FUNC rssmx_ManageILSSettings(ViSession instrSession,
                                          ViInt32   path,
                                          ViInt32   action,
                                          ViString  filename);

ViStatus _VI_FUNC rssmx_ConfigureILSComponent(ViSession instrSession,
                                              ViInt32   path,
                                              ViInt32   component);

ViStatus _VI_FUNC rssmx_SetILSGlideSlopeToDefault(ViSession instrSession,
                                                  ViInt32   path);

ViStatus _VI_FUNC rssmx_ConfigureILSGlideSlopeCarrierFrequencySettings(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   carrierFrequencyMode,
                            ViReal64  carrierFrequency,
                            ViInt32   ICAOChannel,
                            ViInt32   EXTAM);

ViStatus _VI_FUNC rssmx_ConfigureILSGlideSlopeSyncWithLocalizer(
                            ViSession instrSession,
                            ViInt32   path,
                            ViBoolean syncWithLocalizer);

ViStatus _VI_FUNC rssmx_ConfigureILSGlideSlopeSignalSettings(ViSession instrSession,
                                                             ViInt32   path,
                                                             ViInt32   operatingMode,
                                                             ViInt32   DDMPolarity,
                                                             ViInt32   frequencyType,
                                                             ViReal64  frequency,
                                                             ViReal64  upDownPhase);

ViStatus _VI_FUNC rssmx_ConfigureILSGlideSlopeAmplitude(ViSession instrSession,
                                                        ViInt32   path,
                                                        ViInt32   flyMode,
                                                        ViReal64  sumOfDepth,
                                                        ViInt32   DDMCoupling,
                                                        ViInt32   DDMStep);

ViStatus _VI_FUNC rssmx_ConfigureILSGlideSlopeAmplitudeDDMValue(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   DDMType,
                            ViReal64  DDMValue);

ViStatus _VI_FUNC rssmx_SetILSLocalizerToDefault(ViSession instrSession,
                                                 ViInt32   path);

ViStatus _VI_FUNC rssmx_ConfigureILSLocalizerCarrierFrequencySettings(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   carrierFrequencyMode,
                            ViReal64  carrierFrequency,
                            ViInt32   ICAOChannel,
                            ViInt32   EXTAM);

ViStatus _VI_FUNC rssmx_ConfigureILSLocalizerSyncWithGlideSlope(
                            ViSession instrSession,
                            ViInt32   path,
                            ViBoolean syncWithGlideSlope);

ViStatus _VI_FUNC rssmx_ConfigureILSLocalizerSignalSettings(ViSession instrSession,
                                                            ViInt32   path,
                                                            ViInt32   operatingMode,
                                                            ViInt32   DDMPolarity,
                                                            ViInt32   frequencyType,
                                                            ViReal64  frequency,
                                                            ViReal64  leftRightPhase);

ViStatus _VI_FUNC rssmx_ConfigureILSLocalizerAmplitude(ViSession instrSession,
                                                       ViInt32   path,
                                                       ViInt32   flyMode,
                                                       ViReal64  sumOfDepth,
                                                       ViInt32   DDMCoupling,
                                                       ViInt32   DDMStep);

ViStatus _VI_FUNC rssmx_ConfigureILSLocalizerAmplitudeDDMValue(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   DDMType,
                            ViReal64  DDMValue);

ViStatus _VI_FUNC rssmx_ConfigureILSLocalizerCOMID(ViSession instrSession,
                                                   ViInt32   path,
                                                   ViBoolean COMIDState,
                                                   ViString  code,
                                                   ViReal64  period,
                                                   ViReal64  frequency,
                                                   ViReal64  depth);

ViStatus _VI_FUNC rssmx_ConfigureILSLocalizerCOMIDTimeSchema(ViSession instrSession,
                                                             ViInt32   path,
                                                             ViInt32   timeSchema,
                                                             ViReal64  dotLength,
                                                             ViReal64  dashLength,
                                                             ViReal64  symbolSpace,
                                                             ViReal64  letterSpace);

ViStatus _VI_FUNC rssmx_SetILSMarkerBeaconsToDefault(ViSession instrSession,
                                                     ViInt32   path);

ViStatus _VI_FUNC rssmx_ConfigureILSMarkerBeaconsCarrierFrequencySettings(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   carrierFrequencyMode,
                            ViReal64  carrierFrequency);

ViStatus _VI_FUNC rssmx_ConfigureILSMarkerBeaconsSignalSettings(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   markerFrequency,
                            ViReal64  markerDepth,
                            ViBoolean pulsedMarker);

ViStatus _VI_FUNC rssmx_ConfigureILSMarkerBeaconsCOMID(ViSession instrSession,
                                                       ViInt32   path,
                                                       ViBoolean COMIDState,
                                                       ViString  code,
                                                       ViReal64  period,
                                                       ViReal64  frequency,
                                                       ViReal64  depth);

ViStatus _VI_FUNC rssmx_ConfigureILSMarkerBeaconsCOMIDTimeSchema(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   timeSchema,
                            ViReal64  dotLength,
                            ViReal64  dashLength,
                            ViReal64  symbolSpace,
                            ViReal64  letterSpace);

ViStatus _VI_FUNC rssmx_ConfigureADFEnabled(ViSession instrSession,
                                            ViInt32   path,
                                            ViBoolean ADFEnabled);

ViStatus _VI_FUNC rssmx_SetADFToDefault(ViSession instrSession,
                                        ViInt32   path);

ViStatus _VI_FUNC rssmx_QueryADFSettingsCatalog(ViSession instrSession,
                                                ViInt32   path,
                                                ViUInt32  arraySize,
                                                ViChar    settingsCatalog[],
                                                ViUInt32* bytesRead);

ViStatus _VI_FUNC rssmx_ManageADFSettings(ViSession instrSession,
                                          ViInt32   path,
                                          ViInt32   action,
                                          ViString  filename);

ViStatus _VI_FUNC rssmx_ConfigureADFCOMID(ViSession instrSession,
                                          ViInt32   path,
                                          ViBoolean COMIDState,
                                          ViString  code,
                                          ViReal64  period,
                                          ViReal64  frequency,
                                          ViReal64  depth);

ViStatus _VI_FUNC rssmx_ConfigureADFCOMIDTimeSchema(ViSession instrSession,
                                                    ViInt32   path,
                                                    ViInt32   timeSchema,
                                                    ViReal64  dotLength,
                                                    ViReal64  dashLength,
                                                    ViReal64  symbolSpace,
                                                    ViReal64  letterSpace);

ViStatus _VI_FUNC rssmx_ConfigureReferenceOscillatorSource(ViSession instrSession,
                                                           ViInt32   source);

ViStatus _VI_FUNC rssmx_ConfigureReferenceOscillatorRFSwitchOffActive(
                            ViSession instrSession,
                            ViBoolean RFSwitchOffActive);

ViStatus _VI_FUNC rssmx_ConfigureReferenceOscillatorExternalFrequency(
                            ViSession instrSession,
                            ViInt32   externalFrequency,
                            ViReal64  userDefinedExternalFrequency);

ViStatus _VI_FUNC rssmx_ConfigureReferenceOscillatorSynchronizationBandwidth(
                            ViSession instrSession,
                            ViInt32   synchronizationBandwidth);

ViStatus _VI_FUNC rssmx_QueryReferenceOscillatorNominalSynchronizationBandwidth(
                            ViSession instrSession,
                            ViUInt32  arraySize,
                            ViChar    nominalSyncBandwidth[]);

ViStatus _VI_FUNC rssmx_ConfigureReferenceOscillatorEFC(ViSession instrSession,
                                                        ViBoolean enabled,
                                                        ViInt32   voltageTuningSensitivity);

ViStatus _VI_FUNC rssmx_ConfigureReferenceOscillatorOutputFrequencyMode(
                            ViSession instrSession,
                            ViInt32   outputFrequencyMode);

ViStatus _VI_FUNC rssmx_ConfigureReferenceOscillatorOutputReferenceFrequency(
                            ViSession instrSession,
                            ViInt32   outputReferenceFrequency);

ViStatus _VI_FUNC rssmx_ConfigureReferenceOscillatorAdjustmentDAC(
                            ViSession instrSession,
                            ViBoolean active,
                            ViReal64  adjustmentDACFrequency);

ViStatus _VI_FUNC rssmx_QueryReferenceOscillatorMinimumLockingRange(
                            ViSession instrSession,
                            ViUInt32  arraySize,
                            ViChar    minimumLockingRange[]);

ViStatus _VI_FUNC rssmx_ReferenceOscillatorPreset(ViSession instrSession);

ViStatus _VI_FUNC rssmx_ConfigureClockSynthesisState(ViSession instrSession,
                                                     ViBoolean state);

ViStatus _VI_FUNC rssmx_ConfigureClockSynthesisOutputSignal(ViSession instrSession,
                                                            ViInt32   outputType,
                                                            ViReal64  frequency,
                                                            ViReal64  level,
                                                            ViReal64  voltage,
                                                            ViReal64  deltaPhase);

ViStatus _VI_FUNC rssmx_ConfigureClockSynthesisDCOffsetState(ViSession instrSession,
                                                             ViBoolean DCOffsetState,
                                                             ViReal64  clockSynthesisDCOffset);

ViStatus _VI_FUNC rssmx_ClockSynthesisResetDeltaPhaseValue(ViSession instrSession);

ViStatus _VI_FUNC rssmx_ConfigurePowerDisplay(ViSession instrSession,
                                              ViInt32   sensor,
                                              ViBoolean displayPermanent,
                                              ViInt32   display);

ViStatus _VI_FUNC rssmx_ConfigurePowerFrequency(ViSession instrSession,
                                                ViInt32   sensor,
                                                ViInt32   useFrequencyOf,
                                                ViReal64  frequency);

ViStatus _VI_FUNC rssmx_ConfigurePowerLevelOffset(ViSession instrSession,
                                                  ViInt32   sensor,
                                                  ViBoolean offsetEnabled,
                                                  ViReal64  offset);

ViStatus _VI_FUNC rssmx_ConfigurePowerApertureTime(ViSession instrSession,
                                                   ViInt32   sensor,
                                                   ViBoolean defaultApertureTimeEnabled,
                                                   ViReal64  apertureTime);

ViStatus _VI_FUNC rssmx_ConfigurePowerUseSParameter(ViSession instrSession,
                                                    ViInt32   sensor,
                                                    ViBoolean sParameter);

ViStatus _VI_FUNC rssmx_ConfigurePowerSParameterDataSet(ViSession instrSession,
                                                        ViInt32   sensor,
                                                        ViReal64  sParameterDataSet);

ViStatus _VI_FUNC rssmx_QueryPowerSParameterDataSetList(ViSession instrSession,
                                                        ViInt32   sensor,
                                                        ViUInt32  arraySize,
                                                        ViChar    sParameterDataSetList[],
                                                        ViUInt32* bytesRead);

ViStatus _VI_FUNC rssmx_ConfigurePowerLoggingState(ViSession instrSession,
                                                   ViInt32   sensor,
                                                   ViBoolean state);

ViStatus _VI_FUNC rssmx_ConfigurePowerUnit(ViSession instrSession,
                                           ViInt32   sensor,
                                           ViInt32   powerUnit);

ViStatus _VI_FUNC rssmx_PowerContinuous(ViSession instrSession,
                                        ViInt32   sensor,
                                        ViBoolean continuous);

ViStatus _VI_FUNC rssmx_ReadPower(ViSession instrSession,
                                  ViInt32   sensor,
                                  ViReal64* power);

ViStatus _VI_FUNC rssmx_ZeroPower(ViSession instrSession,
                                  ViInt32   sensor);

ViStatus _VI_FUNC rssmx_ConfigurePowerFilterMode(ViSession instrSession,
                                                 ViInt32   sensor,
                                                 ViInt32   powerFilterMode);

ViStatus _VI_FUNC rssmx_ConfigurePowerFilterLength(ViSession instrSession,
                                                   ViInt32   sensor,
                                                   ViInt32   filterLength);

ViStatus _VI_FUNC rssmx_ConfigurePowerFilterFixedNoise(ViSession instrSession,
                                                       ViInt32   sensor,
                                                       ViReal64  noiseSignalRatio,
                                                       ViReal64  noiseSignalRatioTimeout);

ViStatus _VI_FUNC rssmx_PowerFilterSearchOnce(ViSession instrSession,
                                              ViInt32   sensor,
                                              ViUInt32  maxTime);

ViStatus _VI_FUNC rssmx_PowerSensorDevice(ViSession  instrSession,
                                          ViInt32    sensor,
                                          ViBoolean* device);

ViStatus _VI_FUNC rssmx_PowerSensorType(ViSession instrSession,
                                        ViInt32   sensor,
                                        ViUInt32  arraySize,
                                        ViChar    sensorType[]);

ViStatus _VI_FUNC rssmx_PowerSensorVersion(ViSession instrSession,
                                           ViInt32   sensor,
                                           ViUInt32  arraySize,
                                           ViChar    version[]);

ViStatus _VI_FUNC rssmx_PowerSensorSerialNumber(ViSession instrSession,
                                                ViInt32   sensor,
                                                ViUInt32  arraySize,
                                                ViChar    serialNumber[]);

ViStatus _VI_FUNC rssmx_ConfigureSensorMappingScan(ViSession instrSession,
                                                   ViBoolean scanEnabled);

ViStatus _VI_FUNC rssmx_QueryPowerSensorMappingList(ViSession instrSession,
                                                    ViUInt32  arraySize,
                                                    ViChar    mappingList[],
                                                    ViUInt32* bytesRead);

ViStatus _VI_FUNC rssmx_AddPowerSensorMappingUSBSensor(ViSession instrSession,
                                                       ViString  deviceID,
                                                       ViInt32   serial);

ViStatus _VI_FUNC rssmx_AddPowerSensorMappingLANSensor(ViSession instrSession,
                                                       ViString  IPAdress);

ViStatus _VI_FUNC rssmx_RemovePowerSensorMappingLANSensors(ViSession instrSession);

ViStatus _VI_FUNC rssmx_RemovePowerSensorMappingUSBSensors(ViSession instrSession);

ViStatus _VI_FUNC rssmx_RemoveAllPowerSensorMappingSensors(ViSession instrSession);

ViStatus _VI_FUNC rssmx_ConfigurePowerSensorMapping(ViSession instrSession,
                                                    ViInt32   sensorFromList,
                                                    ViInt32   sensorChannel);

ViStatus _VI_FUNC rssmx_PowerAnalysisInitiate(ViSession instrSession);

ViStatus _VI_FUNC rssmx_PowerAnalysisAbort(ViSession instrSession);

ViStatus _VI_FUNC rssmx_StorePowerAnalysisTraceReference(ViSession instrSession,
                                                         ViInt32   trace);

ViStatus _VI_FUNC rssmx_ConfigurePowerAnalysisTrace(ViSession instrSession,
                                                    ViInt32   trace,
                                                    ViInt32   state,
                                                    ViInt32   color,
                                                    ViInt32   source);

ViStatus _VI_FUNC rssmx_DefinePowerAnalysisFrequencyReferencePoints(
                            ViSession instrSession,
                            ViInt64   pointAFrequencyX,
                            ViReal64  pointAPowY,
                            ViInt64   pointBFrequencyX,
                            ViReal64  pointBPowY);

ViStatus _VI_FUNC rssmx_DefinePowerAnalysisPowerReferencePoints(
                            ViSession instrSession,
                            ViReal64  pointAPowerX,
                            ViReal64  pointAPowY,
                            ViReal64  pointBPowerX,
                            ViReal64  pointBPowY);

ViStatus _VI_FUNC rssmx_DefinePowerAnalysisTimeReferencePoints(
                            ViSession instrSession,
                            ViReal64  pointATimeX,
                            ViReal64  pointAPowY,
                            ViReal64  pointBTimeX,
                            ViReal64  pointBPowY);

ViStatus _VI_FUNC rssmx_GeneratePowerAnalysisReference(ViSession instrSession,
                                                       ViInt32   mode);

ViStatus _VI_FUNC rssmx_QueryPowerAnalysisReferencePoints(ViSession instrSession,
                                                          ViInt32   mode,
                                                          ViInt32*  points);

ViStatus _VI_FUNC rssmx_ConfigurePowerAnalysisMath(ViSession instrSession,
                                                   ViInt32   trace,
                                                   ViInt32   mode,
                                                   ViBoolean mathEnabled,
                                                   ViInt32   subtractionOperands);

ViStatus _VI_FUNC rssmx_ConfigurePowerAnalysisMathAxis(ViSession instrSession,
                                                       ViInt32   trace,
                                                       ViInt32   mode,
                                                       ViReal64  xAxis,
                                                       ViReal64  yAxis);

ViStatus _VI_FUNC rssmx_ConfigurePowerAnalysisSource(ViSession instrSession,
                                                     ViInt32   trace,
                                                     ViInt32   mode,
                                                     ViInt32   source);

ViStatus _VI_FUNC rssmx_QueryPowerAnalysisTraceDataPoints(ViSession instrSession,
                                                          ViInt32   trace,
                                                          ViInt32*  dataPoints);

ViStatus _VI_FUNC rssmx_QueryPowerAnalysisTraceDataX(ViSession instrSession,
                                                     ViInt32   trace,
                                                     ViUInt32  arraySize,
                                                     ViReal64  dataXValues[],
                                                     ViInt32*  dataCount);

ViStatus _VI_FUNC rssmx_QueryPowerAnalysisTraceDataY(ViSession instrSession,
                                                     ViInt32   trace,
                                                     ViUInt32  arraySize,
                                                     ViReal64  dataYValues[],
                                                     ViInt32*  dataCount);

ViStatus _VI_FUNC rssmx_QueryPowerAnalysisYValue(ViSession instrSession,
                                                 ViInt32   trace,
                                                 ViReal64  xValue,
                                                 ViReal64* yValue);

ViStatus _VI_FUNC rssmx_ConfigurePowerAnalysisMeasurementMode(
                            ViSession instrSession,
                            ViInt32   mode);

ViStatus _VI_FUNC rssmx_ConfigurePowerAnalysisMeasurementExecution(
                            ViSession instrSession,
                            ViInt32   execution);

ViStatus _VI_FUNC rssmx_ConfigurePowerAnalysisFrequencyMode(ViSession instrSession,
                                                            ViReal64  minimum,
                                                            ViReal64  maximum,
                                                            ViInt32   steps,
                                                            ViInt32   timing,
                                                            ViInt32   spacing,
                                                            ViInt32   execution);

ViStatus _VI_FUNC rssmx_ConfigurePowerAnalysisPowerMode(ViSession instrSession,
                                                        ViReal64  minimum,
                                                        ViReal64  maximum,
                                                        ViInt32   steps,
                                                        ViInt32   timing,
                                                        ViInt32   execution);

ViStatus _VI_FUNC rssmx_ConfigurePowerAnalysisTimeMode(ViSession instrSession,
                                                       ViReal64  minimum,
                                                       ViReal64  maximum,
                                                       ViInt32   steps,
                                                       ViInt32   average,
                                                       ViInt32   execution,
                                                       ViInt32   triggerEvents);

ViStatus _VI_FUNC rssmx_ConfigurePowerAnalysisDiagramGraphicalSettings(
                            ViSession instrSession,
                            ViBoolean gridEnabled,
                            ViInt32   backgroundColor);

ViStatus _VI_FUNC rssmx_ConfigurePowerAnalysisDiagramView(ViSession instrSession,
                                                          ViBoolean standardViewEnabled,
                                                          ViBoolean markerViewEnabled,
                                                          ViBoolean pulseDataViewEnabled,
                                                          ViBoolean fullscreenEnabled);

ViStatus _VI_FUNC rssmx_ConfigurePowerAnalysisDiagramScale(ViSession instrSession,
                                                           ViInt32   mode,
                                                           ViReal64  minLevel,
                                                           ViReal64  maxLevel,
                                                           ViInt32   autoScale);

ViStatus _VI_FUNC rssmx_ResetPowerAnalysisDiagramAutoScale(ViSession instrSession,
                                                           ViInt32   mode);

ViStatus _VI_FUNC rssmx_ConfigurePowerAnalysisGateIndicationEnabled(
                            ViSession instrSession,
                            ViBoolean indicationEnabled);

ViStatus _VI_FUNC rssmx_ConfigurePowerAnalysisGateMode(ViSession instrSession,
                                                       ViInt32   gateTrace,
                                                       ViBoolean enabled,
                                                       ViReal64  gate1Start,
                                                       ViReal64  gate1Stop,
                                                       ViReal64  gate2Start,
                                                       ViReal64  gate2Stop);

ViStatus _VI_FUNC rssmx_QueryPowerAnalysisGateAverage(ViSession instrSession,
                                                      ViInt32   gate,
                                                      ViReal64* average);

ViStatus _VI_FUNC rssmx_QueryPowerAnalysisGateMaximum(ViSession instrSession,
                                                      ViInt32   gate,
                                                      ViReal64* maximum);

ViStatus _VI_FUNC rssmx_ConfigurePowerAnalysisSensorLevelOffset(
                            ViSession instrSession,
                            ViInt32   sensor,
                            ViInt32   mode,
                            ViBoolean offsetEnabled,
                            ViReal64  offset);

ViStatus _VI_FUNC rssmx_ConfigurePowerAnalysisSensorFrequencySeparateFreq(
                            ViSession instrSession,
                            ViInt32   sensor,
                            ViBoolean useSeparateFreq,
                            ViReal64  separateFreqStart,
                            ViReal64  separateFreqStop);

ViStatus _VI_FUNC rssmx_ConfigurePowerAnalysisSensorPowerSeparateFreq(
                            ViSession instrSession,
                            ViInt32   sensor,
                            ViBoolean useSeparateFreq,
                            ViReal64  separateFreq);

ViStatus _VI_FUNC rssmx_ConfigurePowerAnalysisSensorTimeSeparateFreq(
                            ViSession instrSession,
                            ViInt32   sensor,
                            ViBoolean useSeparateFreq,
                            ViReal64  separateFreq);

ViStatus _VI_FUNC rssmx_ConfigurePowerAnalysisPulseDataAllIndicationsEnabled(
                            ViSession instrSession,
                            ViInt32   trace,
                            ViBoolean allIndicationsEnabled);

ViStatus _VI_FUNC rssmx_ConfigurePowerAnalysisPulseData(ViSession instrSession,
                                                        ViInt32   trace,
                                                        ViBoolean enabled,
                                                        ViInt32   base,
                                                        ViReal64  distal,
                                                        ViReal64  mesial,
                                                        ViReal64  proximal);

ViStatus _VI_FUNC rssmx_QueryPowerAnalysisPulseDataCalculated(
                            ViSession instrSession,
                            ViInt32   trace,
                            ViInt32*  baseCalculated,
                            ViReal64* distalCalculated,
                            ViReal64* mesialCalculated,
                            ViReal64* proximalCalculated);

ViStatus _VI_FUNC rssmx_ConfigurePowerAnalysisPulseDataNotifications(
                            ViSession instrSession,
                            ViInt32   trace,
                            ViBoolean dutyCycleEnabled,
                            ViBoolean pulseWidthEnabled,
                            ViBoolean pulsePeriodEnabled,
                            ViBoolean pulseOffTimeEnabled);

ViStatus _VI_FUNC rssmx_QueryPowerAnalysisPulseMeasurements(ViSession  instrSession,
                                                            ViInt32    trace,
                                                            ViReal64*  dutyCycle,
                                                            ViReal64*  pulseWidth,
                                                            ViReal64*  pulsePeriod,
                                                            ViReal64*  pulseOffTime,
                                                            ViBoolean* pulseState);

ViStatus _VI_FUNC rssmx_ConfigurePowerAnalysisTransitionTimesNotifications(
                            ViSession instrSession,
                            ViInt32   trace,
                            ViBoolean transitionRiseTimeEnabled,
                            ViBoolean transitionFallTimeEnabled,
                            ViBoolean transitionStartTimeEnabled,
                            ViBoolean transitionStopTimeEnabled,
                            ViBoolean overshootRisingEdgeEnabled,
                            ViBoolean overshootFallingEdgeEnabled);

ViStatus _VI_FUNC rssmx_QueryPowerAnalysisTransitionTimesMeasurements(
                            ViSession instrSession,
                            ViInt32   trace,
                            ViReal64* riseTime,
                            ViReal64* fallTime,
                            ViReal64* startTime,
                            ViReal64* stopTime,
                            ViReal64* overshootRisingEdge,
                            ViReal64* overshootFallingEdge);

ViStatus _VI_FUNC rssmx_ConfigurePowerAnalysisSignalPowerNotifications(
                            ViSession instrSession,
                            ViInt32   trace,
                            ViBoolean minimalEnabled,
                            ViBoolean peakEnabled,
                            ViBoolean averageEnabled);

ViStatus _VI_FUNC rssmx_QueryPowerAnalysisSignalPowerMeasurements(
                            ViSession instrSession,
                            ViInt32   trace,
                            ViReal64* minimal,
                            ViReal64* peak,
                            ViReal64* average);

ViStatus _VI_FUNC rssmx_ConfigurePowerAnalysisPulsePowerNotifications(
                            ViSession instrSession,
                            ViInt32   trace,
                            ViBoolean topEnabled,
                            ViBoolean baseEnabled,
                            ViBoolean distalEnabled,
                            ViBoolean mesialEnabled,
                            ViBoolean proximalEnabled);

ViStatus _VI_FUNC rssmx_QueryPowerAnalysisPulsePowerMeasurements(
                            ViSession instrSession,
                            ViInt32   trace,
                            ViReal64* top,
                            ViReal64* base,
                            ViReal64* distal,
                            ViReal64* mesial,
                            ViReal64* proximal);

ViStatus _VI_FUNC rssmx_ConfigurePowerAnalysisTrigger(ViSession instrSession,
                                                      ViInt32   sensor,
                                                      ViInt32   mode,
                                                      ViReal64  level,
                                                      ViInt32   slope,
                                                      ViReal64  hysteresis,
                                                      ViReal64  dropOutTime);

ViStatus _VI_FUNC rssmx_PowerAnalysisTriggerAutoSet(ViSession instrSession,
                                                    ViInt32   sensor);

ViStatus _VI_FUNC rssmx_ConfigureScreenshotFileOptions(ViSession instrSession,
                                                       ViString  fileName,
                                                       ViInt32   format,
                                                       ViInt32   device,
                                                       ViInt32   size);

ViStatus _VI_FUNC rssmx_ExecuteScreenshot(ViSession instrSession);

ViStatus _VI_FUNC rssmx_ConfigureScreenshotCSVOptions(ViSession instrSession,
                                                      ViInt32   orientation,
                                                      ViInt32   columnSeparator,
                                                      ViInt32   decimalSeparator,
                                                      ViInt32   headerEnabled);

ViStatus _VI_FUNC rssmx_ReadScreenshotData(ViSession instrSession,
                                           ViUInt32  arraySize,
                                           ViChar    screenshotData[],
                                           ViUInt32* bytesRead);

ViStatus _VI_FUNC rssmx_ConfigureScreenshotAutomaticNamingEnabled(
                            ViSession instrSession,
                            ViBoolean enabled);

ViStatus _VI_FUNC rssmx_ConfigureScreenshotAutomaticNaming(ViSession instrSession,
                                                           ViString  directory,
                                                           ViBoolean prefixEnabled,
                                                           ViString  prefix,
                                                           ViBoolean yearEnabled,
                                                           ViBoolean monthEnabled,
                                                           ViBoolean dayEnabled);

ViStatus _VI_FUNC rssmx_ClearScreenshotAutomaticNamingDirectory(ViSession instrSession);

ViStatus _VI_FUNC rssmx_QueryScreenshotAutomaticNamingDate(ViSession instrSession,
                                                           ViInt32*  year,
                                                           ViInt32*  month,
                                                           ViInt32*  day);

ViStatus _VI_FUNC rssmx_QueryScreenshotAutomaticNamingNumber(ViSession instrSession,
                                                             ViInt32*  number);

ViStatus _VI_FUNC rssmx_QueryScreenshotAutomaticNamingFileName(
                            ViSession instrSession,
                            ViUInt32  bufferSize,
                            ViChar    fileName[]);

ViStatus _VI_FUNC rssmx_readToFile(ViSession instrSession,
                                   ViString  source,
                                   ViString  destination);

ViStatus _VI_FUNC rssmx_writeFromFile(ViSession instrSession,
                                      ViString  source,
                                      ViString  destination);

ViStatus _VI_FUNC rssmx_reset(ViSession instrSession);

ViStatus _VI_FUNC rssmx_self_test(ViSession instrSession,
                                  ViInt16*  selfTestResult,
                                  ViChar    selfTestMessage[]);

ViStatus _VI_FUNC rssmx_revision_query(ViSession instrSession,
                                       ViChar    instrumentDriverRevision[],
                                       ViChar    firmwareRevision[]);

ViStatus _VI_FUNC rssmx_error_query(ViSession instrSession,
                                    ViInt32*  errorCode,
                                    ViChar    errorMessage[]);

ViStatus _VI_FUNC rssmx_SystemStaticError(ViSession instrSession,
                                          ViInt32   bufferSize,
                                          ViChar    buffer[]);

ViStatus _VI_FUNC rssmx_error_message(ViSession instrSession,
                                      ViStatus  errorCode,
                                      ViChar    errorMessage[]);

ViStatus _VI_FUNC rssmx_ConfigureErrorChecking(ViSession instrSession,
                                               ViBoolean optionChecking,
                                               ViBoolean rangeChecking,
                                               ViBoolean statusChecking);

ViStatus _VI_FUNC rssmx_ClearStatus(ViSession instrSession);

ViStatus _VI_FUNC rssmx_IDQueryResponse(ViSession instrSession,
                                        ViChar    IDQueryResponse[]);

ViStatus _VI_FUNC rssmx_ProcessAllPreviousCommands(ViSession instrSession);

ViStatus _VI_FUNC rssmx_QueryOPC(ViSession instrSession,
                                 ViInt32*  OPC);

ViStatus _VI_FUNC rssmx_SetVISATimeout(ViSession instrSession,
                                       ViUInt32  VISATimeout);

ViStatus _VI_FUNC rssmx_GetVISATimeout(ViSession instrSession,
                                       ViUInt32* VISATimeout);

ViStatus _VI_FUNC rssmx_SetOPCTimeout(ViSession instrSession,
                                      ViInt32   OPCTimeout);

ViStatus _VI_FUNC rssmx_GetOPCTimeout(ViSession instrSession,
                                      ViInt32*  timeout);

ViStatus _VI_FUNC rssmx_Calibrate(ViSession instrSession,
                                  ViInt32   timeout,
                                  ViInt32*  result);

ViStatus _VI_FUNC rssmx_QuerySCPIRecordingData(ViSession instrSession,
                                               ViUInt32  arraySize,
                                               ViChar    data[],
                                               ViUInt32* bytesRead);

ViStatus _VI_FUNC rssmx_QueryDeviceFootprint(ViSession instrSession,
                                             ViUInt32  arraySize,
                                             ViChar    deviceFootprint[],
                                             ViUInt32* bytesRead);

ViStatus _VI_FUNC rssmx_ConfigureAutoSystemErrQuery(ViSession instrSession,
                                                    ViBoolean autoSystErrQuery);

ViStatus _VI_FUNC rssmx_ConfigureMultiThreadLocking(ViSession instrSession,
                                                    ViBoolean multiThreadLocking);

ViStatus _VI_FUNC rssmx_ConfigureHardCopyImageSettings(ViSession instrSession,
                                                       ViInt32   graphicFormat,
                                                       ViInt32   region);

ViStatus _VI_FUNC rssmx_HardCopyExecute(ViSession instrSession);

ViStatus _VI_FUNC rssmx_QueryHardCopyImage(ViSession instrSession,
                                           ViUInt32  arraySize,
                                           ViChar    data[],
                                           ViUInt32* bytesRead);

ViStatus _VI_FUNC rssmx_ConfigureHardCopyAutoNaming(ViSession instrSession,
                                                    ViBoolean autoNamingEnabled,
                                                    ViString  autoFileDirectory,
                                                    ViBoolean autoPrefixEnabled,
                                                    ViString  prefix,
                                                    ViBoolean day,
                                                    ViBoolean month,
                                                    ViBoolean year);

ViStatus _VI_FUNC rssmx_ConfigureHardCopyFileName(ViSession instrSession,
                                                  ViString  fileName);

ViStatus _VI_FUNC rssmx_QueryHardCopyGeneratedAutoFileName(ViSession instrSession,
                                                           ViUInt32  arraySize,
                                                           ViChar    generatedAutoFileName[]);

ViStatus _VI_FUNC rssmx_QueryHardCopyAutoFileName(ViSession instrSession,
                                                  ViUInt32  arraySize,
                                                  ViChar    autoFileName[]);

ViStatus _VI_FUNC rssmx_QueryHardCopyCurrentNumber(ViSession instrSession,
                                                   ViInt32*  currentNumber);

ViStatus _VI_FUNC rssmx_ClearHardCopyFile(ViSession instrSession);

ViStatus _VI_FUNC rssmx_QueryMMEMoryCatalog(ViSession instrSession,
                                            ViString  directoryPath,
                                            ViUInt32  arraySize,
                                            ViChar    catalog[],
                                            ViUInt32* bytesRead);

ViStatus _VI_FUNC rssmx_QueryMMEMoryCatalogLength(ViSession instrSession,
                                                  ViString  directoryPath,
                                                  ViInt32*  catalogLength);

ViStatus _VI_FUNC rssmx_QueryMMEMoryDirectoryCatalog(ViSession instrSession,
                                                     ViString  directoryPath,
                                                     ViUInt32  arraySize,
                                                     ViChar    catalog[],
                                                     ViUInt32* bytesRead);

ViStatus _VI_FUNC rssmx_QueryMMEMoryDirectoryCatalogLength(ViSession instrSession,
                                                           ViString  directoryPath,
                                                           ViInt32*  catalogLength);

ViStatus _VI_FUNC rssmx_MMEMoryChangeDirectory(ViSession instrSession,
                                               ViString  directory);

ViStatus _VI_FUNC rssmx_MMEMoryCopy(ViSession instrSession,
                                    ViString  sourcePath,
                                    ViString  destinationPath);

ViStatus _VI_FUNC rssmx_MMEMoryMove(ViSession instrSession,
                                    ViString  fileSource,
                                    ViString  fileDestination);

ViStatus _VI_FUNC rssmx_MMEMoryDelete(ViSession instrSession,
                                      ViString  fileName);

ViStatus _VI_FUNC rssmx_MMEMoryMakeDirectory(ViSession instrSession,
                                             ViString  directory);

ViStatus _VI_FUNC rssmx_MMEMoryRemoveDirectory(ViSession instrSession,
                                               ViString  directory);

ViStatus _VI_FUNC rssmx_WriteMMEMoryBlockData(ViSession instrSession,
                                              ViString  fileName,
                                              ViUInt32  binaryBlockLength,
                                              ViChar    binaryBlockData[]);

ViStatus _VI_FUNC rssmx_ReadMMEMoryBlockData(ViSession instrSession,
                                             ViString  fileName,
                                             ViUInt32  arraySize,
                                             ViChar    binaryBlockData[],
                                             ViUInt32* bytesRead);

ViStatus _VI_FUNC rssmx_RecallImmediate(ViSession instrSession,
                                        ViInt32   recallImmediate);

ViStatus _VI_FUNC rssmx_SaveImmediate(ViSession instrSession,
                                      ViInt32   saveImmediate);

ViStatus _VI_FUNC rssmx_MMEMoryLoadState(ViSession instrSession,
                                         ViInt32   number,
                                         ViString  fileName);

ViStatus _VI_FUNC rssmx_MMEMoryStoreState(ViSession instrSession,
                                          ViInt32   number,
                                          ViString  fileName);

ViStatus _VI_FUNC rssmx_SelectMMEMoryDrive(ViSession instrSession,
                                           ViString  MMEMoryDrive);

ViStatus _VI_FUNC rssmx_QueryFreeMemory(ViSession instrSession,
                                        ViInt32*  totalPhysicalMemory,
                                        ViInt32*  applicationMemory,
                                        ViInt32*  usedHeapMemory,
                                        ViInt32*  availableHeapMemory);

ViStatus _VI_FUNC rssmx_SetStatusRegister(ViSession instrSession,
                                          ViInt32   registerOperation,
                                          ViInt32   questionableRegister,
                                          ViInt32   enable,
                                          ViInt32   PTransition,
                                          ViInt32   NTransition);

ViStatus _VI_FUNC rssmx_GetStatusRegister(ViSession instrSession,
                                          ViInt32   questionableRegister,
                                          ViInt32   registerPart,
                                          ViInt32*  registerValue);

ViStatus _VI_FUNC rssmx_GetError(ViSession instrSession,
                                 ViStatus* code,
                                 ViInt32   bufferSize,
                                 ViChar    description[]);

ViStatus _VI_FUNC rssmx_ClearError(ViSession instrSession);

ViStatus _VI_FUNC rssmx_WriteInstrData(ViSession instrSession,
                                       ViString  writeBuffer);

ViStatus _VI_FUNC rssmx_QueryViBoolean(ViSession  instrSession,
                                       ViString   command,
                                       ViBoolean* value);

ViStatus _VI_FUNC rssmx_QueryViInt32(ViSession instrSession,
                                     ViString  command,
                                     ViInt32*  value);

ViStatus _VI_FUNC rssmx_QueryViReal64(ViSession instrSession,
                                      ViString  command,
                                      ViReal64* value);

ViStatus _VI_FUNC rssmx_QueryViString(ViSession instrSession,
                                      ViString  command,
                                      ViInt32   bufferSize,
                                      ViChar    value[]);

ViStatus _VI_FUNC rssmx_SetAttributeViInt32(ViSession instrSession,
                                            ViString  repeatedCapabilityName,
                                            ViAttr    attributeId,
                                            ViInt32   attributeValue);

ViStatus _VI_FUNC rssmx_SetAttributeViInt64(ViSession instrSession,
                                            ViString  repeatedCapabilityName,
                                            ViAttr    attributeId,
                                            ViInt64   attributeValue);

ViStatus _VI_FUNC rssmx_SetAttributeViReal64(ViSession instrSession,
                                             ViString  repeatedCapabilityName,
                                             ViAttr    attributeId,
                                             ViReal64  attributeValue);

ViStatus _VI_FUNC rssmx_SetAttributeViString(ViSession instrSession,
                                             ViString  repeatedCapabilityName,
                                             ViAttr    attributeId,
                                             ViString  attributeValue);

ViStatus _VI_FUNC rssmx_SetAttributeViBoolean(ViSession instrSession,
                                              ViString  repeatedCapabilityName,
                                              ViAttr    attributeId,
                                              ViBoolean attributeValue);

ViStatus _VI_FUNC rssmx_GetAttributeViInt32(ViSession instrSession,
                                            ViString  repeatedCapabilityName,
                                            ViAttr    attributeId,
                                            ViInt32*  attributeValue);

ViStatus _VI_FUNC rssmx_GetAttributeViInt64(ViSession instrSession,
                                            ViString  repeatedCapabilityName,
                                            ViAttr    attributeId,
                                            ViInt64*  attributeValue);

ViStatus _VI_FUNC rssmx_GetAttributeViReal64(ViSession instrSession,
                                             ViString  repeatedCapabilityName,
                                             ViAttr    attributeId,
                                             ViReal64* attributeValue);

ViStatus _VI_FUNC rssmx_GetAttributeViString(ViSession instrSession,
                                             ViString  repeatedCapabilityName,
                                             ViAttr    attributeId,
                                             ViInt32   bufferSize,
                                             ViChar    attributeValue[]);

ViStatus _VI_FUNC rssmx_GetAttributeViBoolean(ViSession  instrSession,
                                              ViString   repeatedCapabilityName,
                                              ViAttr     attributeId,
                                              ViBoolean* attributeValue);

ViStatus _VI_FUNC rssmx_GetAttributeRepCapName(ViSession instrSession,
                                               ViAttr    attributeId,
                                               ViString  repCapNameID,
                                               ViInt32   index,
                                               ViInt32   bufferSize,
                                               ViChar    repCapName[]);

ViStatus _VI_FUNC rssmx_GetAttributeRepeatedCapabilityIds(ViSession instrSession,
                                                          ViAttr    attributeId,
                                                          ViInt32   bufferSize,
                                                          ViChar    repeatedCapabilityId_s[]);

ViStatus _VI_FUNC rssmx_GetAttributeRepeatedCapabilityIdNames(
                            ViSession instrSession,
                            ViAttr    attributeId,
                            ViString  repeatedCapabilityId,
                            ViInt32   bufferSize,
                            ViChar    repeatedCapabilityIdName_s[]);

ViStatus _VI_FUNC rssmx_close(ViSession instrSession);

/****************************************************************************
 *------------------------ Error And Completion Codes ----------------------*
 ****************************************************************************/

#define RSSMX_WARN_MEASURE_UNCALIBRATED    (RS_CLASS_WARN_BASE + 0x0001L)
#define RSSMX_WARN_OVER_RANGE              (RS_CLASS_WARN_BASE + 0x0002L)

#define RSSMX_ERROR_INCORRECT_STATE        (RS_CLASS_ERROR_BASE + 0x0003L)
#define RSSMX_ERROR_INCORRECT_RESPONSE     (RS_CLASS_ERROR_BASE + 0x0002L)
#define RSSMX_ERROR_MAX_TIME_EXCEEDED      (RS_CLASS_ERROR_BASE + 0x0001L)

#define RSSMX_WARNMSG_MEASURE_UNCALIBRATED "Uncalibrated measurement"
#define RSSMX_WARNMSG_OVER_RANGE           "Measurement overrange"

#define RSSMX_ERRMSG_INCORRECT_STATE     "Instrument in incorrect state"
#define RSSMX_ERRMSG_INCORRECT_RESPONSE      "Response incorrect; report problem to R&S support!"
#define RSSMX_ERRMSG_MAX_TIME_EXCEEDED     "Max Time Exceeded"

#define RSSMX_ERROR_CODES_AND_MSGS \
    {RSSMX_WARN_MEASURE_UNCALIBRATED,  RSSMX_WARNMSG_MEASURE_UNCALIBRATED},\
    {RSSMX_WARN_OVER_RANGE,            RSSMX_WARNMSG_OVER_RANGE},\
    {RSSMX_ERROR_INCORRECT_STATE ,   RSSMX_ERRMSG_INCORRECT_STATE },\
    {RSSMX_ERROR_INCORRECT_RESPONSE,     RSSMX_ERRMSG_INCORRECT_RESPONSE},\
    {RSSMX_ERROR_MAX_TIME_EXCEEDED,    RSSMX_ERRMSG_MAX_TIME_EXCEEDED}

/****************************************************************************
 *---------------------------- End Include File ----------------------------*
 ****************************************************************************/

#if defined(__cplusplus) || defined(__cplusplus__)
}
#endif
#endif /* __RSSMX_HEADER */
